UNIT ADDRMENU;

INTERFACE
USES ADDRIO,ADDRKEY2,CRT,DOS;
 TYPE
  MenuBoxArray = Array[1..12] of ShortStr;

  PROCEDURE ResetBox;
  PROCEDURE SetMenuBox(XM,YM: Integer; Menus: MenuBoxArray; NumPicks: Integer);
  FUNCTION PickMenuBox : Char;

IMPLEMENTATION

CONST
  ColorF1   = Yellow;      { Menu Phrase colors }
  ColorB1   = Blue;
  ColorF2   = LightCyan;   { Foreground Color - First Letter }
  ColorB2   = Blue;
  ColorF3   = Black;       { Reverse Video colors for bar cursor }
  ColorB3   = LightGray;
  ColorF4   = LightGray;     { Border Colors }
  ColorB4   = Black;

VAR
  MenuMsg   : MenuBoxArray;
  MenuLtr   : Array[1..12] of Char;
  NWide     : Integer;
  NPicks    : Integer;
  XPos,YPos : Integer;
  Pick      : Integer;
  LastPick  : Integer;
  SaveAttr  : Byte;
{============================================================================}
PROCEDURE ResetBox;

{ Writes the box menu on the screen }

VAR
    P : Integer;
BEGIN
    SaveAttr := TextAttr;
    SetColor(ColorF4,ColorB4);
    SBorder(XPos,YPos,XPos+NWide+3,YPos+NPicks+1,'');
    For P := 1 to NPicks do Begin
      SetColor(ColorF1,ColorB1);
      WriteSt(' '+MenuMsg[P]+' ',XPos+1,YPos+P);
      SetColor(ColorF2,ColorB2);
      WriteCh(MenuLtr[P],XPos+2,YPos+P);
    End;
    TextAttr := SaveAttr;
END;

{============================================================================}
  PROCEDURE SetMenuBox(XM,YM: Integer; Menus: MenuBoxArray; NumPicks: Integer);

  VAR
    II : Integer;

  BEGIN
     NWide := 1;
     NPicks := NumPicks;
     XPos := XM; YPos := YM;
     Pick := 1;
     LastPick := 1;

     For II := 1 to NPicks do
     Begin
       MenuMsg[II] := Menus[II];
       If Length(MenuMsg[II]) > NWide
          then NWide := Length(MenuMsg[II]);
       MenuLtr[II] := UpCase(MenuMsg[II,1]);
     End;
     For II := 1 to NPicks do MenuMsg[II] := Trim(MenuMsg[II],NWide);
     ResetBox;
  END;

{============================================================================}
  FUNCTION PickMenuBox : Char;

CONST
     UpArrow   = #72;
     Downarrow = #80;
     EnterKey  = #13;
     EscKey    = #27;
     Abort     = #0;
VAR
  Err,II    : Integer;
  Ch        : Char;
  PickExit  : Boolean;
  BeepOn    : Boolean;

BEGIN
  SaveAttr := TextAttr;       { Save the current colors }
  HideCursor;                 { Hide the cursor }
  PickExit := False;
  SetColor(ColorF3,ColorB3);

                             { Write the first pick in reverse }
  WriteSt(' '+MenuMsg[Pick]+' ',XPos+1,YPos+Pick);

  Repeat     { Main keystroke reading loop -- continue until enter or escape }

     Ch := Nextkey;     { Read a key }


  If not FunctKey then Case Ch of     { Handle non function keys here }
    #32..#125: Begin                  { Check characters against 1st letters }
                BeepOn := True;
                 For II := 1 to NPicks do Begin
                   If UpCase(Ch) = MenuLtr[II] then Begin   { got a match }
                      Pick := II;
                      PickExit := True;
                      BeepOn := False;
                   End;
                 End;
                If BeepOn then Beep;
               End;
    EnterKey,EscKey : PickExit := True;  { Get ready to exit }

   End; {case not functkey}

  If FunctKey then  Case Ch of     { Handle function keys here }
       UpArrow   : Pick := Pred(Pick);  { Move up one }
       DownArrow : Pick := Succ(Pick);  { Move down one }
       Else Beep;

     End; {case functkey}


   If Pick > NPicks then Pick := 1;    { Make sure Pick in bounds }
   If Pick < 1 then Pick := NPicks;

   If Pick <> LastPick then Begin
     SetColor(ColorF1,ColorB1);            { Restore last pick }
     WriteSt(' '+MenuMsg[LastPick]+' ',XPos+1,YPos+LastPick);
     SetColor(ColorF2,ColorB2);            { Restore 1st letter last pick }
     WriteCh(MenuLtr[LastPick],XPos+2,YPos+LastPick);

     SetColor(ColorF3,ColorB3);            { Highlight new pick in reverse }
     WriteSt(' '+MenuMsg[Pick]+' ',XPos+1,YPos+Pick);

     LastPick := Pick;
   End;

 Until PickExit;

  If Ch = EscKey then PickMenuBox := Abort    { set function to #0 }
                else PickMenuBox := MenuLtr[Pick];  {set function to letter }
  ShowCursor;
  TextAttr := SaveAttr;   { Restore colors }
 End;


END. {of unit}
