{ BDH - writes header portion of BGI driver }
{ Copyright (c) 1991 Knight Software        }
{$I-}

CONST
  VERSION      : byte = 2;    {Version Number of header}
  REVISION     : byte = 0;    {Revision number of header}
  MIN_VERSION  : byte = 1;    {Minimum Version Number}
  MIN_REVISION : byte = 1;    {Minimum Revision Number}
  HEADER_SIZE  : word = 160;  {Size (in bytes) of header}
  drv_num      : word = 0;    {driver number (not used)}
  Null         : byte = 0;    {Null definition}
CONST
  HEADER_TEXT = 110;          {Max header text length (bytes)}

VAR
  Dname       : String[8];    {Dname is exactly eight char}
  Tname       : String;
  Iname,Oname : String;
  Hname,Stemp : String;
  ISize,OSize : integer;
  HSize       : integer;
  i,Error     : integer;
  Result      : word;
  Ifile,Ofile : file;
  Hfile       : file;
  Fptr        : Pointer;
  Hdata       : array [0..HEADER_TEXT] of char;

{----------------------------------------------------------------}
begin
  if ParamCount < 1 then
  begin
    writeln('BGI Driver Builder - May 5 1991 - Knight Software');
    writeln('Version 1.01 -- Usage is:');
    writeln('  BDH [drv_name] [header.DAT] [input.BIN] [output.BGI]');
    writeln('    [drv_name] is the device name (i.e. EGA or CGA)');
    writeln('    [header.DAT] is the header data file.');
    writeln('    [input.BIN] is the DRIVER.BIN from EXE2BIN.');
    writeln('    [output.BGI] is the DRIVER.BGI file name.');
    Halt(1);
  end;  

  {--------------------------------------------------------------}
  fillchar(Dname,length(Dname),0);         {Clear Dname to zeros}
  Dname := ParamStr(1);                    {Driver name from cmdline}
  for i := 1 to length(Dname) do           {Convert name to uppercase}
    Dname[i] := UpCase(Dname[i]);

  {--------------------------------------------------------------}
  Tname := Dname;
  if ParamCount > 1 then
    Tname := ParamStr(2);
  if pos('.',Tname) <> 0 then
    Tname := copy(Tname,1,pred(pos('.',Tname)));
  Hname := Tname+'.DAT';
  Assign(Hfile, Hname);
  Reset(Hfile, 1);                         {Open header file}
  Error := IOresult;
  if Error <> 0 then                       {Did the open suceed?}
  begin
    writeln( 'ERROR: Could not open header file ',Hname );
    Halt(2);                               {Leave the program}
  end;
  HSize := FileSize(Hfile);                {Get length of the file}
  if HSize > HEADER_TEXT then              {Limit to max size}
    HSize := HEADER_TEXT;
  reset(Hfile, 1);                         {Reset the header file}
  blockread(Hfile, Hdata, HSize, Result);  {Read source byte}
  Close(Hfile);			                       {Close header file}
  Error := IOresult;
  if (Error <> 0) or (Result <> HSize) then
  begin
    Writeln(Error, 'Error reading header file ',Hname);
    Halt(4);
  end;

  {--------------------------------------------------------------}
  Iname := Tname;
  if ParamCount > 2 then
    Iname := ParamStr(3);
  if pos('.',Iname) <> 0 then
    Iname := copy(Iname,1,pred(pos('.',Iname)));
  Tname := Iname;
  Iname := Iname+'.BIN';
  Assign(Ifile, Iname);
  Reset(Ifile, 1);                         {Open input file}
  Error := IOresult;
  if Error <> 0 then                       {Did the open suceed?}
  begin
    writeln( 'ERROR: Could not open input file ',Iname );
    Halt(2);                               {Leave the program}
  end;
  ISize := FileSize(Ifile);                {Get length of the file}
  reset(Ifile, 1);                         {Reset the input file}
  GetMem(Fptr, ISize);
  blockread(Ifile, Fptr^, ISize, Result);  {Read source byte}
  Close(Ifile);			                       {Close input file}
  Error := IOresult;
  if (Error <> 0) or (Result <> ISize) then
  begin
    Writeln(Error, 'Error reading input file ',Iname);
    Halt(4);
  end;

  {--------------------------------------------------------------}
  Oname := Tname;
  if ParamCount > 3 then
    Oname := ParamStr(4);
  if pos('.',Oname) <> 0 then
    Oname := copy(Oname,1,pred(pos('.',Oname)));
  Tname := Oname;
  Oname := Oname+'.BGI';
  Assign(Ofile, Oname);
  Rewrite(Ofile, 1);                       {Open output file}
  Error := IOresult;
  if Error <> 0 then                       {Did the open suceed?}
  begin
    writeln( 'ERROR: Could not open output file ',Oname );
    Halt(3);                               {Leave the program}
  end;

  {--------------------------------------------------------------}
  Stemp := 'pk'+#8+#8;                     {set file mark}
  for i := 0 to pred(HSize) do             {add in header data}
    if not ((HData[i] = #$1A) or (HData[i] = #0)) then
      Stemp := Stemp+HData[i];
  Stemp := Stemp+#0+#$1A;                  {tag it with null, ^Z} 
  blockwrite(Ofile, Stemp[1], length(Stemp)); {write heading info}

  blockwrite(Ofile, word(HEADER_SIZE), 2); {Write size of header}
  blockwrite(Ofile, word(drv_num), 2);     {Write driver number}
  blockwrite(Ofile, integer(ISize), 2);    {Size (in bytes) of driver}

  blockwrite(Ofile, byte(VERSION), 1);     {Write the version number}
  blockwrite(Ofile, byte(REVISION), 1);    {Write the revision number}

  blockwrite(Ofile, byte(MIN_VERSION), 1); {Write the version number}
  blockwrite(Ofile, byte(MIN_REVISION), 1);{Write the revision number}

  OSize := FilePos(Ofile);                 {Find loc in output file}
  for i := OSize to $7F do
    blockwrite(Ofile, byte(null), 1);      {Finish title area with 0s}

  blockwrite(Ofile, word(HEADER_SIZE), 2); {Write size of header}
  blockwrite(Ofile, word(drv_num), 2);     {Write driver number}
  blockwrite(Ofile, integer(ISize), 2);    {Size (in bytes) of driver}

  blockwrite(Ofile, byte(VERSION), 1);     {Write the version number}
  blockwrite(Ofile, byte(REVISION), 1);    {Write the revision number}

  blockwrite(Ofile, byte(MIN_VERSION), 1); {Write the version number}
  blockwrite(Ofile, byte(MIN_REVISION), 1);{Write the revision number}

  blockwrite(Ofile, DName, Length(Dname)+1);{Write device name to file}

  OSize := FilePos(Ofile)+1;               {How big is header so far}
  for i := OSize to HEADER_SIZE do
    blockwrite(Ofile, byte(Null), 1);      {Pad header with zeros}

  blockwrite(Ofile, Fptr^, ISize, Result); {Write the BIN file data}
  FreeMem(Fptr, ISize);                    {Release heap memory}
  Close(Ofile);                            {Close output file}
end.
