/////////////////////////////////////////////////////////////////////////////
//
// betedit.h: C++ derived edit controls for numbers
//
// Copyright (C) 1992 Cambridge Electronic Design
// All rights reserved.
//

#ifndef _BETEDIT_H_
#define _BETEDIT_H_

#ifndef RC_INVOKED
#ifndef __AFXWIN_H__
#include <afxwin.h>
#endif
#endif


#define BEM_VALIDATE WM_USER+400                // Specific to these controls
#define BEM_NOTVALID WM_USER+401         // so we don't need to register them

#define BEN_ILLEGALCHAR 0x8000     // above the range for normal EN_ messages
                                 // sent to parent when illegal character hit
                                  // return 0 if you want parsed edit to beep


/////////////////////////////////////////////////////////////////////////////
//
// CLongEdit is a specialized CEdit control that is designed to edit a
// long (32 bit) integer.

class CLongEdit : public CEdit
{
private:
    long        m_lValue;                           // The current data value
    long        m_lMax;
    long        m_lMin;

public:
// Constructor without params
	CLongEdit();

// Destructor
	~CLongEdit();

// explicit constructor
	BOOL Create(DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID);

// subclassed constructor for overlaying existing dialog CEdit
	BOOL SubclassEdit(UINT nID, CWnd* pParent, LONG lMin, LONG lMax);

// message handler for character validation
	afx_msg void OnChar(UINT nChar, UINT nRepCnt, UINT nFlags);

// message handler for loss-of-focus behavior
    afx_msg void OnKillFocus(CWnd* pWnd);

// message handler for self-check request
    afx_msg LONG OnValidate(UINT, LONG);

// update the displayed text to show the value
    void RefreshText();

// set limits to allowed data values
	BOOL SetRange(LONG lMin, LONG lMax);

// Validate the data, return TRUE if OK
	BOOL Validate(BOOL bShow);

// Set current value - with limit check
    void CLongEdit::SetValue(LONG lVal);

// Get current value
    LONG CLongEdit::GetValue();

// for WNDCLASS Registered window - not implemented as yet
//	static BOOL RegisterControlClass();

// Overridables
	virtual void OnBadInput();

	DECLARE_MESSAGE_MAP()
};


/////////////////////////////////////////////////////////////////////////////
//
// CRealEdit is a specialized CEdit control that is designed to edit a
// real (float) number. Much the same as CLongEdit.

class CRealEdit : public CEdit
{
private:
    float       m_fValue;                           // The current data value
    float       m_fMax;
    float       m_fMin;

public:
// Constructor without params
	CRealEdit();

// Destructor
	~CRealEdit();

// explicit constructor
	BOOL Create(DWORD dwStyle, const RECT& rect,
		CWnd* pParentWnd, UINT nID);

// subclassed constructor for overlaying existing dialog CEdit
	BOOL SubclassEdit(UINT nID, CWnd* pParent, float fMin, float fMax);

// message handler for character validation
	afx_msg void OnChar(UINT nChar, UINT nRepCnt, UINT nFlags);

// message handler for loss-of-focus
    afx_msg void OnKillFocus(CWnd* pWnd);

// message handler for self validation
    afx_msg LONG OnValidate(UINT, LONG);

// update the displayed text to show the value
    void RefreshText();

// set limits to allowed data values
	BOOL SetRange(float fMin, float fMax);

// Validate the data, return TRUE if OK
	BOOL Validate(BOOL bShow);

// Set current value - with limit check
    void CRealEdit::SetValue(float lVal);

// Get current value
    float CRealEdit::GetValue();

// for WNDCLASS Registered window - not implemented as yet
//	static BOOL RegisterControlClass();

// Overridables
	virtual void OnBadInput();

	DECLARE_MESSAGE_MAP()
};

#endif
