
;BGI256.ASM
;COPYRIGHT (c) 1991  by Knight Software
;P.O. Box 22902, Milwaukie, OR 97222
;BGI driver for Super VGA 256 color modes
;As of 17 May 1991

;**********************************************************************
;The following macro builds the required BGI header block.
;This macro is derived from Borland's own BGI toolkit
;from the DEVICE.INC file for compatibilty.
BGI	macro	name		; the following macro builds
	public	name		; the required ddo header block
name	proc	far		; the vector table must be at
				; 'ddovec' if vector not speced
	push	ds		; (push ds, push cs are signature)
	push	cs		; make ds=cs
	pop	ds
	cld			; just in case
	push	bp		; 6 bytes
	call	[DDOVEC+si]	; call the local function
	pop	bp		
	pop	ds		; restore regs
	ret			; ret far
name	endp
	db	'CB'		; just for fun
	dw	?
EMULATE:ret			; emulate function (nop until
	dw	0,0		; patched by loader)
RESERVED:
NONE:	ret			; near ret for unused functions
	endm

;**********************************************************************

CODE	SEGMENT PARA PUBLIC 'CODE'
	ASSUME  DS:CODE, CS:CODE

	BGI	BGI256		; Define the interface type

	INCLUDE	SDATA.INC	; Include all data / variables
	INCLUDE	SPAL.INC	; Include default palette data
	INCLUDE SHARD.INC	; Include init/bank subroutines
	INCLUDE	SPIXEL.INC	; Include pixel subroutines
	INCLUDE SLINE.INC	; Include line subroutines
	INCLUDE SBIT.INC	; Incluce bitmap subroutines
	INCLUDE	SBLT.INC	; Include blt subroutines
	INCLUDE STEXT.INC	; Include text subroutines
	INCLUDE	SFLOOD.INC	; Include floodfill subroutines

;hardware specific code - select one appropriate include file
	INCLUDE V7INIT.INC	; Include V7 init subroutines
;@@	INCLUDE SVESA.INC	; Include VESA init subroutines

;--- INSTALL ----------------------------------------------------------
;Driver initialization and installation
;The Install function is used to prepare the driver for use. 
;
;The calls to this function allow the kernal to inquire the 
;mode information, and allow the kernal to install the mode 
;infomation.                                                
;Entry:   AL = 00	CL = Mode number
;Return:  ES:BX = Pointer to Device Status Table
;
;Entry:	  AL = 01	CL = N/A
;Return:  CX = Number of modes supported (last mode+1)
;
;Entry:	  AL = 02	CL = Mode number
;Return:  ES:BX = Pointer to Mode name (Pascal type String)
;Destory: None

_install PROC	NEAR
	PUSH	SI
	PUSH	DI
	PUSH	AX
	PUSH	DX
	CMP	AL,0
	JNZ	@Install1 	;Install device command 
	CMP	CL,MaxModes	;if greater than Max # modes
	JC	@installa	;then force default mode
	MOV	CL,DefaultMode	;else use selected value
@installa:
	MOV	[ModeSelect],CL	;save requested mode
	PUSH	CS
	POP	ES		;Point ES:BX to status block
	LEA	BX,StatBlock
	CALL	DetectCard	;determine what card we have
	JNZ	@InstallDone	;unknown display
	CALL	MiscInit	;initalize misc variables
	JMP	@InstallDone

@Install1:	; Mode Query Command
	CMP	AL,1
	JNZ	@Install2
	CMP	BYTE PTR [ModeXflag],0
	JZ	@Install1a	 ;if mode switch set
	MOV	CH,0
	MOV	CL,[ModeSelect]  ;return current mode instead
	INC	CX
	MOV	BYTE PTR [ModeXflag],0 ;then clr the flag
	JMP	@InstallDone
@Install1a:
	MOV	CX,[NumberModes]  ;return number of modes available
	JMP	@InstallDone

@Install2:	; Mode Name Command
	CMP	AL,2
	MOV	AH,grError	 ;Bad cmd
	JNZ	@Installx
	CALL	GetModeName
	MOV	AH,grInvalidMode ;Bad mode number
	JNC	@Installx
	JMP	@InstallDone

@Installx:	; Bad Install Exit
	MOV	[StatBlock.stat],AH	;save error
@Installdone:
	POP	DX
	POP	AX
	POP	DI
	POP	SI
	RET
_install ENDP


;--- INIT -------------------------------------------------------------
;Initialize device for output 
;Entry:   ES:BX points to Device Information Table 
;                        (not Device Status Table)
;Return:  N/A
;Destory: None

_init	PROC	NEAR
	PUSH	AX
	PUSH	CX
	MOV	CH,ES:[BX]	;get InitColor
	MOV	CL,ES:[BX+1]	;Get Init flag
	CMP	CL,65H		;if 65H don't init
	JZ	@InitExit
	MOV	[InitColor],CH	  ;save init color to use
	CALL	[InitDisplayProc] ;call the selected init code
	JZ	@InitExit
	MOV	AH,grInvalidMode  ;bad mode
	MOV	[StatBlock.stat],AH
@InitExit:
	POP	CX
	POP	AX
	RET
_init	ENDP


;--- CLEAR ------------------------------------------------------------
;Clear graphics device (screen) and ready it for new output. 
;Entry:   N/A
;Return:  N/A
;Destroy: None

_clear	PROC	NEAR
	PUSH	SI
	PUSH	DI
	PUSH	DX
	PUSH	CX
	PUSH	BX
	PUSH	AX
	MOV	CX,0		      ;Start clear at 0,0
	MOV	DX,0
	MOV	AX,[StatBlock.BytesPerScanLine] ;Clear full width of screen
	MOV	BX,[StatBlock.TotalScanLines]   ;Clear full height of screen
	LEA	SI,FillPattern	      ;DS:SI = start of cpu memory 
	LEA	DI,WriteClear         ;point DI at proc to use
	CALL	DoBitMap	      ;Copy cpu memory to screen
	POP	AX
	POP	BX
	POP	CX
	POP	DX
	POP	DI
	POP	SI
	RET
	RET
_clear	ENDP


;--- POST -------------------------------------------------------------
;Post screen. This is required for devices that need to be posted.
;Since the IBM displays are always visible, there is no need to
;do anything here.
;Entry:   N/A
;Return:  N/A
;Destory: None

_post	PROC	NEAR
	RET
_post	ENDP


;--- MOVE -------------------------------------------------------------
;Set the master Current Pointer to coordinates passed in AX, BX.
;Entry:   AX = X,  BX = Y 
;Return:  N/A
;Destory: None

_move   PROC  NEAR
	MOV	[CPX],AX
	MOV	[CPY],BX
	RET
_move	ENDP


;--- DRAW -------------------------------------------------------------
;Draw a line from the master Current Pointer to the 
;coordinates passed in AX,BX.
;Entry:    AX = X,  BX = Y 
;Return:   N/A
;Destroys: None

_draw	PROC	NEAR
	PUSH	CX
	PUSH	DX
	MOV	CX,[CPX]
	MOV	DX,[CPY]
	CALL	PlotLine
	POP	DX
	POP	CX
	RET
_draw	ENDP


;--- VECT -------------------------------------------------------------
;Draw a line between the coordinates passed in AX, BX, CX, DX.
;Entry:    AX = X Start,  BX = Y Start
;          CX = X End,    DX = Y End
;Return:   N/A
;Destroys: None

_vect	PROC	NEAR
	CALL	PlotLine
	RET
_vect	ENDP


; @EM --- BAR ---------------------------------------------------------
;Draw a filled rectangle with the CP as the Lower Left corner, 
;and the coordinate in AX, BX as the upper right.
;Entry:    AX = X upper corner  BX = Y upper corner
;          CX = Depth for 3D bars
;          DX = Draw Top Flag (DX<>0 = Draw Top)
;Return:   N/A
;Destroys: None

;_bar  PROC  NEAR                   	 ; @@@@@@                    
;	RET                         	 ; this function is emulated 
;_bar	ENDP                        	 ; @@@@@@                    


;--- PATBAR -----------------------------------------------------------
;Draw a patterned box at the passed coordinates in AX, BX, CX, DX. 
;The pattern is provided by the information previously passed to 
;the set fill pattern function.
;Entry:    AX = X1 Corner  BX = Y1 Corner
;          CX = X2 Corner  DX = Y2 Corner
;Return:   N/A
;Destroys: None

_patbar	PROC	NEAR
	PUSH	SI
	PUSH	DI
	PUSH	DX
	PUSH	CX
	PUSH	BX
	PUSH	AX
	CMP	AX,CX		;insure that start X is in CX
	JNC	@PatBar1
	XCHG	AX,CX
@PatBar1:
	CMP	BX,DX		;insure that start Y is in DX
	JNC	@PatBar2
	XCHG	BX,DX
@PatBar2:
	SUB	AX,CX		;put X size in AX
	INC	AX
	SUB	BX,DX		;put Y size in BX
	INC	BX
	LEA	SI,FillPattern	 ;DS:SI = start of cpu memory 
	LEA	DI,WriteFillLine ;point DI at proc to use
	CALL	DoBitMap	 ;Copy cpu memory to screen
	POP	AX
	POP	BX
	POP	CX
	POP	DX
	POP	DI
	POP	SI
	RET
_patbar	ENDP


; @EM --- ARC ---------------------------------------------------------
;Draw an elliptical arc from the start angle in AX, 
;to the end angle in BX, using the the CP as the Center Point, 
;and the X and Y Radii in CX, DX. Angles are 0-360 degrees.
;Entry:   AX = Start Angle  BX = End Angle
;         CX = X radius of arc
;         DX = Y radius of arc
;Return:  N/A
;Destory: None

;_arc  PROC  NEAR                        ; @@@@@@                    
;	RET                              ; this function is emulated 
;_arc	ENDP                             ; @@@@@@                    


; @EM --- PIESLICE ----------------------------------------------------
;Draw an elliptical sector from the start angle in AX,
;to the end angle in BX, using the CP as the Center Point, 
;and the X and Y Radii in CX, DX. Angles are 0-360 degrees.
;Entry:   AX = Start Angle  BX = End Angle
;         CX = X radius of arc
;         DX = Y radius of arc
;Return:  N/A
;Destory: None

;_pieslice  PROC  NEAR                   ; @@@@@@                    
;	RET                              ; this function is emulated 
;_pieslice ENDP                          ; @@@@@@                    


; @EM --- FILLED ELLIPSE ----------------------------------------------
;Draw an ellipse using the CP as the Center Point, 
;and the X and Y radii passed in AX, BX.
;Entry:   AX = X Radius of ellipse
;         BX = Y Radius of ellipse
;Return:  N/A
;Destory: None

;_filled_ellipse  PROC  NEAR             ; @@@@@@                    
;	RET                              ; this function is emulated 
;_filled_ellipse ENDP                    ; @@@@@@                    


;--- PALETTE ---------------------------------------------------------- 
;Set a specific palette entry to a given color.
;Entry:
; Top two bits of AX contain command code.
; 00 : set EGA color - Index in AL, Color in BL
; 01 : not used 
; 10 : set specific VGA palette - BX=red, CX=green, DX=blue, AX=index
; 11 : set EGA background color - BL=Color
;Return:  N/A
;Destory: None

_palette PROC	NEAR
	AND	AH,0C0H
	JZ	@Pal0	;set color value
	CMP	AH,80H
	JZ	@Pal2	;set dac color register
	CMP	AH,40H	;this function not used
	JZ	@PalX
	MOV	AL,0	;set background color
@Pal0:	MOV	BH,BL	;set color palette
	MOV	BL,AL
	MOV	AX,1000H
	INT	10H
	JMP	@PalX

@Pal2:	MOV	CH,CL	;set an individual DAC color register
	MOV	CL,DL	;BX=red, CX=green, DX=blue, AX=index
	MOV	DH,BL	;top two bits of AX = 10
	MOV	DL,0
	MOV	BX,AX
	AND	BX,00FFH
	MOV	AX,1010H
	INT	10H
@PalX:	RET
_palette ENDP


;--- ALL PALETTE ------------------------------------------------------
;Load an EGA palette with a table of colors.
;Entry:   ES:BX = EGA Palette Table to load
;Return:  N/A
;Destory: None

_allpalette PROC  NEAR
	MOV	DX,BX
	MOV	AX,1002H
	INT	10H
	RET
_allpalette ENDP


;--- COLOR ------------------------------------------------------------
;Set the palette indexs to the specified drawing color in AL
;and the specified fill color in AH.
;Entry:   AL = Drawing Color
;         AH = Fill color
;Return:  N/A
;Destroy: None

_color  PROC  NEAR
	MOV	[DrawForeColor],AL
	MOV	[FillForeColor],AH
	RET
_color	ENDP


;--- FILL STYLE -------------------------------------------------------
;Set the fill pattern to the specified pattern number passed in AL.
;If the pattern number is 0FFH, the pattern is define in
;a user supplied array pointed at by ES:BX.
;Entry:   AL = Pattern # (0FFH for user def)
;         ES:BX = User pattern data if needed
;Return:  N/A
;Destroy: None

_fillstyle PROC	NEAR
	PUSH	ES
	PUSH	DI
	PUSH	CX
	PUSH	BX
	PUSH	AX
	MOV	CX,4		 ;Copy four words (eight bytes)
	LEA	DI,FillPattern   ;copy pattern to Fill Pattern array
	CMP	AL,0FFH
	JZ	@UserFill
	PUSH	DS
	POP	ES
	MOV	AH,0	;Adjust for table index
	SHL	AX,1	;Pattern number times eight
	SHL	AX,1
	SHL	AX,1
	LEA	BX,FillTable
	ADD	BX,AX	;ES:BX points into table
@UserFill:
	MOV	AX,ES:[BX] ;Copy pattern to drawing array
	MOV	[DI],AX
	INC	BX
	INC	DI
	INC	BX
	INC	DI
	LOOP	@UserFill
	POP	AX
	POP	BX
	POP	CX
	POP	DI
	POP	ES
	RET
_fillstyle ENDP


;--- LINE STYLE -------------------------------------------------------
;Set the line style to the specified style number passed in AL.
;If the style number is > 3  then the style is define in the user 
;supplied pattern in BX. The line width to use is passed in CX. 
;Note: The calling Graph unit actually does the triple width
;      by drawing the line three times, so don't do it in here. 
;Entry:   AL = Style # ( >3 for user def)
;         BX = User Line Pattern
;         CX = Line Width (1 or 3)
;Return:  N/A
;Destory: None

_linestyle PROC	NEAR
	PUSH	DI
	PUSH	BX
	PUSH	AX
	MOV	[LineWidth],CL
	CMP	AL,4
	JNC	@LineStyleX
	AND	AX,03H
	ADD	AX,AX
	LEA	DI,LinePatternTable
	ADD	DI,AX
	MOV	BX,[DI]
@LineStyleX:
	MOV	[LinePattern],BX
	POP	AX
	POP	BX
	POP	DI
	RET
_linestyle ENDP


;--- TEXT STYLE -------------------------------------------------------
;Set the text attributes for font rendering
;Entry:   AL = Font Number
;         AH = Font Path and Direction
;         BX = Desired X Character Size
;         CX = Desired Y Character Size
;Return:  BX = Actual X Character Size
;         CX = Actual Y Character Size
;Destroy: None

_textstyle PROC NEAR
	PUSH	CX
	PUSH	BX
	PUSH	AX
	MOV	[FontNumber],AL
	MOV	[FontDir],AH
	SHR	CL,1
	SHR	CL,1
	SHR	CL,1
	JNZ	@TextStyle1
	INC	CL
@TextStyle1:
	CMP	CL,10
	JL	@TextStyle2
	MOV	CL,10
@TextStyle2:
	MOV	[FontMultX],CL

	SHR	BL,1
	SHR	BL,1
	SHR	BL,1
	JNZ	@TextStyle3
	INC	BL
@TextStyle3:
	CMP	BL,10
	JL	@TextStyle4
	MOV	BL,10
@TextStyle4:
	MOV	[FontMultY],BL

	MOV	AL,[CharSizeX]
	MUL	BL
	MOV	BL,AL
	MOV	BH,0
	MOV	[FontSizeX],BL

	MOV	AL,[CharSizeY]
	MUL	CL
	MOV	CL,AL
	MOV	CH,0
	MOV	[FontSizeY],CL
	POP	AX
	POP	BX
	POP	CX
	RET
_textstyle ENDP


;--- TEXT -------------------------------------------------------------
;Draw a string in the current font with the
;justification point at the CP.
;Entry:   ES:BX = Pointer to string
;         CX    = String Length
;Return:  N/A
;Destroy: None

_drawtext	PROC	NEAR
	CALL	DrawText
	RET
_drawtext	ENDP


;--- TEXT SIZE --------------------------------------------------------
;Calculate the dimensions (in pixels) of an input text string.
;Entry:   ES:BX = Pointer to string
;         CX    = String Length
;Return:  BX    = Width of string
;         CX    = Height of string
;Destroy: None

_textsize PROC	NEAR
	PUSH	AX
	MOV	AH,0
	MOV	AL,[FontSizeX]
	MUL	CX
	MOV	BX,AX
	MOV	CH,0
	MOV	CL,[FontSizeY]
	POP	AX
	RET
_textsize ENDP

;--- FLOOD FILL -------------------------------------------------------
;Do a floodfill in the current color using the specified X,Y address 
;in AX, BX as the seed point.
;Entry:   AX = Seed X, BX = Seed Y
;Return:  N/A
;Destroy: None

_floodfill PROC NEAR
;	RET

	PUSH	BP
	PUSH	DI
	PUSH	SI
	PUSH	DX
	PUSH	CX
	PUSH	BX
	PUSH	AX

	MOV	[PixelX],AX
	MOV	[PixelY],BX
	CALL	GetPixelAddress
	CALL	ReadPixel
	MOV	DL,[PixelColor]
	MOV	[SeedColor],DL
	CMP	DL,[FillForeColor]
	JZ	FloodFillExit

	PUSH	AX
	PUSH	AX
	PUSH	AX
	PUSH	AX
	MOV	DX,BX
	MOV	BX,AX
	MOV	BP,SP
	SUB	BP,2
	CALL	DoFloodLine
	POP	AX
	POP	AX
	POP	AX
	POP	AX

	PUSH	BX
	PUSH	CX
	PUSH	BX
	PUSH	CX
	MOV	DI,1
	DEC	DX
	CALL	ReFloodFill

	MOV	DI,-1
	ADD	DX,2
	CALL	ReFloodFill
	POP	CX
	POP	BX
	POP	CX
	POP	BX

FloodFillExit:
	POP	AX
	POP	BX
	POP	CX
	POP	DX
	POP	SI
	POP	DI
	POP	BP
	RET
_floodfill ENDP


;--- GET PIXEL --------------------------------------------------------
;Read a pixel from the coordinates specified in AX, BX.
;Entry:   AX = X, BX = Y 
;Return:  DL = Pixel value read
;Destroy: None

_getpixel PROC NEAR
	MOV	[PixelX],AX
	MOV	[PixelY],BX
	CALL	GetPixelAddress
	CALL	ReadPixel
	MOV	DL,[PixelColor]
	RET
_getpixel ENDP


;--- SET PIXEL --------------------------------------------------------
;Write a pixel to the coordinate specified in AX, BX.
;Entry:   AX = X, BX = Y 
;         DL = Pixel value to write
;Return:  N/A
;Destory: None

_setpixel PROC NEAR
	MOV	[PixelX],AX
	MOV	[PixelY],BX
	MOV	[PixelColor],DL
	CALL	GetPixelAddress
	CALL	DrawPixel
	RET
_setpixel ENDP


;--- BIT MAP UTILITIES ------------------------------------------------
;Return a pointer to a table of misc bit map driver utilities. 
;Entry:   N/A
;Return:  ES:BX = Base of table
;Destory: None
;The table is configured as follows:
;BitMapTable:  	DW GOTOGRAPHIC
;       	DW EXITGRAPHIC
;       	DW PUTPIX
;       	DW GETPIX
;       	DW BITSPERPIXEL
;       	DW SETPAGE
;       	DW SETVISUAL
;       	DW SETWRITEMODE

_bitmaputil PROC NEAR
	PUSH	CS
	POP	ES
	LEA	BX,BitMapTable
	RET
_bitmaputil ENDP


;--- SAVE BIT MAP -----------------------------------------------------
;Save a portion of the screen to CPU memory.
;Entry:   ES:BX = Pointer to CPU memory buffer
;         CX    = Start X coord of block to save
;         DX    = Start Y coord of block to save
;         ES:[BX]   = Width of block to save
;         ES:[BX+2] = Height of block to save
;Return:  Data in CPU memory buffer starting at ES:[BX+4]
;Destory: None

_savebitmap PROC NEAR
	PUSH	SI
	PUSH	DI
	PUSH	BX
	PUSH	AX
	MOV	SI,BX
	MOV	AX,ES:[SI]	;[SI]= X size (width)
	INC	AX		;Save in AX
	MOV	BX,ES:[SI+2]	;[SI+2]= Y size (height)
	INC	BX		;Save in BX
	ADD	SI,4		;ES:SI = start of cpu memory 
	LEA	DI,ReadBitMap   ;point DI at proc to use
	CALL	DoBitMap	;Copy screen to cpu memory
	POP	AX
	POP	BX
	POP	DI
	POP	SI
	RET
_savebitmap ENDP


;--- RESTORE BIT MAP --------------------------------------------------
;Restore a portion of the screen from CPU memory.
;Entry:   ES:BX = Pointer to CPU memory buffer
;         CX    = Start X coordinate of area to restore
;         DX    = Start Y coordinate of area to restore
;         ES:[BX]   = Width of area to restore
;         ES:[BX+2] = Height of area to restore
;         ES:[BX+4] = Data to restore to screen
;         AL    = Write mode to use for restoring the area
;		 (see SetWriteMode procedure for modes allowed)
;Return:  N/A
;Destory: None

_restorebitmap PROC NEAR
	PUSH	SI
	PUSH	DI
	PUSH	BX
	PUSH	AX
	AND	AL,1FH		;strip off any garbage in AL
	MOV	[BitMapPixelWriteMode],AL ;save the mode select
	MOV	AH,0		;index into proc pointer table
	ADD	AX,AX		;to get the proc address
	LEA	SI,DrawModeTable
	ADD	SI,AX
	MOV	DI,[SI]		;get selected proc addr into DI
	LEA	SI,BitMapPixelProc ;save selected draw mode procedure 
	MOV	[SI],DI		;address in the procedure pointer

	MOV	SI,BX
	MOV	AX,ES:[SI]	;[SI]= X size (width)
	INC	AX		;Save in AX
	MOV	BX,ES:[SI+2]	;[SI+2]= Y size (height)
	INC	BX		;Save in BX
	ADD	SI,4		;ES:SI = start of cpu memory 
	LEA	DI,WriteBitMap  ;point DI at proc to use
	CALL	DoBitMap	;Copy cpu memory to screen
	POP	AX
	POP	BX
	POP	DI
	POP	SI
	RET
_restorebitmap	ENDP


;--- SET CLIP ---------------------------------------------------------
;Set the clipping window to the rectangle defined by the 
;the coordinates passed in AX, BX as the upper left corner
;and CX, DX as the lower left corner.
;Note: Clipping per the BGI defintion is actually done in the
;calling graph unit. There's no real need to clip in the BGI driver.
;Entry:   AX = Upper Left X 
;         BX = Upper Left Y 
;         CX = Lower Right X 
;         DX = Lower Right Y 
;Return:  N/A
;Destory: None

_setclip PROC NEAR
	MOV	[ClipX1],AX
	MOV	[ClipY1],BX
	MOV	[ClipX2],CX
	MOV	[ClipY2],DX
	RET
_setclip ENDP


;--- COLOR QUERY ------------------------------------------------------
;Return the color parameters of the device
;Entry:   AL = Command for query
;Return: 
;  if AL = 0 : BX = Total colors available
;              CX = Maximum color value (BX minus one)
;  if AL = 1 : ES:BX pointer to EGA palette 
;Destory: None
 
_color_query PROC NEAR
	CMP	AL,0
	JZ	@Query0	;get max palette
	CMP	AL,1
	JZ	@Query1	;get default palette
	JMP	SHORT @QueryX	;error

@Query0:
	MOV	BH,0
	MOV	BL,[StatBlock.ctblf]
	INC	BX
	MOV	CX,BX
	DEC	CX
	JMP	SHORT @QueryX

@Query1:
	PUSH	CS
	POP	ES
	LEA	BX,EgaPalette
@QueryX:
	RET
_color_query ENDP


;----------------------------------------------------------------------
;35 DUP (NONE)    ; Reserved Entry Points


;**********************************************************************
;WARNING! The data segment (DS) is not correctly setup when BMI code
;is called. Neither is ES. All registers probably should be saved.

;--- BMI ENTER BIT MAP MODE -------------------------------------------
;switch to pixel graphics mode
;Entry:   N/A
;Return:  N/A
;Destory: None

bmi_gotographic	PROC FAR
	RET		; do nothing
bmi_gotographic	ENDP


;--- BMI EXIT BIT MAP MODE --------------------------------------------
;return to normal graphics mode
;Entry:   N/A
;Return:  N/A
;Destory: None

bmi_exitgraphic	PROC FAR
	RET		; do nothing
bmi_exitgraphic	ENDP


;--- BMI SET PIXEL ----------------------------------------------------
;Fast version of Set Pixel. 
;Entry:   AX = X, BX = Y 
;         DL = Pixel value to write
;Return:  N/A
;Destory: None

bmi_putpixel	PROC FAR
	PUSH	DS
	PUSH	CS
	POP	DS
	CALL	_setpixel   ;See _setpixel for more information
	POP	DS 
	RET		    ;since we only have one setpixel version
bmi_putpixel	ENDP


;--- BMI GET PIXEL ----------------------------------------------------
;Fast version of Get Pixel. 
;Entry:   AX = X, BX = Y 
;Return:  DL = Pixel value read
;Destory: None

bmi_getpixel	PROC FAR
	PUSH	DS
	PUSH	CS
	POP	DS
	CALL	_getpixel   ;See _getpixel for more information
	POP	DS 
	RET		    ;since we only have one getpixel version
bmi_getpixel	ENDP


;--- BMI BITS PER PIXEL -----------------------------------------------
;Return how many bits are in a pixel
;Entry:   N/A
;Return:  AX = number of bits used to define a pixel
;Destory: None

bmi_bitsperpixel PROC FAR
	PUSH	DS
	PUSH	CS
	POP	DS
	MOV	AL,[StatBlock.PixelBits]
	MOV	AH,0
	POP	DS
	RET
bmi_bitsperpixel ENDP


;--- BMI SET PAGE -----------------------------------------------------
;Draw the page passed in AL
;Entry:   AL = Page to draw
;Return:  N/A
;Destory: None

bmi_setpage	PROC FAR
	RET			;no draw page selection being done
bmi_setpage	ENDP


;--- BMI SET VISUAL PAGE ----------------------------------------------
;Select the visual display page passed AL
;Entry:   AL = Display page to select
;Return:  N/A
;Destory: None

bmi_setvisual	PROC FAR
	RET			;no visual page selection being done
bmi_setvisual	ENDP


;--- BMI SET WRITE MODE -----------------------------------------------
;Set pixel write mode as passed in AL for the procedure group specified
;Entry:  AL = Bits 0-4 = write mode.  
;             Bits 6-7 = procedure select.
;             Bit    5 = not used.
;
;  0= MOVE write       8= FORE MOVE write      16= BACK MOVE write     
;  1= XOR write        9= FORE XOR write       17= BACK XOR write      
;  2= OR write        10= FORE OR write        18= BACK OR write      
;  3= AND write       11= FORE AND write       19= BACK AND write     
;  4= NOT MOVE write  12= FORE NOT MOVE write  20= BACK NOT MOVE write
;  5= NOT XOR write   13= FORE NOT XOR write   21= BACK NOT XOR write 
;  6= NOT OR write    14= FORE NOT OR write    22= BACK NOT OR write  
;  7= NOT AND write   15= FORE NOT AND write   23= BACK NOT AND write 
;
;  24 = Set Background color to currently selected foreground color.
;  25 = Return current selected mode in next GetMaxMode call.
;  26 -> 31 = not used.
;
;  Bits 6 and 7
;  00 = 0:(00) Line procedures
;  01 = 1:(40) Fill procedures
;  10 = 2:(80) Text procedures
;  11 = 3:(C0) BitMap procedures
;
;Return:  N/A
;Destory: None

bmi_setwritemode PROC FAR
	PUSH	DS
	PUSH	DX
	PUSH	CX
	PUSH	BX
	PUSH	AX
	PUSH	CS
	POP	DS
	MOV	CX,AX		;stuff procedure selection in CL
	AND	CL,0C0H		;strip off draw mode select
	AND	AL,1FH		;strip off proc sel info in AL
	CMP	AL,24
	JL	@Bmiswp		;Go set proc mode
	JZ	@Bmiswb		;Do Background color selection
	CMP	AL,25
	JNZ	@Bmiswx		;Unknown function
	MOV	[ModeXflag],AL	;Set mode switch flag
	JMP	@Bmiswx

@Bmiswb:
	MOV	AH,[DrawForeColor] ;function 24, set background
	LEA	BX,DrawBackColor
	CMP	CL,00H
	JZ	@Bmiswc
	LEA	BX,FillBackColor
	CMP	CL,40H
	JZ	@Bmiswc
	LEA	BX,TextBackColor
	CMP	CL,80H
	JZ	@Bmiswc
	LEA	BX,BitMapBackColor
@Bmiswc:
	MOV	[BX],AH
	JMP	@Bmiswx

@Bmiswp:
	MOV	CH,AL 		;save the mode select
	MOV	AH,0		;index into proc pointer table
	ADD	AX,AX		;to get the proc address
	LEA	BX,DrawModeTable
	ADD	BX,AX
	MOV	DX,[BX]		;get selected proc addr into DX
	LEA	AX,LinePixelWriteMode
	LEA	BX,LinePixelProc
	CMP	CL,0		;set for line procs?
	JZ	@Bmiswm
	LEA	AX,FillPixelWriteMode
	LEA	BX,FillPixelProc
	CMP	CL,40H		;set for fill procs?
	JZ	@Bmiswm
	LEA	AX,TextPixelWriteMode
	LEA	BX,TextPixelProc
	CMP	CL,80H		;set for text procs?
	JZ	@Bmiswm
	LEA	AX,BitMapPixelWriteMode
	LEA	BX,BitMapPixelProc ;must be BitMap mode
@Bmiswm:
	MOV	[BX],DX	;save selected draw mode procedure address
	MOV	BX,AX	;in the selected group procedure pointer
	MOV	[BX],CH ;save mode select number
@Bmiswx:
	POP	AX
	POP	BX
	POP	CX
	POP	DX
	POP	DS
	RET
bmi_setwritemode ENDP

;----------------------------------------------------------------------

CODE	ENDS
	END
