

dgroup        GROUP  data

              SUBTTL Convert Days Since 01/01/80 to MM/DD/yyYY$
COMMENT *=======================================================================

NAME: ConvJD

PURPOSE: Converts the integer days since 01/01/80 to MM/DD/YY string format

DECLARATION: DECLARE SUB ConvJD (JD%, MMDDYY$)

USAGE: JD% is the integer days since 01/01/80
       MMDDYY$ is a string, at least 8 characters long, where the result will
       be placed

===============================================================================*

Date_Text     SEGMENT PUBLIC WORD 'code'
              ASSUME cs:Date_Text, ds:dgroup

data          SEGMENT PUBLIC WORD 'data'
DaysPerMonth  dw     31,28,31,30,31,30,31,31,30,31,30,31 ;number of days in each month
data          ENDS

;parameters
StringDes     EQU    WORD PTR [bp+6]
JD            EQU    WORD PTR [bp+8]

;local data
Year          EQU    WORD PTR [bp-2]
DayOfYear     EQU    WORD PTR [bp-4]
StringLen     EQU    WORD PTR [bp-6]

Feb           EQU    WORD PTR dgroup:[DaysPerMonth + 2]
BaseYear      EQU    80

              PUBLIC ConvJD
ConvJD        PROC   FAR
              push   bp
              mov    bp, sp
              sub    sp, 6
              push   di
              push   si
              mov    bx,StringDes
              mov    cx,[bx]        ;cx = length of string
              mov    di,[bx+2]      ;get the address
              cmp    cx,8
              jae    CJD_00                ;exit if there isn't room for the result

              jmp    CJD_Error_Exit

CJD_00:       mov    ax,JD
              cmp    ax,8000h
              jb     CJD_01

              jmp CJD_Error_Exit          ;jump if we received a negative number

;calculate INT(JD / 365.25)
CJD_01:       sub    cx,2
              mov    StringLen,cx   ;StringAddr + StringLen = -> to Year$
              mov    bx,100
              mul    bx             ;dx:ax = JD * 100
              mov    cx,36525
              div    cx             ;ax = (JD * 100) / 36525
                                    ;ax = INT(JD / 365.25) = Year
              mov    Year,ax        ;save it
;calculate DayOfYear = INT(JD - Year * 365.25) + 1
              mul    cx             ;dx:ax = (Year * 365.25) * 100
              push   dx
              push   ax
              mov    ax,JD
              mul    bx      ;dx:ax = JD * 100
              pop    bx
              pop    cx      ;cx:bx = (Year * 365.25) * 100
              sub    ax,bx
              sbb    dx,cx   ;dx:ax = (JD * 100) - (Year * 365.35) * 100
              mov    cx,100
              div    cx             ;ax = INT(JD - Year * 365.25)
              inc    ax
              mov    DayOfYear,ax

;leap year?
              mov    ax,Year
              ror    ax,1
              ror    ax,1           ;ah = Year MOD 4 shr 6
              mov    Feb,28         ;default to 28 days
              or     ah,0
              jnz    ConvJD_02                 ;jump if it's not a leap year

              mov    Feb,29
ConvJD_02:    mov    si, OFFSET dgroup:DaysPerMonth
              xor    cx,cx

;determine the month
ConvJD_04:    lodsw                         ;ax = # of days in a month
              inc    cx             ;cx = Month
              cmp    DayOfYear,ax
              jbe    ConvJD_06                 ;jump if we found the month

              sub    DayOfYear,ax
              cmp    cx,12          ;put a limit on the number of time we can loop
              jbe    ConvJD_04

;Year = Year - 80
;cx = Month
;DayOfYear = DayOfMonth

ConvJD_06:    mov    ax,cx          ;ax = Month
              call   MakeAscii

              mov    WORD PTR [di],ax  ;move Month$
              mov    ax,DayOfYear   ;ax = Day
              call   MakeAscii

              mov    WORD PTR [di+3],ax  ;move Day$
              mov    ax,Year
              add    ax,BaseYear
              cmp    ax,100
              jb     ConvJD_08      ;jump if 19xx

              sub    ax,100         ;01/01/2000 or later
ConvJD_08:
              call   MakeAscii

              add    di,StringLen
              mov    WORD PTR [di],ax  ;move Year$
              jmp    SHORT CJD_Exit

CJD_Error_Exit:
;if invalid parameters then di = -> to the string, cx = length
              or     cx,cx
              jz     CJD_Exit                   ;jump if the string has no length

              push   es
              push   ds
              pop    es             ;es:di = -> to string
              xor    al,al
              rep    stosb                     ;string = 0
              pop    ds

CJD_Exit:
              pop    si
              pop    di
              mov    sp,bp
              pop    bp
              ret    4

MakeAscii     PROC
;ax = the hex value of a month, day or year
              mov    cl,10
              div    cl             ;al = tens
                                            ;ah = ones
              or     ax,3030h       ;ax = 2 ascii digits
              ret

MakeAscii     ENDP

ConvJD        ENDP

              SUBTTL Make Julian Date (Days since 01/01/80)
COMMENT *=======================================================================

NAME: MakeJD

PURPOSE: Converts a Date in the form of MM/DD/yyYY to an integer value
         expressed as days since 01/01/80

DECLARATION: DECLARE FUNCTION MakeJD% (MMDDYY$)

USAGE: MMDDYY$ must be in dgroup
               Charaters 0 & 1 must be the month
               Characters 3 & 4 must be the day
               The last 2 characters must be the year

DATE: 10/05/88

===============================================================================*

data          SEGMENT PUBLIC WORD 'data'
XtraDays      db             0,3,3,6,8,11,13,16,19,21,24,26
data          ENDS

Descriptor    EQU    WORD PTR [bp+6]
Month         EQU    WORD PTR [bp-2]
Day           EQU    WORD PTR [bp-4]
Year          EQU    WORD PTR [bp-6]

BaseYear      EQU    80

              PUBLIC MakeJD
MakeJD        PROC   FAR
              push   bp
              mov    bp,sp
              sub    sp,6
              push   di
              push   si

              mov    bx,Descriptor  ;bx = -> to QB string descriptor
              mov    cx,[bx]        ;cx = length of string
              cmp    cx,8
              jae    MakeJD_00
              jmp    Error_Exit  ;return 0 if the string is < 8 characters long

;break it up into MM, DD, YY
;Month
MakeJD_00:    mov    si,[bx+2]      ;si = -> to string
              mov    ax,[si]        ;ax = Month$
              call   ConvPartDate

              cmp    ax,1           ;return 0 if 1 > Month > 12
              jae    MakeJD_01
              jmp    Error_Exit

MakeJD_01:    cmp    ax,12
              ja Error_Exit

              mov    Month,ax       ;save it
;Day
              mov    ax,[si+3]      ;ax = Day$
              call   ConvPartDate

              cmp    ax,1           ;return if 1 > Day > 31
              jb     Error_Exit
              cmp    ax,31
              ja     Error_Exit
              mov    Day,ax

;year
              add    si,cx          ;si = -> end of string
              sub    si,2           ;si = -> YY
              lodsw                 ;ax = Year$
              call   ConvPartDate

              sub    ax,BaseYear    ;ax = Year - BaseYear
              jns    MakeJD_014                ;jump if Year < 0 (date is 01/01/2000 or later)

              add    ax,BaseYear + 20
MakeJD_014:   mov    Year,ax
              mov    dx,365
              mul    dx             ;ax= Year * 365
              push   ax             ;save it
              mov    ax,Year
              dec    ax             ;ax = Year - 1
              shr    ax,1
              shr    ax,1           ;ax = INT(Year - 1) / 4
              cmp    ax,BaseYear
              jb     MakeJD_017
              mov    ax,0ffffh      ;must be the base year
MakeJD_017:   push   ax             ;save it

;calculate (Month - 1) * 28
              mov    ax,Month
              mov    cx,ax          ;cx = Month
              dec    ax             ;ax = Month - 1
              mov    dx,28
              mul    dl             ;ax = (Month - 1) * 28
              push   ax             ;save it

;get total # of days over 28 per month
              mov    ax,cx
              dec    ax             ;al = Month - 1
              mov    bx,offset dgroup:XtraDays
              xlat                  ;al = extra days
              push   ax             ;save it

;add a day if the date is after February and it's a leap year
              mov    ax,cx
              xor    bx,bx          ;bx = False = Leap Day
              cmp    ax,2
              jbe    MakeJD_06                 ;jump if Month <= 2

MakeJD_02:    mov    ax,Year
              ror    ax,1
              ror    ax,1           ;ah = (Year MOD 4) shl 6

              or     ah,ah
              jnz    MakeJD_06                 ;jump if it's not a leap year

              inc    bx

;add it all together
MakeJD_06:
              pop    ax             ;ax = extra days over 28
              add    ax,bx          ;add the leap day
              pop    bx             ;bx = (Month - 1) * 28
              add    ax,bx
              pop    bx             ;bx = INT(Year - 1) / 4
              add    ax,bx
              pop    bx             ;bx = Year * 365
              add    ax,bx
              add    ax,Day         ;done
              jmp    SHORT MakeJD_Exit

Error_Exit:   xor    ax,ax
MakeJD_Exit:  pop    si
              pop    di
              mov    sp,bp
              pop    bp
              ret    2

ConvPartDate  PROC
              and    ax,0f0fh       ;strip the ascii
              mov    bl,ah          ;save the ones
              xor    ah,ah          ;strip the ones
              mov    dx,10
              mul    dl             ;ax = Tens * 10
              add    al,bl
              ret

ConvPartDate  ENDP

MakeJD        ENDP

Date_Text     ENDS

              END
