{ MPI - Multiple Precision Positive Integer Arithmetic Package   }
{ Copyright 1989, KSU Research Foundation, Manhattan KS  66506   }
{ License granted to copy, but not for sale or profit   }
 {test program in Turbo Pascal to interface to multiple precision positive}
 {integer arithmetic package and implement Knuth's (p. 261) suggested severe}
 {test for correctness of the division implementation}
 {$L mpiface}
 program divtest;
    const regmax = 9;
    type
      c4 = packed array[1..4] of char;
      register = array[0..regmax] of word;
      ctrlptr = ^ctrl;
      regptrtyp = ^register;
      ctrl = record
        operation: c4;                          {character routine abbrev.}
        ptr1,ptr2: regptrtyp;                   {argument locations}
        ln1,ln2: word;                          {argument lengths}
      end;
    var
      argsdescr: ctrl;
      argsptr: ctrlptr;
      store,two30,two45,two48,two48m1,two60,two75,two80,two80m1,two48t80m : register;
      output : packed array[1..40] of char;
 {by interspersing these next variables with zeroes}
 {we easily achieve the leading zero-valued guard word requirement}
      zone,one,ztwo3,two3,ztwo5,two5,ztwo15,two15 : word;
      i : byte;
procedure makecont(cptr:ctrlptr; a1,a2:regptrtyp; l1,l2:word; op:c4);
 {PURPOSE: populate control structure preparatory to an MPIFACE|PPTFACE call}
 {first argument specifies (points to) control record to be filled in}
 {second and third arguments point to values to be operands in MPI call}
 {fourth and fifth arguments are the lengths of the operands for MPI call}
 {sixth argument is the 4-letter truncation of MPI routine to execute}
 {dummy arguments must be substituted where called routine doesn't use any}
   begin
     cptr^.ptr1 := a1;
     cptr^.ptr2 := a2;
     cptr^.ln1  := l1;
     cptr^.ln2  := l2;
     cptr^.operation := op;
   end;
 procedure moveit(into,from:regptrtyp; l:word);
 {PURPOSE: move value from MPI register into a safe place}
 {first argument is pointer to (left edge, element[0] of) target register}
 {second argument is pointer to the source value in an MPI register}
 {third argument is length of the value to be moved}
   var i:byte;
   begin
     for i := 0 to l do
     into^[regmax-l+i] := from^[i];
   end;
    function mpiface(a: ctrlptr): ctrlptr;  external;
    function pptface(a: ctrlptr): char; external;
    begin
        one := 1;  two3 := 8;  two5 := 32;  two15 := 32768;
        zone := 0;  ztwo3 := 0;  ztwo5 := 0;  ztwo15 := 0;
        argsptr := addr(argsdescr);
        makecont(argsptr,addr(ztwo15),addr(ztwo15),1,1,'MULT');
 {we keep track of the lengths of the various values in our heads, instead of}
 {keeping a set of ctrl records which remember the returned value lengths at}
 {each stage;  if we were doing something real, we'd have to keep ctrl records}
 {or, at least, lengths of the various registers}
        argsptr := mpiface(argsptr);
        moveit(addr(two30),argsptr^.ptr1,argsptr^.ln1);           {2**30}
 {the following statement makes a copy of the value for CONB to destroy}
        store := two30;
        output := '                                        ';
 {we point to that copied value in STORE for the CONB2D call}
 {the '1' is a dummy argument here}
        makecont(argsptr,addr(output[40]),addr(store[regmax-2]),1,2,'CONB');
        argsptr := mpiface(argsptr);
        writeln('2 ** 30 is:        ',output);
        makecont(argsptr,addr(two30[regmax-2]),addr(ztwo15),2,1,'MULT');
        argsptr := mpiface(argsptr);
 {we have to make a copy of this product to chain multiply it}
        moveit(addr(two45),argsptr^.ptr1,argsptr^.ln1);           {2**45}
 {the following statement makes a copy of the value for CONB to destroy}
        store := two45;
        output := '                                        ';
 {we point to that copied value in STORE for the CONB2D call}
 {the '1' is a dummy argument here}
        makecont(argsptr,addr(output[40]),addr(store[regmax-3]),1,3,'CONB');
        argsptr := mpiface(argsptr);
        writeln('2**45 is:          ',output);
        makecont(argsptr,addr(two30[regmax-2]),addr(two30[regmax-2]),2,2,'MULT');
        argsptr := mpiface(argsptr);
 {we have to make a copy of this product to chain multiply it}
        moveit(addr(two60),argsptr^.ptr1,argsptr^.ln1);           {2**60}
 {the following statement makes a copy of the value for CONB to destroy}
        store := two60;
        output := '                                        ';
 {we point to that copied value in STORE for the CONB2D call}
 {the '1' is a dummy argument here}
        makecont(argsptr,addr(output[40]),addr(store[regmax-4]),1,4,'CONB');
        argsptr := mpiface(argsptr);
        writeln('2**60 is:          ',output);
        makecont(argsptr,addr(two60[regmax-4]),addr(ztwo15),4,1,'MULT');
        argsptr := mpiface(argsptr);
        moveit(addr(two75),argsptr^.ptr1,argsptr^.ln1);           {2**75}
 {the following statement makes a copy of the value for CONB to destroy}
        store := two75;
        output := '                                        ';
 {we point to that copied value in STORE for the CONB2D call}
 {the '1' is a dummy argument here}
        makecont(argsptr,addr(output[40]),addr(store[regmax-5]),1,5,'CONB');
        argsptr := mpiface(argsptr);
        writeln('2**75 is:          ',output);
        makecont(argsptr,addr(two75[regmax-5]),addr(ztwo5),5,1,'MULT');
        argsptr := mpiface(argsptr);
        moveit(addr(two80),argsptr^.ptr1,argsptr^.ln1);           {2**80}
 {the following statement makes a copy of the value for CONB to destroy}
        store := two80;
        output := '                                        ';
 {we point to that copied value in STORE for the CONB2D call}
 {the '1' is a dummy argument here}
        makecont(argsptr,addr(output[40]),addr(store[regmax-6]),1,6,'CONB');
        argsptr := mpiface(argsptr);
        writeln('2**80 is:          ',output);
        makecont(argsptr,addr(two45[regmax-3]),addr(ztwo3),3,1,'MULT');
        argsptr := mpiface(argsptr);
        moveit(addr(two48),argsptr^.ptr1,argsptr^.ln1);           {2**48}
 {the following statement makes a copy of the value for CONB to destroy}
        store := two48;
        output := '                                        ';
 {we point to that copied value in STORE for the CONB2D call}
 {the '1' is a dummy argument here}
        makecont(argsptr,addr(output[40]),addr(store[regmax-4]),1,4,'CONB');
        argsptr := mpiface(argsptr);
        writeln('2**48 is:          ',output);
        makecont(argsptr,addr(two48[regmax-4]),addr(zone),4,1,'SUBT');
        argsptr := mpiface(argsptr);
        moveit(addr(two48m1),argsptr^.ptr1,argsptr^.ln1);         {2**48 - 1}
 {the following statement makes a copy of the value for CONB to destroy}
        store := two48m1;
        output := '                                        ';
 {we point to that copied value in STORE for the CONB2D call}
 {the '1' is a dummy argument here}
        makecont(argsptr,addr(output[40]),addr(store[regmax-3]),1,3,'CONB');
        argsptr := mpiface(argsptr);
        writeln('2**48 - 1 is:      ',output);
        makecont(argsptr,addr(two80[regmax-6]),addr(zone),6,1,'SUBT');
        argsptr := mpiface(argsptr);
        moveit(addr(two80m1),argsptr^.ptr1,argsptr^.ln1);         {2**80 - 1}
 {the following statement makes a copy of the value for CONB to destroy}
        store := two80m1;
        output := '                                        ';
 {we point to that copied value in STORE for the CONB2D call}
 {the '1' is a dummy argument here}
        makecont(argsptr,addr(output[40]),addr(store[regmax-5]),1,5,'CONB');
        argsptr := mpiface(argsptr);
        writeln('2**80 - 1 is:      ',output);
        makecont(argsptr,addr(two80m1[regmax-5]),addr(two48m1[regmax-3]),5,3,'MULT');
        argsptr := mpiface(argsptr);
        moveit(addr(two48t80m),argsptr^.ptr1,argsptr^.ln1);       {2**80*2**48---}
 {the following statement makes a copy of the value for CONB to destroy}
        store := two48t80m;
        output := '                                        ';
 {we point to that copied value in STORE for the CONB2D call}
 {the '1' is a dummy argument here}
        makecont(argsptr,addr(output[40]),addr(store[regmax-8]),1,8,'CONB');
        argsptr := mpiface(argsptr);
        writeln('Product, last 2 is:',output);
        makecont(argsptr,addr(two48t80m[regmax-8]),addr(two48m1[regmax-3]),8,3,'DIVI');
        argsptr := mpiface(argsptr);
 {the following statement makes a copy of the value for CONB to destroy}
        moveit(addr(store),argsptr^.ptr1,argsptr^.ln1);           {2**48-1 again}
        output := '                                        ';
 {we point to that copied value in STORE for the CONB2D call}
 {the '1' is a dummy argument here}
        makecont(argsptr,addr(output[40]),addr(store[regmax-5]),1,5,'CONB');
        argsptr := mpiface(argsptr);
        writeln('Dividend,(-1)/(-3):',output);
    end.
