        NAME    DWIND
        Title   Draw Window Routine for Quick Basic
        Page    60,132

comment *
Assembly language interface to QuickBasic 4.x for window drawing
Call from Basic as follows:
        DWind  w1,SEG WText$(0),Title$,LCol%,UpRow%
where,
        w1              variable of WindowType
        WText$()        Array of Strings to Print inside Window
                               dimensioned for each row inside window
        Title$          String for Title of Window ("" if none)
        LCol%           Left Column of Window
        UpRow%          Upper Row of Window

structure of WindowType
        BordAttr        Integer    -Color Attribute for Window Border
        WindAttr        Integer    -Color Attribute for Window
        XLen            Integer    -Width of Window
        YLen            Integer    -Heigth of Window
        HorizLines      Long       -Bit Pattern for Rows With Horiz. Lines
        HorizType       Long       -Bit Pattern for Type of Horiz. Lines
        LineType        Integer    -Border Line Type (2-double,1-single,0-none)
        Shadow          Integer    -Shadow Flag (1-Yes,0-No)


Bits in HorizLines will be set as follows:
        Bit location of 0 represents the first row down from the top of
        the window.  Bit 1 represents the second row down, etc.  A value
        of 1 in the bit position means there is a horizontal line to
        draw at that row.  The Line Type for the line is determined by
        the value of the bit at the corresponding position in HorizType.
        If the value is 1 then a double line will be drawn, if 0 then a
        single line will be drawn. However, if LineType is 0 (no lines
        for borders), then the horizontal line will have no line type.
        All horizontal lines will be drawn with the border color
        attribute.

The text specified in the array WText$() will be automatically centered
in the window horizontally.  WText$(0) is the text to display in the
first row down from the top of the window. WText$(1) is the text to
display in the second row down from the window, etc.  If no text is to
be drawn at a row WText$() for that row should be ""(null string). Array
should be dimensioned for each row inside the window.  For example, if
the window is 10 rows high, there are 8 rows inside the window.  The
array would be dimensioned as Dim WText$(0 to 7).

*


;set up equates
;       offsets into WindowType structure
BordAttr        equ     0
WindAttr        equ     2
XLen            equ     4
YLen            equ     6
HorizLF         equ     8
HorizLT         equ     12
BordT           equ     16
Shadow          equ     18

;       define line characters
;       double line characters
dupleft         equ     201
dupright        equ     187
dloleft         equ     200
dloright        equ     188
dhoriz          equ     205
dvert           equ     186

dvert_horiz_left        equ     204
dvert_horiz_right       equ     185

;single line characters
supleft         equ     218
supright        equ     191
sloleft         equ     192
sloright        equ     217
shoriz          equ     196
svert           equ     179

svert_horiz_left        equ     195
svert_horiz_right       equ     180

;double to single interface characters
dvert_shoriz_left       equ     199
dvert_shoriz_right      equ     182

;single to double interface characters
svert_dhoriz_left       equ     198
svert_dhoriz_right      equ     181


space           equ     32


.model medium

.data

Row             dw      0       ;current row to display info
Col             dw      0       ;current column to display info
WindTAdd        dw      0       ;Offset address of window type structure
TitlAdd         dw      0       ;Address of String descriptor for Title
WTextOff        dw      0       ;Offset of WText$() descriptors
WTextSeg        dw      0       ;Segment of WText$() descriptors

LeftCol         dw      0       ;Leftmost column of window
UpRow           dw      0       ;Uppermost row of window
LineLength      dw      0       ;length of line inside window
LastRow         dw      0       ;last row of window

color_attr_1    dw      0       ;color attribute for border
color_attr_2    dw      0       ;color attribute for inside of window
video_stat      dw      0       ;video status (3DAh for CGA, else 0)

left_char       db      0       ;character for left side of window
mid_char        db      0       ;character for middle of window
right_char      db      0       ;character for right side of window

.code
                public  dwind
dwind           proc    far

;set up frame pointer
        push    bp
        mov     bp,sp
;set up variables
        mov     bx,[bp+6]       ;upper row of window
        mov     ax,[bx]
        mov     uprow,ax
        mov     bx,[bp+8]       ;left column of window
        mov     ax,[bx]
        mov     leftcol,ax
        mov     bx,[bp+10]      ;address of title string descriptor
        mov     titladd,bx
        mov     bx,[bp+12]      ;offset of Text Array string descriptors
        mov     wtextoff,bx
        mov     bx,[bp+14]      ;segment of text array string descriptors
        mov     wtextseg,bx
        mov     bx,[bp+16]      ;address of windowtype
        mov     windtadd,bx
        mov     ax,[bx][xlen]           ;width of window
        mov     linelength,ax
        mov     ax,[bx][ylen]           ;heigth of window
        add     ax,uprow                ;add to top row to get last row
        dec     ax                      ;set last row
        mov     lastrow,ax

;get screen segment and monitor type
        mov     dx,0                    ;look at low memory using es
        mov     es,dx
        mov     bx,0b000h               ;assume mono screen segment
        mov     al,es:[410h]            ;get equipment list
        and     al,48                   ;look at monitor type
        cmp     al,48                   ;is it mono?
        jz      Get_screen_1            ;yes, skip over adding 800h
        add     bx,800h                 ;no, adjust for color screen memory
        mov     al,es:[487h]            ;check for ega
        cmp     al,0                    ;if ega or above will not be 0
        jnz     Get_screen_1            ;if ega, leave screen status at 0
        mov     video_stat,3dah         ;if cga,specify port to check retrace

Get_Screen_1:
        mov     es,bx                   ;screen segment in es

;draw window on screen
        mov     ax,uprow                ;set starting row
        mov     row,ax

        mov     ax,leftcol              ;set starting column
        mov     col,ax

Dr_Wind_1:

        call    get_offset              ;get offset into di
        mov     ax,row
        cmp     ax,uprow                ;is this first line?
        jnz     Dr_Wind_2               ;jump if not
        call    Draw_First              ;draw first line
        jmp     Dr_Wind_3               ;continue looping

Dr_Wind_2:

        cmp     ax, lastrow             ;is this last line?
        jnz     Dr_Wind_4               ;jump if not
        call    Draw_Last               ;draw last line
        jmp     Dr_Wind_Done            ;done drawing main window

Dr_Wind_4:

        sub     ax,uprow                ;get delta from top row
        mov     cl,al
        dec     cl                      ;index of current row
        call    Draw_Mid                ;draw row to screen

Dr_Wind_3:

        inc     row                     ;next row
        jmp     Dr_Wind_1               ;continue looping


Dr_Wind_Done:

;print title on top line

;get length of title
        mov     bx,titladd
        mov     cx,[bx]                 ;length of string
        mov     si,[bx+2]               ;address of string

        cmp     cx,0
        jz      Print_Titl_1            ;do not print if null string
;get top row
        mov     ax,uprow
        mov     row,ax
;get starting column
        mov     ax,Leftcol
        mov     col,ax
        mov     ax,LineLength           ;get line length
        shr     ax,1                    ;divide length by 2
        add     col,ax                  ;add to column position

        mov     ax,cx                   ;length of string
        shr     ax,1                    ;divide by 2
        sub     col,ax                  ;subtract to get starting position

        call    get_offset              ;screen offset into di

        mov     bx,windtadd             ;establish addressing to windtadd
        mov     ax,[bx][bordattr]       ;get border color
        mov     ah,al


Print_Titl_2:
        mov     al,byte ptr [si]        ;get character from title
        push    cx
        mov     cx,1                    ;copy 1 word
        call    Wrt_Wrd_Scn
        pop     cx
        inc     si                      ;point to next character
        loop    Print_Titl_2            ;continue until title is printed

Print_Titl_1:


;draw shadow to screen if required
        mov     si,windtadd
        mov     ax,[si][Shadow]         ;get shadow flag
        cmp     ax,0                    ;flag required? (0 means no)
        jz      Dr_Shad_1               ;no, skip shadow code

        mov     ax,lastrow              ;get last row of window
        inc     ax                      ;point to row for bottom shadow
        mov     row,ax

        mov     ax,leftcol              ;get left column of window
        inc     ax                      ;move one column over for shadow effect
        mov     col,ax

        call    get_offset              ;get offset into di
        inc     di                      ;point to attribute byte

        mov     al,8                    ;attribute -gray on black
        mov     cx,linelength           ;counter for linelength
        call    Wrt_Byt_Scn             ;write bottom shadow to screen

;do right side shadow
        mov     ax,uprow
        inc     ax
        mov     row,ax                  ;top row of right side shadow

        mov     ax,leftcol
        add     ax,linelength           ;get right column for shadow
        mov     col,ax

        mov     si,windtadd
        mov     ax,[si][ylen]           ;heigth of window
        mov     cx,ax                   ;for length of right side shadow

        mov     al,8                    ;color attribute gray on black

Dr_Shad_2:
        call    get_offset              ;get offset into di
        inc     di                      ;point to attribute byte
        push    cx
        mov     cx,1                    ;1 byte at a time
        call    Wrt_Byt_Scn             ;write attribute byte
        pop     cx
        inc     row                     ;point to next row for shadow
        loop    Dr_Shad_2               ;do until shadow complete

Dr_Shad_1:

;print text in window
        mov     ax,uprow                ;get top row
        inc     ax                      ;point to first row for text
        mov     row,ax
        mov     bx,0                    ;use bx for indexing descriptors

Pr_Text_3:
        push    es                      ;save screen segment
        mov     ax,wtextseg             ;get segment of string descriptor
        mov     es,ax                   ;move into es

        mov     si,wtextoff
        mov     cx,es:[si][bx]          ;get length of string
        cmp     cx,0                    ;null string?
        jnz     Pr_Text_4               ;no,continue to print string
        pop     es                      ;yes,pop es and do next row
        jmp     Pr_Text_1

Pr_Text_4:

        mov     ax,es:2[si][bx]         ;get address of string
        mov     si,ax

        mov     ax,leftcol
        mov     col,ax                  ;get leftmost column
        mov     ax,linelength           ;get linelength
        shr     ax,1                    ;divide by 2
        add     col,ax                  ;add to left column
        push    cx
        shr     cx,1                    ;divide string length by 2
        sub     col,cx                  ;subtract to get starting column
        pop     cx

        call    get_offset              ;offset into di
        pop     es                      ;restore es to screen segment

Pr_Text_2:
        mov     al,byte ptr [si]        ;character to print
        push    cx                      ;save loop counter
        mov     cx,1                    ;print 1 character at a time
        call    Wrt_Byt_Scn             ;write character to screen
        pop     cx                      ;restore loop counter
        inc     si                      ;point to next character
        loop    Pr_Text_2

Pr_Text_1:
        add     bx,4                    ;index for next string descriptor
        inc     row                     ;point to next row
        mov     ax,lastrow
        cmp     row,ax                  ;done inside window?
        jl      Pr_Text_3               ;no, do next row

        pop     bp
        ret     12                      ;return and pop stack


dwind   endp





;--------------------------------------------------------------------------
;subroutine to draw a line on screen
;       row contains row to start
;       col contains column to start line
;       color_attr_1 contains border color
;       color_attr_2 contains middle color
;       left_char contains left side character
;       right_char contains right side character
;       mid_char contains middle character
;       LineLength contains length of line
;--------------------------------------------------------------------------

Draw_Line:

        call    get_offset              ;get offset into di
        mov     ah,byte ptr color_attr_1        ;border color
        mov     al,left_char            ;get left side character
        mov     cx,1
        call    Wrt_Wrd_Scn             ;write to screen
        mov     ah, byte ptr color_attr_2       ;middle color
        mov     al,mid_char             ;middle character
        mov     cx,LineLength
        sub     cx, 2                   ;length of middle part of line
        call    Wrt_Wrd_Scn             ;write to screen
        mov     ah, byte ptr color_attr_1       ;border color
        mov     al, right_char          ;right side character
        mov     cx,1
        call    Wrt_Wrd_Scn             ;write to screen

        ret



;--------------------------------------------------------------------------
;subroutine to draw middle lines of window to screen
;       row contains current row
;       col contains current starting column
;       cx=delta from top of window(1st line down=0,2nd=1, etc)
;--------------------------------------------------------------------------

Draw_Mid:

        mov     si,windtadd
        mov     ax,[si][horizlf]        ;get horizontal line flag
        shr     ax,cl                   ;shift for test of bit
        and     ax,01                   ;is it set?
        jnz     Horiz_Line              ;yes,draw horiz line
        jmp     No_Horiz_Line           ;no, skip code

;draw horizontal line at this row

Horiz_Line:
        mov     ax,[si][bordattr]       ;get border color
        mov     color_attr_1,ax         ;set border color
        mov     color_attr_2,ax         ;set middle color
        mov     ax,[si][horizlt]        ;get horizontal line type
        shr     ax,cl                   ;get appropriate bit
        and     ax,01                   ;is it set?
        jz      Sing_Horiz_Line         ;if not then do single line type

;draw double horizontal line at this row

        mov     ax, [si][bordt]         ;get border type
        cmp     ax, 2                   ;is it double lines?
        jnz     Draw_Mid_1              ;if not, continue test

;double vertical/double horizontal
        mov     left_char,dvert_horiz_left
        mov     mid_char,dhoriz
        mov     right_char,dvert_horiz_right
        jmp     Draw_Mid_2

Draw_Mid_1:
        cmp     ax,1                    ;single border lines?
        jnz     Draw_Mid_3

;single vertical/double horizontal
        mov     left_char,svert_dhoriz_left
        mov     mid_char,dhoriz
        mov     right_char,svert_dhoriz_right
        jmp     Draw_Mid_2

Draw_Mid_3:
;no border lines
        mov     left_char,space
        mov     mid_char,space
        mov     right_char,space
        jmp     Draw_Mid_2

;draw a single horizontal line at this row
Sing_Horiz_Line:
        mov     ax,[si][bordt]          ;get border type
        cmp     ax,2                    ;double lines?
        jnz     Draw_Mid_4

;double vertical/single horizontal
        mov     left_char,dvert_shoriz_left
        mov     mid_char,shoriz
        mov     right_char,dvert_shoriz_right
        jmp     Draw_Mid_2

Draw_Mid_4:
        cmp     ax,1                    ;single lines?
        jnz     Draw_Mid_5

;single vertical/single horizontal
        mov     left_char,svert_horiz_left
        mov     mid_char,shoriz
        mov     right_char,svert_horiz_right
        jmp     Draw_Mid_2

;no border
Draw_Mid_5:
        mov     left_char,space
        mov     mid_char,space
        mov     right_char,space
        jmp     Draw_Mid_2

;No horizontal Line at this row
No_Horiz_Line:
        mov     ax,[si][bordattr]       ;get border color
        mov     color_attr_1,ax
        mov     ax,[si][windattr]       ;get window color
        mov     color_attr_2,ax
        mov     ax,[si][bordt]          ;get border type
        cmp     ax,2                    ;double lines?
        jnz     Draw_Mid_6

;double line border
        mov     left_char,dvert
        mov     mid_char,space
        mov     right_char,dvert
        jmp     Draw_Mid_2

Draw_Mid_6:
        cmp     ax,1                    ;single lines?
        jnz     Draw_Mid_7

;single line border
        mov     left_char,svert
        mov     mid_char,space
        mov     right_char,svert
        jmp     Draw_Mid_2

;no border lines
Draw_Mid_7:
        mov     left_char,space
        mov     mid_char,space
        mov     right_char,space

Draw_Mid_2:
        call    Draw_Line
        ret




;--------------------------------------------------------------------------
;subroutine to draw first line of window to screen
;       row contains current row
;       col contains current starting column
;--------------------------------------------------------------------------
Draw_First:

        mov     si,windtadd
        mov     ax,[si][bordattr]       ;get border attribute
        mov     color_attr_1,ax
        mov     color_attr_2,ax
        mov     ax,[si][bordt]          ;get border type
        cmp     ax,02                   ;double lines?
        jnz     Dr_First_1

        mov     left_char,dupleft       ;left side-double line
        mov     mid_char,dhoriz         ;double horizontal lines
        mov     right_char,dupright     ;right side-double lines
        call    Draw_Line               ;draw line on screen
        jmp     Draw_First_End          ;jump to end of routine

Dr_First_1:

        cmp     ax,01                   ;single lines?
        jnz     Dr_First_2              ;jump if not
        mov     left_char,supleft       ;left side-single line
        mov     mid_char, shoriz        ;single horizontal lines
        mov     right_char,supright     ;right side-single lines
        call    Draw_Line               ;draw line on screen
        jmp     Draw_First_End          ;jump to end of routine

Dr_First_2:

        mov     left_char, space        ;space
        mov     mid_char, space         ;space
        mov     right_char,space        ;space
        call    Draw_Line               ;draw line on screen

Draw_First_End:

        ret


;--------------------------------------------------------------------------
;subroutine to draw last line of window to screen
;       row contains current row
;       col contains current starting column
;--------------------------------------------------------------------------
Draw_Last:

        mov     si,windtadd
        mov     ax,[si][bordattr]       ;get border attribute
        mov     color_attr_1,ax
        mov     color_attr_2,ax
        mov     ax,[si][bordt]          ;get border type
        cmp     ax,02                   ;double lines?
        jnz     Dr_Last_1

        mov     left_char, dloleft      ;left side-double line
        mov     mid_char, dhoriz        ;double horizontal lines
        mov     right_char,dloright     ;right side-double lines
        call    Draw_line               ;draw line on screen
        jmp     Draw_Last_End           ;jump to end of routine

Dr_Last_1:

        cmp     ax,01                   ;single lines?
        jnz     Dr_Last_2               ;jump if not
        mov     left_char, sloleft      ;left side-single line
        mov     mid_char, shoriz        ;single horizontal lines
        mov     right_char,sloright     ;right side-single lines
        call    Draw_Line               ;draw line on screen
        jmp     Draw_Last_End           ;jump to end of routine

Dr_Last_2:

        mov     left_char, space        ;space
        mov     mid_char, space         ;space
        mov     right_char,space        ;space
        call    Draw_Line

Draw_Last_End:

        ret

;--------------------------------------------------------------------------
;subroutine to calculate offset into screen segment based on current
;row and column
;       row is stored in variable row
;       column is stored in variable col
;       di=offset into screen segment (returned value)
;--------------------------------------------------------------------------

get_offset:
            push  cx
            push  bx
            push  ax
            mov   ax,row
            dec   ax          ;change from 1-25 to 0-24
            mov   cl,160
            mul   cl          ;multiply row by 160
            mov   bx,col      ;get starting column
            dec   bx          ;change from 1-80 to 0-79
            shl   bx,1        ;multiply (row-1) by 2
            add   ax,bx       ;get offset
            mov   di,ax       ;move to destination index
            pop   ax
            pop   bx
            pop   cx

            ret


;--------------------------------------------------------------------------
;subroutine to write attribute or character to screen
;       di=offset into screen memory
;       cx=# of bytes to write
;       al=attribute or char to write
;       dx=3dah if cga else 0
;       es=screen segment
;--------------------------------------------------------------------------

Wrt_Byt_Scn:

        push    dx
        push    ax                      ;save character info
        mov     dx,video_stat           ;get video status

        cld
        cmp     dl,0                    ;mono/ega/vga ?
        jz      By_Mono                 ;bypass checking if it is

No_Retrace_Byte:

        in      al,dx                   ;get video status byte
        test    al,1                    ;test for retrace in progress
        jnz     No_Retrace_Byte         ;wait until a retrace is started

Retrace_Byte:

        in      al,dx                   ;get video status byte
        test    al,1                    ;test for retrace in progress
        jz      Retrace_Byte            ;wait until done retracing
        pop     ax                      ;restore character info
        push    ax                      ;save character info
        stosb                           ;store byte at position
        inc     di                      ;point to next byte
        loop    No_Retrace_Byte         ;loop until finished
        jmp     Wrt_Byt_Done            ;jump to finishing code

By_Mono:

        stosb                           ;store byte into screen position
        inc     di                      ;point to next byte
        loop    By_Mono                 ;loop until finished

Wrt_Byt_Done:

        pop     ax
        pop     dx
        ret


;--------------------------------------------------------------------------
;subroutine to write character and attribute to screen
;       di=offset into screen memory
;       cx=# of words to write
;       al=char to write
;       ah=attribute to write
;       dx=3dah if cga else 0
;       es=screen segment
;--------------------------------------------------------------------------

Wrt_Wrd_Scn:

        push    dx
        push    ax                      ;save character info
        mov     dx,video_stat           ;get video status

        cld
        cmp     dl,0                    ;mono/ega/vga ?
        jz      Wd_Mono                 ;bypass checking if it is

No_Retrace_Word:

        in      al,dx                   ;get video status byte
        test    al,1                    ;test for retrace in progress
        jnz     No_Retrace_Word         ;wait until a retrace is started

Retrace_Word:

        in      al,dx                   ;get video status byte
        test    al,1                    ;test for retrace in progress
        jz      Retrace_Word            ;wait until done retracing
        pop     ax                      ;restore character information
        push    ax                      ;save character info
        stosw                           ;store word at position
        loop    No_Retrace_Word         ;loop until finished
        jmp     Wrt_Wrd_Done            ;jump to finishing code

Wd_Mono:

        stosw                           ;store word into screen position
        loop    Wd_Mono                 ;loop until finished

Wrt_Wrd_Done:

        pop     ax
        pop     dx
        ret
        end

