/*==========================================================================\
|                                                                           |
|       Name:           FONTLIB.H Copyright 1993 (c) Joseph V. Gagliano     |
|       Author:         Joseph V. Gagliano                                  |
|       Language:       Microsoft QuickC v2.5                               |
|       Description:    Include file to be used with FONTLIB v1.0 Library.  |
|                                                                           |
|                                                                           |
|                                                                           |
\==========================================================================*/

/*==========================================================================\
|                                                                           |
|                               DEFINES                                     |
|                                                                           |
\==========================================================================*/

#define MAX_CHARS 128           /* max number of character/keys allowed */
#define MAX_ROWS  60            /* max number of rows allowed */
#define MAX_COLS  30            /* max number of cols allowed */

#define TO_RADS(deg) (deg/180.0) * 3.14159265358979   /* convert deg to rad */

/* preferred vectors */
#define RIGHT  0
#define LEFT   180
#define UP     90
#define DOWN   270

/* preferred spacing adjustments */
#define S0     0
#define S1     1
#define S2     2
#define S3     3

/* preferred shadow depths */
#define SHADOFF 0
#define SHAD1   1
#define SHAD2   2
#define SHAD3   3

/*==========================================================================\
|                                                                           |
|                         FONTKEY STRUCTURE                                 |
|                                                                           |
\==========================================================================*/

/* structure used to store and index fonts */
typedef struct FONTKEY
{
  char key;
  unsigned long hbit[MAX_ROWS];
}FONTKEY;



/*==========================================================================\
|                                                                           |
|                         FUNCTION PROTOTYPES                               |
|                                                                           |
\==========================================================================*/

/* register fonts */
int regfont(char *fname);

/* draw individual letters */
void draw_letter(float,int , int ,unsigned long *,int );
void draw_letter_bold(float ,int , int ,unsigned long *,int);

/* standard lettering functions */
void letter(int ,int , char *,int ,int ,int);
void letter_bold(int ,int , char *,int ,int ,int);
void letter_w(double ,double , char *,int ,int ,int );
void letter_bold_w(double ,double , char *,int ,int ,int );

/* columnar lettering functions */
void letter_column(int ,int , char *,int );
void letter_column_bold(int ,int , char *,int );
void letter_column_w(double ,double , char *,int );
void letter_column_bold_w(double ,double , char *,int );

/* clear font & release allocated memory */
void freefont(void);

/* change the shadow color: default = 8 (lightgrey) */
void set_shadow_color( int );


