;RM82TS8,16,24,32,40,48
;Updated 11/20/90

comment |
 =============================================================================
 DECLARE FUNCTION SAYBREAK% (STATUS%)
 Trap CTRL-BREAK,CTRL-C KEYPRESS

 Status% =  0  to uninstall
 Status% = -1 to install
 Returns
       0 = did what user asked
      -1 = error

 =============================================================================
	|
comment |
Written by Jim Mack, CIS ID 76630,2012.  Please direct all questions or
comments to me in the MSSYS forum in the BASIC or MASM sections.
	NOTICE
This and the accompanying QB/BC code are Copyright 1990, Editing Services Co.
Permission is granted for unlimited reproduction, as long as the complete
text is included.  Use this at your own risk: Editing Services Co. disclaims
all warranties and the Secretary disavows any knowledge of your actions.
{SJK, after this appeared in QBNEWS I believe it became public domain}
	|

; Keyboard Equates
Kb_Data         EQU      60h
Kb_Ctrl         EQU      61h
EOI             EQU      20h
Int_Ctrl_Port   EQU      20h
Interrupt_No    EQU       9H

CTRL_KEY        EQU      1DH
ALT_KEY         EQU      38H
LSHIFT_KEY      EQU      2AH
RSHIFT_KEY      EQU      36H
CAPS_LOCK_KEY   EQU      3AH
INS_KEY         EQU      52H
NUM_LOCK_KEY    EQU      45H

SCROLL_LOCK_K   EQU      46H
C_KEY           EQU      2EH

;the following are undocumented control break keys
Num_2_Key       EQU      03H         ;CTRL-2
Pg_Down         EQU      51h         ;ALT-PGDN

;so can ignore a key release too
SCROLL_LOCK_KS  EQU      46H + 80h
C_KEYS          EQU      2EH + 80h

;the following are undocumented control break keys
Num_2_KeyS      EQU      03H + 80h
Pg_DownS        EQU      51h + 80h

DOSSEG
.model medium

	PUBLIC SAYBREAK

.CODE

EXTRN   B_OnExit:FAR                    ; this is in the BC runtime
EXTRN   SetUEvent:FAR                   ; ditto

EVEN
PreviousVector  LABEL DWORD             ; placed in the code segment to...
PVOffset        dw      0               ;
PVSegment       dw      0               ;

EVEN
DataSegAdd      dw       0              ; store DGROUP address inside QB.EXE

EVEN
B_Exit_Notify   db       0              ; keep track if ever installed
Installed       db       0              ; a status byte to tell if uninstalled
Error_Status    db       0              ; say we got an error before
Ignore_Key_Rec  db       0              ; notify system that E0 keycode rec'd
SetU_Busy       db       0              ; to prevent re-entry until processing
					; is complete.

comment |
This takes care of wrapping our metaphorical tentacles around the PC.
We take over the interrupt vector associated with the IRQ level passed,
saving the previous contents so that we can put things back the way they
were when we exit, and enable the interrupt level in the PC hardware.
	|

EVEN
SAYBREAK  PROC FAR
	Push    BP
	Mov     BP,SP
	Mov     BX,[BP+6]
	Mov     AX,[BX]
	Or      AX, AX                  ; see if supposed to uninstall
	JNZ     @f
	Call    FAR PTR REMOVEINT       ; if Status% = 0 then uninstall
	Xor     AX, AX                  ; return zero: all OK
	Jmp     SHORT Finiss            ; quit
@@:
	CMP     AX, -1                  ; see if want to install
	JNE     Instl_Err               ; if not Status% <> -1 then quit
	CMP     Error_Status, -1        ; if got an error before
	JE      Instl_Err               ; then quit
	CMP     B_Exit_Notify,0         ; insure this is first time
	JNE     Reset_Inter             ; so only call B_OnExit once in program
	Mov     Installed,1             ; note that program is installed
	Mov     B_Exit_Notify,1         ; and the B_OnExit called once
	Mov     DX, offset REMOVEINT
	Push    CS                      ; Push far address of RemoveInt
	Push    DX                      ; to register the exit routine
	Call    B_OnExit                ; so that we don't hang machine
	OR      AX, AX                  ; registered OK?
	JNZ     Reset_Inter             ; if AX = 0 then too many registered
					; routines
	Mov     Error_Status, -1        ; note for future trys, that we got
					; an error so don't try to reinstall
	Jmp     SHORT Instl_Err             ; and exit

Reset_Inter:
	Mov     AX,DS                   ; get DGROUP address
	Mov     DataSegAdd,AX           ; store it in CS:DataSegAdd
					; this prevents a crash in QB.EXE
					; which changes the DS pointer

	Mov     AX, Interrupt_No        ; B_Onexit corrupted this
	Mov     AH, 35H                 ; use DOS to get current...
	INT	21H			; ...holder of the vector
	Mov     PVSegment, es           ; save for chaining, removal
	Mov     PVOffset, BX
	Mov     AX, Interrupt_No        ; get INT number
	Push    DS                      ; hold DS a moment
	Push    CS                      ; transfer CS to DS
	Pop     DS                      ; through the stack

	ASSUME  DS:@code

	Mov     DX, offset INTHANDLER   ; get address of our MASM handler
	Mov     AH, 25H                 ; tell DOS we're taking this vector
	INT     21H                     ; grab the vector
	Pop     DS                      ; restore DS

	ASSUME  DS:@data

	Mov     Installed, 1            ; Note that we are installed
	Xor     AX, AX                  ; return zero: all OK
Finiss:
	Pop     BP
	RET     2                       ; remove parameters from stack

Instl_Err:
	Mov     Installed, 0            ; note that it is not installed
	Mov     AX, -1                  ; couldn't install: return TRUE
	Jmp     SHORT Finiss

SAYBREAK  ENDP

comment |
RemoveInt will undo the InstallHandler by restoring the PC to the state it
was in before we came along.  This is vital, because otherwise when the
program terminates the interrupt vector will point into volatile space,
guaranteeing an eventual machine hang.
Because we "register" this routine with B_OnExit, restoring the machine state
becomes a part of BASIC's normal shutdown process: we don't need to call it.
B_OnExit can end 32 routines
	|

EVEN
REMOVEINT  PROC FAR
	CMP     Installed,1
	JE      @f ;unistall only in Installed = 1
	RET        ;not installed so don't need to uninstall
@@:
	Push    DS
	Mov     DX, PVOffset            ; get old values
	Mov     AX, PVSegment
	Mov     DS, AX
	Mov     AX, Interrupt_No        ; level we're using
	Mov     AH, 25H                 ; SET VECTOR call
	INT     21H                     ; restore it
	Pop     DS                      ; restore registers
	Mov     Installed, 0            ; note that it is uninstalled
	RET
REMOVEINT       ENDP


comment	|
	This does the main work.  It looks for a CTRL-BREAK, a CTRL-C
as well as the CTRL-2 and ATL-3 (since the INKEY$ string is the same)
and removes them from the keyboard input stream.  Because the LINE INPUT
and INPUT routines do not use the BIOS or DOS services to check for
CTRL-BREAK we have to hide the keystrokes from QBASIC.  Only this part
of the code is Copr., Copyright 1990, Sidney J. Kelly, All rights
reserved.  We have to have saved DS because SETUEVENT, inside QB.EXE
changes DS.
	|

EVEN
INTHANDLER  PROC FAR
	Assume  CS:@code, DS:@data, ES:nothing
	Sti                     ; allow more important interrupts
	Push    BP
	Push    DS              ; only save the registers you really need to
	Push    DI
	Push    ES              ; here, we assume BC can destroy anything
	Push    SI
	Push    DX
	Push    CX
	Push    BX
	Push    AX
	Pushf                    ; flags will now be stored with STI
	Mov     AX,DataSegAdd    ; make DS point to DGROUP so program
				 ; can be called inside QB.EXE
	Mov     DS,AX            ; RESTORE DS to original status
	In      AL, Kb_Data      ; read keyboard
	CMP     AL, 0E0h         ; see if got extended keyboard key
	JE      E0_Reset         ; eliminate extended keycode
	CMP     AL, 0E1H         ; see if got the PAUSE key
	JE      Ignore_Key       ; eliminate pause extended keycode
	Cmp	AL,0FFh		 ; eliminate keyboard noise
	JE      Ignore_Key       ; eliminate pause extended keycode

	;This wont work with the new extended keyboard
	;Test    AL, 80H          ; Test just for key press
	;JNZ     Not_Found        ; ignore any release codes

	Push    AX               ; save Scan code from keyboard
	Mov     AH, 2            ; read keyboard status
	Int     16h
	Test    AL, 4            ; see if control key pressed
	JZ      Test_Alt         ; nope, so skip & test alt key
	Pop     AX               ; get back scan code

	CMP     AL, SCROLL_LOCK_K
	JE      Kb_Reset         ; CTRL-BREAK pressed so reset
	CMP     AL,C_KEY
	JE      Kb_Reset         ; CTRL-C pressed so reset
	CMP     AL,Num_2_Key
	JE      Kb_Reset         ; CTRL-2 pressed so reset
	Jmp     Short   Not_Found

	;Don't uncomment because this test is unnecessary & takes too long
	;CMP     AL, SCROLL_LOCK_KS
	;JE      Ignore_Key         ; CTRL-BREAK released so reset
	;CMP     AL,C_KEYS
	;JE      Ignore_Key         ; CTRL-C released so reset
	;CMP     AL,Num_2_KeyS
	;JE      Ignore_Key         ; CTRL-2 released so reset

Test_Alt:
	Test    AL,8              ; Test to see if ALT Key pressed
	Pop     AX                ; restore AX
	JZ      Not_Found         ; if not pressed then quit

	CMP     AL,Pg_Down
	JE      Kb_Reset          ; ALT-PGDN pressed so reset

	;Don't test for this because it takes too long
	;uncomment if want to use
	;CMP     AL,Pg_DownS      ; ALT-PGDN release so reset
	;JE      KB_Reset

Not_Found:
	Popf
	Pop     AX
	Pop     BX
	Pop     CX
	Pop     DX
	Pop     SI
	Pop     ES
	Pop     DI
	Pop     DS
	Pop     BP
	Jmp     CS:[PreviousVector] ;continue on with prior interrupt path
	Assume  CS:@code, DS:@data, es:nothing

EVEN
E0_Reset:                         ; tell bios that we got a 0E0h key
	Sub     BX,BX             ; this will let bios return
	Mov     ES,BX             ; extended keycodes
	Or      Byte Ptr ES:[0496h],010b ; set the Last Code was E0h bit

Ignore_Key:
	Mov     Ignore_Key_Rec,1

Kb_Reset:
        In      AL,Kb_Ctrl        ; reset keyboard
	Mov     AH,AL             ; so QBASIC does not catch on
	OR      AL,80h
	Out     Kb_Ctrl,AL        ; reset keyboard
	Mov     AL,AH
	Jmp     SHORT $+2         ; add a delay for AT keyboards
        Out     Kb_Ctrl,AL

        Cli                       ; turn off interrupts
	Mov     AL,EOI            ; load AL with generic EOI code
	Jmp     SHORT $+2         ; add a delay for AT keyboards
	Out     Int_Ctrl_Port,AL  ; reset PIC with an end_of_interrupt
        Sti                       ; allow timer interrupt

        Cmp     Ignore_Key_Rec,1  ; should we ignore key
	JE      @F                ; yes so skip forward
	Cmp     SetU_Busy,1       ; don't call if busy (Very important to 
				  ; prevent re-entrancy!)
	JE      @f                ; if yes skip ahead
	Mov     SetU_Busy,1       ; say interrupt routine is busy
				  ; this prevents recursive interrupts
	Call    SetUEvent         ; notify BC/QB that the event occurred
	Mov     SetU_Busy,0       ; tell routine that we are not busy
@@:
	Mov     Ignore_Key_Rec,0  ; reset Ignore_Key_Rec
	PopF                      ; restore everything as it was
        Pop     AX
	Pop     BX
	Pop     CX
	Pop     DX
	Pop     SI
	Pop     ES
	Pop     DI
	Pop     DS
	Pop     BP
	Iret                     ; return to system
INTHANDLER      ENDP
END
