;RM82TS8,16,24,32,40,48
;updated 11/20/90

;=============================================================================
;DECLARE FUNCTION BREAKERHANDLER% (STATUS%)
;Trap CTRL-BREAK, CTRL-C, CTRL-2, ALT-3(PgDn) KEYPRESSES
;Input:
;	If status = 0 then uninstall
;	If status = -1 then install
;	Else = exit with error
;Returns:
;	0 if no errors.
;	-1 if an error observed, bad input or B_OnExit is full.
;=============================================================================

comment |

Written by Jim Mack, CIS ID 76630,2012.  Please direct all questions or
comments to me IN the MSSYS forum in the BASIC or MASM sections.

				NOTICE

This and the accompanying QB/BC code are Copyright 1990, Editing Services Co.
Permission is granted for unlimited reproduction, as long as the complete
text is included.  Use this at your own risk: Editing Services Co. disclaims
all warranties and the Secretary disavows any knowledge of your actions.
{SJK This material became public domain when published in QBASIC news}
	|
;-------------------------Equates to make Programming Easier--------------
KB_Data         EQU      60h
KB_Ctrl         EQU      61h
EOI             EQU      20h
Int_Ctrl_Port   EQU      20h
Interrupt_No    EQU       9H

CTRL_KEY        EQU      1DH
ALT_KEY         EQU      38H
LSHIFT_KEY      EQU      2AH
RSHIFT_KEY      EQU      36H
CAPS_LOCK_KEY   EQU      3AH
INS_KEY         EQU      52H
NUM_LOCK_KEY    EQU      45H

SCROLL_LOCK_K   EQU      46H
C_KEY           EQU      2EH
;the following are undocumented control break keys
Num_2_Key       EQU      03H         ;CTRL-2
Pg_Down         EQU      51h         ;ALT-PGDN

;so can ignore a key release too
SCROLL_LOCK_KS  EQU      46H + 80h
C_KEYS          EQU      2EH + 80h
;the following are undocumented control break keys
Num_2_KeyS      EQU      03H + 80h
Pg_DownS        EQU      51h + 80h

DOSSEG
.model medium
        PUBLIC BREAKHANDLER
.CODE

EXTRN   B_OnExit:FAR                    ; this is in the BC runtime

EVEN
PreviousVector  LABEL DWORD             ;kept in code segment for
PVOffset        DW      0               ;to reduce use of DGROUP
PVSegment       DW      0

Installed       DB      0               ; status byte to tell if uninstalled
Error_Status    DB      0               ; keep track of previous errors
B_Exit_Notify   DB      0               ; status byte to report if ever
					; installed

comment |
We take over the interrupt vector associated with the IRQ level passed,
saving the previous contents so that we can put things back the way they
were when we exit.
	|

STATUS          EQU      [BP+6]        ; make it easy to remember variable
					; on the stack
EVEN
BREAKHANDLER  PROC FAR
	Push    BP
	Mov     BP,SP
	Mov     BX,STATUS               ; read address of status
	Mov     AX,[BX]                 ; get status into AX
	Or      AX,AX                   ; see if supposed to uninstall
	JNZ     @f                      ; jump if AX was not 0
	Call    FAR PTR REMOVEINT       ; if Status% = 0 then uninstall
	Xor     AX, AX                  ; return zero: all OK
	JMP     SHORT finis             ; quit
@@:
	Cmp     AX, -1                  ; see if want to install
	JNE     InErr                   ; if not Status% <> -1 then quit
	Cmp     Error_Status, -1        ; if got an error before
	JE      InErr                   ; then quit
	Cmp     B_Exit_Notify,0         ; insure this is first time
	JNE     Reset_Inter             ; so only call B_OnExit once in program
	Mov     Installed,1             ; note that program is installed
	Mov     B_Exit_Notify,1         ; and the B_OnExit called once
	Mov     DX, offset REMOVEINT
	Push    CS                      ; Push far address of RemoveInt
	Push    DX                      ; to register the exit routine
	Call    B_OnExit                ; so that we don't hang machine
	Or      AX, AX                  ; registered OK? Only 32 programs
					; can register
	JNZ     Reset_Inter             ; if AX = 0 then too many registered
					; routines
	Mov     Error_Status, -1        ; note for future trys, that we got
					; an error so don't try to reinstall
	JMP     SHORT InErr             ; and exit
Reset_Inter:
	Mov     AL, Interrupt_No        ; B_Onexit corrupted this
	Mov     AH, 35H                 ; use DOS to get current...
	Int     21h                     ; ...holder of the vector
	Mov     PVSegment, ES           ; save for chaining, removal
	Mov     PVOffset, BX

	Mov     AL, Interrupt_No        ; get Int number
	Push    DS                      ; hold DS a moment
	Push    CS                      ; transfer CS to DS
	Pop     DS                      ; through the stack

	Assume  DS:@CODE

	Mov     DX, offset INTHANDLER   ; get address of our MASM handler
	Mov     AH, 25H                 ; tell DOS we're taking this vector
	Int     21h                     ; grab the vector
	Pop     DS                      ; restore DS

	Assume  DS:@DATA

	Mov     Installed, 1            ; Note that we are installed
	Xor     AX, AX                  ; return zero: all OK
finis:
	Pop     BP
	RET     2                       ; remove parameters from stack

InErr:
	Mov     Installed, 0            ; note that it is not installed
	Mov     AX, -1                  ; couldn't install: return TRUE
	JMP     SHORT finis             ; and quit

BREAKHANDLER  ENDP

comment |
RemoveInt will undo the InstallHandler by restoring the PC to the state it
was in before we came along.  This is vital, because otherwise when the
program terminates the interrupt vector will point into volatile space,
guaranteeing an eventual machine crash.
Because we "register" this routine with B_OnExit, restoring the machine state
becomes a part of BASIC's normal shutdown process: we don't need to call it.
	|

EVEN
REMOVEINT  PROC FAR
	Cmp     Installed,1      ; uninstall only if Installed = 1
	JNE     REMOVE_end       ; not installed so don't need to uninstall
	Push    DS
	Mov     DX, PVOffset            ; get old values
	Mov     AX, PVSegment
	Mov     DS, AX


	Mov     AL, Interrupt_No        ; level we're using
	Mov     AH, 25H                 ; SET VECTOR call
	Int     21h                     ; restore it
	Pop     DS                      ; restore registers
	Mov     Installed, 0            ; note that it is uninstalled

REMOVE_end:                             ; quit
	RET
REMOVEINT       ENDP


comment	|
	This does the main work.  It looks for a CTRL-BREAK Or a CTRL-C
	as well as the CTRL-2  & ALT PgDN (the undocumentd CTRL-BREAK keys)
and removes them from the keyboard input stream.  Because the LINE INPUT
and INPUT routines do not use the DOS services to check for
CTRL-BREAK we have to hide the keystrokes from QBASIC.
A simple INT 1Bh or INT 23h interrupt routine will NOT do it.
	Note:  If you use DOS string editing routines in QBASIC, then B_OnExit is
not called if DOS finds a CTRL-BRK type key.  The result is that our program
will terminate with our interrupt handler installed --- A guaranteed CRASH!!!
So we have to hide these keys from everyone.  A simple INT 23h handler
will halt DOS from crashing program, however screen will be littered with
little old "^C"'s everywhere.  Moreover, INT 23h will not stop QBASIC's INPUT
from finding those nasty old "^C"'s, so we stomp those babies flat!!
In doing so, we lose the ability to enter extended characters using
ALT-3 key.
	Only this part of the code is Copr., Copyright 1990, Sidney J. Kelly
All rights reserved.
	|

EVEN
INTHANDLER  PROC FAR
	Assume  CS:@code, DS:nothing, ES:nothing
	STI                       ; allow more important interrupts
	Push    ES
	Push    BX                ; save used flags
	Push    AX
	PushF			  ; STI stored in with Flags
	In      AL, KB_Data

	;Following instructions prevent Int 16h from being confused
	;if we eliminate the trailing keys from the extended keyboard

	Cmp     AL, 0E0h          ; see if got extended keyboard key
	JE      E0_Reset          ; eliminate extended keycode
	Cmp     AL, 0E1h          ; see if got extended keyboard key
	JE      KB_Reset          ; eliminate extended keycode
	Cmp     AL, 0FFh          ; eliminate keyboard noise
				  ; when we don't clear the key release keys
	JE	KB_Reset	  ; since we don't clear key releases

	Push    AX                ; save the keypress
	Mov     AH, 2             ; read keyboard status
	Int     16h
	Test    AL, 4             ; see if control key pressed
	JZ      Test_Alt          ; control not pressed so Test ALT key
	Pop     AX                ; restore the keypress
	Cmp     AL, SCROLL_LOCK_K
	JE      KB_Reset          ; CTRL-BREAK pressed so reset
	Cmp     AL,C_KEY
	JE      KB_Reset          ; CTRL-C pressed so reset
	Cmp     AL,Num_2_Key
	JE      KB_Reset          ; CTRL-2 pressed so reset
	Jmp     Short   Not_Found

	;Don't test for these because it takes too long
	;uncomment if want to use
	;Cmp     AL, SCROLL_LOCK_KS
	;JE      KB_Reset          ; CTRL-BREAK released so reset
	;Cmp     AL,C_KEYS
	;JE      KB_Reset          ; CTRL-C released so reset
	;Cmp     AL,Num_2_KeyS
	;JE      KB_Reset          ; CTRL-2 released so reset

Test_Alt:
	Test    AL,8              ; test to see if ALT Key pressed
	Pop     AX                ; restore AX
	JZ      Not_Found         ; if not pressed then quit

	Cmp     AL,Pg_Down
	JE      KB_Reset          ; ALT-PGDN pressed so reset

	;Don't test for this because it takes too long
	;uncomment if want to use
	;Cmp     AL,Pg_DownS      ; ALT-PGDN release so reset
	;JE      KB_Reset

Not_Found:
	PopF
	Pop     AX
	Pop     BX
	Pop     ES
	JMP     CS:[PreviousVector] ;continue on with prior interrupt path

	Assume  CS:@code, DS:@data, ES:nothing

EVEN
E0_Reset:                         ; tell bios that we got a 0E0h key
	Sub     BX,BX             ; this will let bios return
	Mov     ES,BX             ; extended keycodes
	Or      Byte Ptr ES:[0496h],010b ; set the Last Code was E0h bit

KB_Reset:
	In      AL,KB_Ctrl        ; reset keyboard
	Mov     AH,AL             ; so QBASIC does not catch on
	Or      AL,80h
	Out     KB_Ctrl,AL        ; reset keyboard
	Mov     AL,AH
	Out     KB_Ctrl,AL
        CLI                       ; turn off interrupts
	Mov     AL,EOI
        JMP     short $+2         ; adds a delay for AT's
	Out     Int_Ctrl_Port,AL  ; reset PIC with an end_of_interrupt
	PopF                      ; restore everything as it was
        Pop     AX
	Pop     BX
	Pop     ES
	IRET                      ; return to system
INTHANDLER      ENDP
END
