PROGRAM LineGraf;

CONST UpperLeftX = 54;
      UpperLeftY = 20;
      LowerRightX = 294;
      LowerRightY = 180;
      Color = 3;
      MaxAttendance = 400;
      Screenwidth = 40;
      LetterWidth = 8;
      MaxPeriod = 11;

TYPE  YearType = 74..84;

VAR   Title      : STRING[Screenwidth];
      Period     : YearType;
      Attendance : ARRAY[1..MaxPeriod] of INTEGER;
      LeftMargin, TopMargin,
      CharIntervalX, CharIntervalY,
      LabelIntervalX, LabelIntervalY,
      IntervalX, IntervalY,
      PixWidth, PixHeight : INTEGER;
      YesNo : BOOLEAN;

PROCEDURE GetTitle;

  BEGIN
    ClrScr;
    Writeln('Type the title for the graph');
    Writeln('   and press <RETURN>:');
    Readln(Title);
  END;

PROCEDURE GetData;

VAR Index : INTEGER;

  BEGIN
    Writeln('For each year shown below,');
    Writeln('type the value of the variable');
    Writeln('to be plotted, then press <RETURN>.');
    Writeln;
    Period := 74;
    FOR Index := 1 to MaxPeriod DO
      BEGIN
        Write(Period : 6,':  ');
        Readln(Attendance[Period]);
        Period := succ(Period)
      END;
  END;

PROCEDURE CenterTitle;

VAR  Len : INTEGER;

  BEGIN
    Len := Length(Title);
    GOTOXY(1,2);
    Write(Title : (Screenwidth + Len) DIV 2);
  END;

PROCEDURE ComputeIntervals;

VAR CharWidth, CharHeight : INTEGER;

  BEGIN
    LeftMargin := UpperLeftX DIV LetterWidth;
    TopMargin := UpperLeftY DIV LetterWidth;
    CharWidth := (LowerRightX DIV LetterWidth) - LeftMargin;
    CharHeight := (LowerRightY DIV LetterWidth) - TopMargin;
    LabelIntervalX := CharWidth DIV (MaxPeriod - 1);
    LabelIntervalY := CharHeight DIV 4;
    PixWidth := LowerRightX - UpperLeftX;
    PixHeight := LowerRightY - UpperLeftY;
    IntervalX := PixWidth DIV (MaxPeriod - 1);
    IntervalY := PixHeight DIV 4;
  END;

PROCEDURE LabelAxes;

VAR  XL, YL, I : INTEGER;

  BEGIN
    XL := 1;                                   {Start labeling ordinate}
    YL := TopMargin + 1;
    FOR I := 0 TO 4 DO
      BEGIN
        GOTOXY(XL,YL);
        Write((MaxAttendance - (100 * I)):LeftMargin);
        YL := YL + LabelIntervalY
      END;
    XL := LeftMargin;                          {Start labeling abcissa}
    YL := (LowerRightY DIV 8) + 2;
    Period := 74;
    FOR I := 1 TO MaxPeriod DO
      BEGIN
        GOTOXY(XL,YL);
        Write(Period : LabelIntervalX);
        Period := Succ(Period);
        XL := XL + LabelIntervalX
      END;
  END;

PROCEDURE DrawAxes;

  BEGIN
    Draw(UpperLeftX,UpperLeftY,UpperLeftX,LowerRightY,Color);
    Draw(UpperLeftX,LowerRightY,LowerRightX,LowerRightY,Color);
    Draw(LowerRightX,LowerRightY,LowerRightX,UpperLeftY,Color);
    Draw(LowerRightX,UpperLeftY,UpperLeftX,UpperLeftY,Color);
    LabelAxes;
  END;

PROCEDURE DrawScales;

VAR  X1, X2, Y, I : INTEGER;

  BEGIN
    X1 := UpperLeftX;
    X2 := UpperLeftX + PixWidth;
    Y := UpperLeftY + IntervalY;
    FOR I := 1 TO 3 DO
      BEGIN
        Draw(X1,Y,X2,Y,Color);
        Y := Y + IntervalY;
      END;
  END;

PROCEDURE PrepareGraphPaper;

  BEGIN
    GraphMode;
    Palette(1);
    CenterTitle;
    ComputeIntervals;
    DrawAxes;
    LabelAxes;
    DrawScales
  END;

FUNCTION ValueY(Attend : INTEGER) : INTEGER;

VAR  Ordinate : INTEGER;

  BEGIN
    Ordinate := Round((Attend / MaxAttendance) * PixHeight);
    ValueY := LowerRightY - Ordinate
  END;


PROCEDURE PlotGraph;

VAR  Attend, Period2, X1, X2, Y1, Y2, I : INTEGER;

  BEGIN
    Period := 74;
    Attend := Attendance[Period];
    X1 := UpperLeftX;
    Y1 := ValueY(Attend);
    FOR I := 1 TO (MaxPeriod-1) DO
      BEGIN
        X2 := X1 + IntervalX;
        Period2 := Succ(Period);
        Attend := Attendance[Period2];
        Y2 := ValueY(Attend);
        Draw(X1,Y1,X2,Y2,Color);
        X1 := X2;
        Y1 := Y2;
        Period := Period2
      END;
  END;

PROCEDURE RepeatOption;

VAR  Answer : CHAR;

  BEGIN
    TextMode(BW40);
    YesNo := False;
    Writeln('Do you wish to make');
    Write('   another graph? (Y/N) ');
    Readln(Answer);
    CASE Answer OF
      'Y' : YesNo := True;
      'y' : YesNo := True
    END;
  END;

BEGIN  { Main Program }
  REPEAT
    GetTitle;
    GetData;
    PrepareGraphPaper;
    PlotGraph;
    Readln;
    RepeatOption;
  UNTIL YesNo = False;
  TextMode(BW80)
END.