UNIT ListMgr;

INTERFACE
 USES IOSTUFF,CRT;
 TYPE
  List  = Array[1..2] of AnyStr;
  PList = ^List;
  ListParam  = Record
           ListHeading : AnyStr;    { Heading for list manager window  }
           ListWidth,               { Width of each list item in chars }
           ListLength,              { Total number of items in list    }
           ListX,ListY,             { Coord. of UL corner of window    }
           ListRows : Integer;      { Number of rows in window         }
           ListPtr : PList;         { Address in memory of List        }
         End;

FUNCTION ListPick (ListRec:ListParam; Default:Integer;
                   VAR PCh:Char;VAR Funct:Boolean): Integer;

IMPLEMENTATION
CONST
  Color1F = LightGray;  { Foreground color of window border }
  Color1B = Blue;       { Background color of window border }
  Color2F = Yellow;     { Foreground color of List items    }
  Color2B = Blue;       { Background color of List items    }
  Color3F = Black;      { Background color of picked list item }
  Color3B = LightGray;  { Background color of picked list item }
{====================================================================}
FUNCTION ListPick (ListRec:ListParam; Default:Integer;
                   VAR PCh:Char; VAR Funct:Boolean): Integer;
{  ListPick is a general purpose pop-up list manager that allows     }
{  the user to scroll the list and make a pick.  ListPick uses the   }
{  record defined by type ListParam as a parameter block. See example}
{  ListMgr.Dem to see how to build the parameter block.              }
{                                                                    }
{ ListParam  = Record                                                }
{          ListHeading : AnyStr;   Text Heading                      }
{          ListWidth,              Width of each list record         }
{          ListLength,             Total number in list              }
{          ListX,ListY,            Coordinates of list window        }
{          ListRows : Integer;     Number of rows in window          }
{          ListPtr : PList;        Address of List                   }
{        End;                                                        }
{                                                                    }
{  Default is the index of the starting pick to hilight              }
{                                                                    }
{  PCh is the character that the user hit to pick the list item      }
{      which is normally an enter -- but could be a D for delete.    }
{                                                                    }
{  Funct is a boolean variable that is true if the character is a    }
{       function key and false if it is a regular character.         }
{                                                                    }
VAR
  II              : Integer;    { General integer for looping, etc    }
  Top,Bot         : Integer;    { Number of top and bottom list items }
  Line, LastLine  : Integer;    { Current & last line of elevator char}
  PickNow         : Integer;    { Current list item picked            }


{====================================================================}
{ This Function writes list item Num in reverse video to highlight   }

 PROCEDURE WriteReverse(Num: Integer);

 VAR
   SaveColor : Byte;

 BEGIN
   SaveColor := TextAttr;         { Save current colors }
   SetColor(Color3F,Color3B);     { Set colors for reverse video }
   With ListRec do                { Write it }
     WriteSt(Trim(ListPtr^[Num],ListWidth),ListX+2,ListY+Num-Top+3);
   TextAttr := SaveColor;         { Restore color }
 END;

{====================================================================}
{ This function writes list item Num in regular video                }

 PROCEDURE WriteRegular(Num: Integer);

 BEGIN
   With ListRec do                 { Write it }
      WriteSt(Trim(ListPtr^[Num],ListWidth),ListX+2,ListY+Num-Top+3);
 END;

{====================================================================}
{ This function writes the little elevator character on the right of }
{ the window.  Also writes arrows to show continuation of list       }

 PROCEDURE WriteRHS;
 VAR
    RLine : Real;
 BEGIN
  With ListRec do Begin
                          { First figure out where to write the char }
                          { Use Real arith. to avoid truncate problems }


    RLine := (PickNow-1) * ListRows / ListLength;
    Line := ListY+3+Trunc(RLine);    { Calc actual line here }

    WriteCh('',ListX+ListWidth+4,Line);      { Write the elev. char }
    If (LastLine <> 0) and (LastLine <> Line) { Blank out the last char }
        then WriteCh(' ',ListX+ListWidth+4,LastLine);
    LastLine := Line;
                                           { Write the continuation arrows }
    If (Top > 1) and (Line <> ListY+3)
       then WriteCh(#24,ListX+ListWidth+4,ListY+3);
    If (Bot < ListLength) and (Line <> ListY+ListRows+2)
      then WriteCh(#25,ListX+ListWidth+4,ListY+ListRows+2);
  End;
END;

{====================================================================}
{ This procedure reads keystrokes and allows the user to scroll the  }
{ list.  User hits enter or escape to exit routine                   }

 PROCEDURE PlayWithPicks;

 CONST
   PageUp     = #73;  { Allowed Keystrokes }
   PageDown   = #81;
   UpArrow    = #72;
   DownArrow  = #80;
   HomeKey    = #71;
   EndKey     = #79;

 VAR
   PlayExit : Boolean;   { Switch for exiting procedure }
   FunctKey : Boolean;   { Test for function key chars  }
   Ch       : Char;      { Keystroke character }
   LastPick : Integer;   { The previous list item picked }
   LastTop  : Integer;   { The previous top list item }
   JJ       : Integer;
 BEGIN
   PlayExit := False;    { Initialization }
   LastPick := PickNow;
   LastTop  := 0;
   LastLine := 0;

   HideCursor;           { Hide the cursor }

   With ListRec do       { Start big keystroke reading loop }
   Repeat
                         { Test for error and exception conditions }
                         { Make sure all indexes are OK  }
     If (PickNow < 1) or (PickNow > ListLength) then Beep;
     If PickNow < 1 then PickNow := 1;
     If PickNow > ListLength then PickNow := ListLength;
     If PickNow < Top Then Top := PickNow;
     If Top < 1 then Top := 1;
     If PickNow > Top + ListRows -1 then Top := PickNow - ListRows +1;
     If Top + ListRows - 1 > ListLength then Top := ListLength - ListRows + 1;
     Bot := Top + ListRows - 1;
                      { If new top then rewrite whole list in window }
     If Top <> LastTop then
        For JJ := Top to Bot do WriteRegular(JJ)
                      { Restore the previous hilighted list item }
     else WriteRegular(LastPick);
     LastTop := Top;
     WriteReverse(PickNow);   { Hilight the current pick }
     WriteRHS;                { Update the elevator character }
     LastPick := PickNow;

                              { Read a key }
     Ch := ReadKey;
     If Ch <> #0 then FunctKey := False else
     Begin
       FunctKey := True;
       Ch := ReadKey;
     End;
                              { Handle function keys here }
     If FunctKey then Case Ch of
      UpArrow    : Dec(PickNow);  { Decrement current pick }
      DownArrow  : Inc(PickNow);  { Increment current pick }

      PageUp     : Begin          { Jump list by number of rows }
                                  { put in fancy stuff to beep at right time }
                    If ((Top = 1) and (PickNow = 1)) then Beep;
                    Top := Top - ListRows;
                    PickNow := PickNow - ListRows;
                    If Top < 1 then PickNow := 1;
                   End;
      PageDown   : Begin          { drop list by number of rows }
                    If ((Bot = ListLength) and (PickNow = ListLength))
                        then Beep;
                    Top := Top + ListRows;
                    PickNow := PickNow + ListRows;
                    If Top + ListRows > ListLength then PickNow := ListLength;
                   End;
      HomeKey    : Begin          { Go to top of list }
                    Top := 1;
                    PickNow := 1;
                   End;
      EndKey     : Begin          { Go to bottom of list }
                    Top := ListLength - ListRows + 1;
                    PickNow := ListLength;
                   End
                   Else Begin
                     PlayExit := True;
                     Funct := True;
                   End;
     End; {Case FunctKey}
                               { Handle non function keys in seperate case }
     If not FunctKey then Begin
       PlayExit := True;
       Funct := False;
     End;

  Until PlayExit;         { End of keystroke reading loop }
  PCh := Ch;              { Return character that caused exit }
  ShowCursor;             { Restore cursor }

 END;

{====================================================================}

BEGIN
  SetColor(Color1F,Color1B);  { Set colors for borders }
  With ListRec do Begin
                              { First do some grooming}
    If ListRows <= 0 then ListRows := 1;
    If ListLength <= 0 then ListLength := 1;
    If ListWidth <= 0 then ListWidth := 10;
    If (ListX < 1) or (ListX > 75) then ListX := 1;
    If (ListY < 1) or (ListY > 22) then ListY := 1;

    If ListRows > ListLength then ListRows := ListLength;
                              { Blank the window to be used }
    With ListRec do Window(ListX,ListY,ListX+ListWidth+5,ListY+ListRows+3);
    ClrScr;
    Window(1,1,80,25);
                              { Draw the boxes }
    Border(ListX,ListY,ListX+ListWidth+5,ListY+ListRows+3,'');
    WriteManyCh('',ListX+1,ListY+2,ListWidth+4);
    WriteCh('',ListX,ListY+2);
    WriteCh('',ListX+ListWidth+5,ListY+2);
    For II := ListY+3 to ListY+ListRows+2 do
        WriteCh('',ListX+ListWidth+3,II);
    WriteCh('',ListX+ListWidth+3,ListY+2);
    WriteCh('',ListX+ListWidth+3,ListY+ListRows+3);
    WriteSt(ListHeading,ListX+2,ListY+1);

    SetColor(Color2F,Color2B);   { Set colors for list items }
    If Default < 1 then Default := 1;
    If Default > ListLength then Default := ListLength;
    {Top := Default;}           { Initialize }
    {Bot := Top+ListRows-1;}

    PickNow := Default;
    PlayWithPicks;            { Do the hard work }
    ListPick := PickNow;      { Return the index of the list item picked }

  End;

END;

END.