





       Light Programming Macros 'LMS' Version 2.50  User's Manual
          Copyright (C)  PAF0600 Nifty Serve Satoru Hurukawa.

                         (revised) June 8, 1989



     NOTE:  The latest version of LMS is v.3.00.  This English
            manual for LMS v2.50 has been prepared by Kenjirou
            Okubo with comments and revisions by the author,
            Satoru Hurukawa.  This version of the manual is
            being used as it describes the routines used in
            LZSFX, SFX for LARC by K.Masuyama, PCVAN.

            Should any faulty statement be found in this manual,
            please notify:

                     CompuServe - K.Okubo 74100,2565
               or
                     GEnie      - K.Okubo



Description
===========

LMS is a set of macros to be used with the Optasm assembler, by SLR
Systems.  By using these macro sets, you may find it easier to express
certain conditional jumps and labelings.  Some routine codes are
expanded automatically with these macros.

               Optasm Copyright (C) 1988 by SLR Systems.

Usage:
======

No specific initialization process is necessary.  Include the accom-
panying file LMS.INC on the head of your source file.

LMS will try to generate high-speed/compact source codes.  You may wish
to select either speed or size to the other, by setting,

     $size = 0 (default)  for higher speed or
     $size = 1            for smaller size.

You may prefer to use the default setting since compactification of the
size is possible for limited environments.  Check this by testing and
comparing the codes generated.

You must specify conditional execution and block names as follows.  The
notation is used throughout this manual.  We abbreviate executional
conditions by Exec-condition.









Light Programming Macros User's Manual                            page 2


Exec-Condition:
===============

Relational operators such as 'e' for 'eq' or 'l' for 'le' can be used.

        e: equality operator, checks equal or not.
        le: less than or equal.

        1 <ax e 0>               ;; ax = 0
        2 <ax e 0 && <bx e 0>>   ;; ax = 0 and bx = 0

An exec-condition is always included in a pair of angle braces '<' and
'>'.  Please do not confuse them with IFB or IFNB directives.

These relational operators are similar to the conditions placed after
'j' for the jump commands 'je' or 'jc' in MASM.  Ordinarily, you need
lines of conditions and jump commands to make these conditonal jumps.
You can simplify conditional jumps with the following logical operators
included in braces:

        &&   and, logical and
        ||   or,  logical or.


        3 <ax e 0 && <bx e 0 || <cx e 0>>>

In stating complex conditions, you may have to use many braces, and you
have to be careful for the number of braces on left and those on the
right.  It looks similar to a LISP sentence if you use too many
conditions on a line.


        4  <al ae 81h && <al be 9fh || <al ae 0e0h && <al be 0fch>>>>


Note:  In LMS, logical and '&&' supercedes logical or '||'.
       Thus, in the following statement:


        5 <ax e 0 || <bx e 0 && <cx e 0>>>

does not mean;

        (ax=0 or bx=0) and (cx=0)

but

        (ax=0) or (bx=0 and cx=0).


There is no macro prepared in this version to let the logical OR
supercedes logical AND, so that you may have to write two separate
conditions sometimes.









Light Programming Macros User's Manual                            page 3


You may specify flags for an unary operators for the conditional jumps:

        6 <c>            ;; jump if cf is on
        7 <c || <e>>     ;; jump if cf or ef is on.


        6 stands for check c-flag for jump.



        8 <cxz>              ;; if cx=0
        9 <cxnz>             ;; if cx not equal 0

If the swtich $size=1 is on, <cxz> will generate smaller codes than
<ex e 0>


Case statements:

        10 <ax in <0,1,2,3>>                 ;; if ax=0,1,2 or 3
        11 <ax in <0,1,2> && <bx in <0,1>>>  ;; if ax=0,1 or 2, and
                                           ;; bx=0 or 1.


        12<ax e 0 || <ax e 1 || <ax e 2 || <ax e 3>>>>

is equivalent to 10.

For the operand with white space, you have to include words in a pair
of braces:

        13 <<byte ptr [si]> e 0>         ;; if byte pointer [si] = 0









Light Programming Macros User's Manual                            page 4


Block Names
===========

Programs in LMA consisist of Blocks.  A Block starts with some macro
directive called exec-condition and terminates with some auxiliarly
control phrase.  Commands are in between an exec-conditon and an aux-
iliarly control phrase.  In the following examples, you will always see
where a Block begins and to what extent the block continues and termi-
nates.

Names are specified by and arbitarary number of alphabets and numerals
lead by an alpahbetical character.

Block names are specified or declared after exec-conditions.

You will notice LMS uses symbols starting from '$','?' and '#'.  You may
have "confused codes" if you use names starting with these characters.

Following subdirectives have been set with .en directive.

          Optimize unconditionnal jumps                   (Y)
          Ignore Upper/Lower Case in String Compares      (Y)
          Auto-Expand Conditional Jumps                   (Y)


Light Programming Macros 'LMS' Version 2.50 will now descrive struc-
tures imbedded in the Optasm assembler.  There are control statements
and auxiliary control phrases.









Light Programming Macros User's Manual                            page 5


Control Statements.
==================


.?
==
  Synopsis.     .? <Exec-Condition> <commnad 1>[,<command 2>]

  function      If the exec-condition is true, then do command 1,
                else do commnad 2.

  Block         None.

  Example.
                .? <ax e 0> <inc bx>,<inc cx> ;; if ax=0 increment bx
                                                 else increment cx
.if
===
  Synopsis      .if <exec-condition>

  Function      If exec-condition is true then execute block.

  Block         With terminater .elseif^.nelseif^.else^.nelse^.endif

  Example       .if <ax ne 0>       ;; if ax not equal to 0 then
                                    ;;    do this
                .elseif <bx ne 0>   ;; <ax ne 0> false and this condition
                        inc bx      ;; do this when <ax=0> AND <bx ne 0>
                .else               ;; for all the rest
                        inc cx      ;; do this
                .endif              ;; end of the block started by .if



.elseif
=======
  Synopsis.     .elseif <exec-condition>.

  Function      Execute Block as long as exec-condition is true.
                You may inserte as many as you want between .ifc.else
                .endif.

  Block         to one of the follwoing
                .elseif  .nelseif  .else  .nelse  .endif

  Example       Refer to the example in .if.









Light Programming Macros User's Manual                            page 6


.nelseif
========
  Synopsis.     .nelseif <exec-condition>

  Function      Same as .elseif Function, use this immediately after
                branch. .elseif Next after a brach.

  Block         ends in .elseif .nelseif .else .nelse .endif.

  Example       .if <ax ne 0>
                        ret           ;; brach to main
                .nelseif <bx ne 0>    ;; immediately after branch
                        jmp label     ;; branch to label <ax=0> AND
                                      ;;   <bx ne 0>
                .nelse                ;; immediately after brach
                        inc ax        ;; do this
                .endif                ;; .if block ends.


.else
=====
  Synopsis.     .else

  Function      if exec-conditon is fault in  .if -- .elseif block
                then execute the following.

  Block         up to .endif.

  Example       .if refer to the example in .if statement.



.nelse
======

  Synopsis.     .nelse

  Function      .else Next after a brach.
                same as .else use this immediately after branch.

  Block         up to .endif

  Example       Refer to .nelseif









Light Programming Macros User's Manual                            page 7


.endif
======

  Synopsis.     .endif

  Function      Terminates Block started by one of the following:
                .if  .elseif .nelseif  .else  .nelse

  Block         started by .if,.elseif,.nelseif,.else and .nelse

  Example       Refer to the example in .if statement


.switch
=======

  Synopsis.     .switch <operand>[,Block name]

  Function      with .case statements inside a Block allows
                several evaluations of exec-conditions.
                If an operand is specifed then the operand is
                evaluted with .case conditions.

  Block         Block started with .switch terminates with .endsw.

  Example       .switch <ax>             ;; evaluate ax in what follows
                        .case <0,1,2>    ;; for ax=0,ax=1,ax=2
                                inc ax   ;; do this
                                .break   ;; and exit.
                        .case <3,4,5>    ;; for these values of ax
                                inc bx   ;; do this
                                .break   ;; and exit.
                        .default         ;; otherwise
                                inc cx   ;; do this
                .endsw                   ;; no more evaluation of ax.


.case
=====
  Synopsis.     .case <constant1[,constant2],,>
                There should be at least one constant inside angle
                braces.

  Function      if the operand specified after .switch takes one of the
                constants specified in the angle braces, execute block.

  Block         up to the '.endsw'

  Example       Refer to the example in .switch statement.









Light Programming Macros User's Manual                            page 8


.default
========
  Synopsis.     .default

  Function      In a block generated by .switch, executes block which
                follows unconditionally.  To suppress execution you
                have to insert .break and exit.

                .default is often there implicitly

  Block         up to .endsw

  Example       refer to the  example for .switch


.endsw
======
  Synopsis.     .endsw

  Function      mark end of a block with .switch , .case  or .default

  Block         begins from .switch or .case  or .default

  Example       Refer to the example for .switch


.while
======

Synopsis.       .while <exec-condition>[,Block Name]

Function        while exec-condition holds execute block.  May not
                execute at all if it does not hold.  An endless loop
                is generated if exec-condition is absent.

Block           up to .enddo.

Example         .while <ax ne 0>     ;; if ax=0 then jumps to .enddo
                        inc ax       ;; else do this
                .enddo               ;; mark end.









Light Programming Macros User's Manual                            page 9


.for
====

  Synopsis.     .for <<command1>,<exec-condition>,<command2>>[,Block Name]

  Function      executes command1 at least once, then while exec-
                condition holds true, executes the block which follows.
                It may not if it does not hold.  Command2 is executed
                before the re-evaluation of exec-condition.  If exec-
                condition is absent, then an endless loop is created.

  Block         up to .enddo.

  Example       .for <<xor ax,ax>,<ax l 50h>,<inc ax>>;; make ax=0 while
                                                      ;; ax is less than
                                                      ;; 50h
                        call func                     ;; do this.
                .enddo                                ;; for 50h times.


.enddo
======

  Synopsis.     .enddo

  Function      marks an end of a block with .while or .for

  Block         begins from .while or .for

  Example       refer to the examples for .while and .for


.do
===

  Synopsis.     .do [Block name]

  Function      Generate a control structure which executes the Block
                at least once.  Exec-condition will follow .until
                or .loop statement.


  Block         up to either .until or .loop.

  Example       .do                ;; structure begins
                        inc bx     ;; executes once then
                .until <ax e 0>    ;; check condition









Light Programming Macros User's Manual                           page 10


.until
======

  Synopsis.     .until <exec-condition>

  Function      while exec-condition is false, execute block which
                follows at least once when it is true.  When exec-
                condition is absent, this block is executed in an
                endless loop.

  Block         Begins with '.do' statement

  Example       Refer to the exmaple for .do


.loop
=====

  Synopsis.     .loop [e|z|ne|nz]

  Function      If exec-condition is true, execute Block as many
                times as specifed by the cx register.
                If cx=0 then Block is executed 0FFFFh+1 times.

  Block         .do is the start of the block.

  Example       mov cx,50h           ;; store the number of times
                .do                  ;; block starts from here
                        call func    ;; what should be done
                .loop                ;; end the execution here.









Light Programming Macros User's Manual                           page 11


Auxiliarly Control phrases.
===========================

.break
======

  Synopsis.     .break <exec-condition> [,Block name ]

  Function      Terminate execution if the block exits out of range.
                Unconditional if exec-condition is absent.  If the
                Block name is absent, then exits to the end of the
                present block.

  Reference     .casei.switch - .endswj
                .while  /.for - .enddo
                .do - .until  / .loop

  Example       .while <ax ne 0>           ;; exec conditon ax not = 0
                        dec ax             ;; what should be done
                        .break <ax e 50h>  ;; exit condition ax=50h
                .enddo                     ;; end of the block
                inc ax                     ;; exit point



.continue
=========
  Synopsis.     .continue <exec-condition> [,Block Name ]

  Function      Branch to the end of the specified Block.  Uncon-
                ditional Brach if exec-condition is absent.  If
                no block name is specified, then jump to the last
                statement of the present block.

  Reference     .while /.for-.enddo
                .do-.until / .loop

  Example       .while <ax ne 0>        ;; while ax not equal to zero
                        call func       ;; call subroutine func.
                        .continue <c>   ;; do as long as carry flag is
                                        ;;   off
                        inc ax          ;; what should be done and re-
                                        ;;   peat
                .enddo                  ;; if carry is on









Light Programming Macros User's Manual                           page 12


.nextcase
=========

  Synopsis.     .nextcase

  Function      In the .case control statement, place it at the end of
                the block.  Then the following .case statement will be
                unconditionally executed.  If you do not state .nextcase
                or .break either, then next exec-condition is evaluated
                in the follwing .case statement.  If .default is found,
                then it is executed anyway.

  Reference     .case

  Example       .switch <ax>              ;; check ax
                        .case <0,1,2>     ;; for ax=0,1,2
                                inc ax    ;; increment ax
                                .nextcase ;; force to do next case
                        .case <3,4,5>     ;; check incremented value
                                inc bx    ;; increment bx
                                .break    ;; end case
                        .default          ;; always do
                                inc cx    ;; command to be done all the
                                          ;;   time.
                .endsw                    ;; That's all for switch <ax>



------------------------------------------------------------------------
          4/Apr/1989 NIFTY-Serve PAF00600 by Satoru Hurukawa.
  Light Programming Macros 'lms' Version 2.50  Copyright (C) 1988 1989
------------------------------------------------------------------------

Changes made from versions 2.4x.

        *Bugs fixed for the use of
                * segment prefix
                * in operator
                * cxz when forward referenc is made

        * c macro omitted from this version

        *.l and zero definitions are not used any more

        * exec-condition evaluation in .loop is now placed at end

        * addition of $size symbol for optimization




                            - end -











-
x


         ----------------end-of-author's-documentation---------------

                        Software Library Information:

                   This disk copy provided as a service of

                        The Public (Software) Library

         We are not the authors of this program, nor are we associated
         with the author in any way other than as a distributor of the
         program in accordance with the author's terms of distribution.

         Please direct shareware payments and specific questions about
         this program to the author of the program, whose name appears
         elsewhere in  this documentation. If you have trouble getting
         in touch with the author,  we will do whatever we can to help
         you with your questions. All programs have been tested and do
         run.  To report problems,  please use the form that is in the
         file PROBLEM.DOC on many of our disks or in other written for-
         mat with screen printouts, if possible.  The P(s)L cannot de-
         bug programs over the telephone.

         Disks in the P(s)L are updated monthly, so if you did not get
         this disk  directly from the P(s)L,  you should be aware that
         the files in this set may no  longer be the current versions.

         For a copy of the latest monthly software library newsletter
         and a list of the 1,800+ disks in the library, call or write

                        The Public (Software) Library
                              P.O.Box 35705 - F
                           Houston, TX 77235-5705
                               (713) 665-7017

