;RM82TS8,16,24,32,40,48 ;
;updated 11/20/90

;============================================================================
;   Copyright (C) Copr. 1990 by Sidney J. Kelly
;           All Rights Reserved.
;           Sidney J. Kelly
;           150 Woodhaven Drive
;           Pittsburgh, PA 15228
;           home phone 412-561-0950 (7pm to 9:30pm EST)
;============================================================================

DOSSEG
.model medium
	public MMKEYBOARD
.code
	EXTRN   B_OnExit:FAR

	; This routine is called by QBASIC just before quitting and returning
	; to DOS.  32 different routines can be registered with B_OnExit.
	; B_OnExit is part of QBASIC run time library.  We use it here
	; to turn off cursor key emulation when program ends.

;===========================================================================
; DECLARE SUB MMKEYBOARD(Install%)
;
; Makes mouse appear as if keyboard keys were pushed
; Mouse motion  = Cursor keys
; Left button   = Enter key
; Right button  = ESC key
;
; Parameters:
;       Install% = 0 to uninstall
;               <> 0 to install
;       Note does not test for mouse existence,
;       nor does it restore old interrupt routine information
;       Use function 14 to do that.
; Returns:
;       Install% = 0 if no error
;                = 1 if error in B_OnExit
;
;===========================================================================

.code
EVEN
Install_Stat    db      0        ; keep track of current installation
F1st_Time       db      0        ; report if ever installed so as to
				 ; prevent multiple calls to B_OnExit

EVEN
MMKEYBOARD  PROC  FAR
	Push    BP
	Mov     BP,SP            ; create stack frame, make SP addressable
	Mov     BX,[BP+6]        ; get address of Install%
	Mov     AX,[BX]          ; get value from indirect address in BX
	OR      AX,AX            ; see if it is zero
	JZ      Uninstall        ; if zero then uninstall
	Cmp     Install_Stat,1   ; see if already installed
	JE      Quiter           ; if yes, then quit, work done

BeginInstall:
	Mov     AX, 33           ; make sure there is a mouse &
	Int     33h              ; turn off mouse cursor &
				 ; shut down any prior handler
	Cmp     AX,33            ; if AX = 33 then mouse error
	JE      Error_Exit       ; and quit with an error

	Cmp     CS:F1st_Time,1   ; see if registered with B_OnExit before
	JNE     Install          ; so only call B_OnExit once to register
	Mov     DX, offset KILLMOUSE
	Push    CS               ; Push far address of KILLMOUSE
	Push    DX               ; to register with the exit routine
	Call    B_OnExit         ; so that we don't hang machine when quit
	OR      AX,AX            ; Registered OK? Only 32 programs
				 ; can register with B_OnExit.
	JZ      Error_Exit       ; if AX = 0 then too many registered
				 ; routines and exit
	Mov     CS:F1st_Time,1   ; report for future calls that we have
				 ; registered with B_OnExit
Install:
	Mov     Install_Stat,1   ; mark that it is installed
	Mov     AX, SEG MSUB     ; install actual handler
	Mov     ES, AX           ; store segment of handler in ES
	Mov     AX, 12           ; function 12
	Mov     CX, 01011b       ; CX: the bit mask of actions to observe
	Mov     DX, OFFSET MSUB  ; handler at ES:DX
	Int     33h              ; call routine, returns no error
	Xor     AX,AX            ; indicate no errors
	Jmp     SHORT Quiter

Uninstall:
	Cmp     Install_Stat,0   ; see if already uninstalled
	JE      Uninstall1       ; yep, report o.k. & quit
	Mov     AX,33            ; Software reset is faster than function 0
	Int     33h              ; reset
	Mov     Install_Stat,0   ; keep track that have uninstalled

Uninstall1:
	Xor     AX,AX            ; indicate no errors
	Jmp     SHORT Quiter

Error_Exit:
	Mov     CS:F1st_Time,0   ; keep track that we did not register
	Mov     Install_Stat,0   ; keep track that not installed
	Mov     AX,1             ; return an error to Install%

Quiter:
	Mov     BX,[BP+6]        ; get address of Install%
	Mov     [BX],AX          ; put value in indirect address in BX
	Pop     BP               ; restore BP
	Ret     2                ; clean up stack  1 variable passed

MMKEYBOARD ENDP

;===========================================================================
; Called by B_OnExit to kill the interrupt driver
;===========================================================================

EVEN
KILLMOUSE       PROC    FAR
	Cmp     Install_Stat,0
	JE      KM1              ; not currently installed so quit
	Mov     AX,33
	Int     33h
	Mov     Install_Stat,0   ; report not installed
KM1:
	Ret
KILLMOUSE       ENDP


;===========================================================================
; The actual Mouse interrupt routine.  Patterned after MOUSEKEY.COM
; in PC Magazine. July, 1987 page 426.  Ziff Davis Publishing
; Major feature is jiggle prevention.  This keeps minor motion from sending
; a cursor key to the keyboard routines, and overflowing keyboard buffer.
; The side effect of this is that it causes reverse acceleration, the slower
; you move the mouse, the faster it sends keys because jiggle prevention is
; not active.
;===========================================================================

Bios_Data    Segment at 40h          ; this is merely a structure
	ORG 1Ah                      ; for access to BIOS ram memory
Buffer_Head     DW      ?            ; to allow fast manipulation of
Buffer_Tail     DW      ?            ; keyboard input buffer
	ORG 80h
Buffer_Start    DW      ?
Buffer_End      DW      ?
Bios_Data    ENDS

;-------Essential data------------------------------------------------------
Skip_num        equ     11          ; this number helps routine
			            ; ignore most cursor movement
				    ; so keyboard does not overflow
				    ; & so no key unless motion is
				    ; major movement over long period of time
				    ; BUT!!
				    ; gives mouse reverse acceleration!!

Skipper         DB      Skip_num             ; delay
vcount          DB      3                    ; vertical delay counter

EVEN
Horztl_flg      DW      0                    ; horizontal direction counters
Vertic_flg      DW      2                    ; vertical direction
Key_Code        DB      4dh, 4bh, 50h, 48h   ; codes for cursor keys
Prior_key       DB      0                    ; prevent jiggle, save prior key

;============================================================================
; The actual mouse interrupt.  Ignore mouse movement until it has happened
; 11 times.
;============================================================================

EVEN
MSUB    PROC    FAR
	Push    DS               ; save registers used

	Test    AX,2             ; left button pressed
	JNZ     Enter_key        ; yes, jump to ENTER key routine
	Test    AX,8             ; right button
	JNZ     ESC_key          ; yes, jump to ESC key

	Cmp     Skipper,0
	JE      @f
	Dec     Skipper
	Jmp     Finis
@@:
	Mov     Skipper,Skip_num ; reset counter
	Mov     AX,11            ; returns signed ints in CX & DX
	Int     33h              ; have to call this rather than
				 ; use values in SI, DI from interrupt
				 ; routine because of a documentation
				 ; error in early MS Mouse Programmers
				 ; Guide that has been parroted by
				 ; other mouse makers

	Mov     CS:Horztl_flg,0  ; initialize sign flags
	Mov     CS:Vertic_flg,2
	Xor     AL,AL            ; zero AL for extended keycode
	Cmp     CX,0             ; Is horizontal count positive
	JGE     Mouse1           ; yes, then branch
	Inc     CS:Horztl_flg    ; record negative condition
	NEG     CX               ; convert negative to positive number

Mouse1:
	Cmp     DX,0             ; Is vertical count positive
	JGE     Mouse2           ; yes, then branch
	Inc     CS:Vertic_flg    ; record negative condition
	NEG     DX               ; convert negative to positive number

Mouse2:
	Mov     BX,CS:Horztl_flg ; assume motion horizontal
	Cmp     CX,DX            ; was it horizontal motion?
	JAE     Mouse3           ; yes, then branch
	Mov     BX,CS:Vertic_flg ; correct assumption, show vertical
	DEC     CS:vcount        ; decrement delay counter
	JZ      Mouse3           ; continue if count is zero
	Jmp     Short Finis      ; exit if it is not

Mouse3:
	Mov     CS:vcount,3      ; reset delay counter
	Mov     AH,Key_Code[BX]  ; get keycode pressed

	;*******************JIGGLE prevention***************

	Cmp     Prior_key,AH     ; test if got a repeat direction key
	JE      @f               ; if yes continue
	Mov     Prior_key,AH     ; else save new direction &
	Jmp     Short Finis      ; quit
@@:
	Jmp     Short Insert     ; insert in keyboard buffer

ESC_key:                         ; left button press
	Mov     AX,011Bh         ; note that it is an ESC key
	Jmp     SHORT Insert     ; insert in keyboard buffer

Enter_key:
	Mov     AX,1C0Dh         ; right button press, note it is ENTER key

Insert:                          ; manages keyboard buffer
	Mov     BX,Bios_Data     ; point DS to Bios_Data segment
	Mov     DS,BX

	Assume  DS:Bios_Data

	CLI                      ; disable keyboard interrupts
	Mov     BX,Buffer_Tail   ; get tail address

	Mov     DX,BX            ; store in DX
	ADD     DX,2             ; calc next buffer position
	CMP     DX,Buffer_End    ; did we overshoot end
	JNE     Insert1          ; no, then continue
	Mov     DX,Buffer_Start  ; yes then wrap around

Insert1:
	CMP     DX,Buffer_Head   ; is buffer full?
	JE      Insert2          ; yes, so end now
	Mov     [BX],AX          ; insert keycode
	Mov     BX,DX            ; advance tail
	Mov     Buffer_Tail,BX   ; record new position

Insert2:
	STI                      ; allow hardware interrupts again

Finis:
	Pop     DS
	Ret
MSUB    ENDP
END
