; updated 10/17/91

;=======================================================================
;   Copyright (C) Copr. 1990, 1991 by Sidney J. Kelly
;           All Rights Reserved.
;           Sidney J. Kelly
;           150 Woodhaven Drive
;           Pittsburgh, PA 15228
;           home phone 412-561-0950 (7pm to 9:30pm EST)
;=======================================================================
;RM82TS8,16,24,32,40,48

;==============================================================================
;A selection of mouse driver routines for MicroSoft Mouse
;Primarily designed for text mode use w/ a standard 80 x 25 mode screen
;==============================================================================

DOSSEG
.MODEL MEDIUM

public  MMCURSON, MMCURSOFF, MMLEFTBUT, MMRIGHTBUT, MMLOCATE, MMPRESSLOCATE
public  MMRELEASELOCATE, MMLIMIT, MMSETLOCATION 
public  MMORKEY, MMTCURSOR, CLRMOUSEKEY, MMSPEED

.DATA
	EXTRN  B$HAVEMOUSE:BYTE  ; = 1 if mouse found
	;If mouse cursor visible, then B$MOUSEON <> 0
	;If mouse cursor not visible then B$MOUSEON = 0
	EXTRN  B$MOUSEON:BYTE    ; = 1 if mouse cursor on

.CODE

; Please do not remove
Copyright       DB    13,10,'Copyright Copr. (C) 1990, 1991 Sidney J. Kelly',13,10
Copyright1      DB    'All Rights Reserved',13,10,26

;===============================================================
; DECLARE SUB MMCURSON()
; Description:  Does test for Mouse existence.
;               Merely turns existing Mouse Cursor On
;               Keeps its own record of the status to stabilize
;               status and prevent multiple calls from hiding
;               cursor forever by preventing proper use of
;               Show Cursor.  Otherwise multiple calls to cursor off
;               would have be followed by equally many calls to cursor
;               on, unless the mouse were reset.
;===============================================================

EVEN
MMCURSON PROC FAR
	Xor     AL,AL             ; make AL = 0
	Mov     B$MOUSEON,AL      ;assume no mouse
	Cmp     B$HAVEMOUSE,AL    ;check for mouse
	Je      @f                ;if 0 then no mouse
	Mov     AX,1
	Int     33h
	Mov     B$MOUSEON,AL      ;note cursor on
@@:
	Ret
MMCURSON  Endp

;===============================================================
; DECLARE SUB MMCURSOFF()
; Description:  Does test for Mouse existence.
;               Turns existing Mouse Cursor OFF.
;               Keeps its own status to prevent multiple calls from
;               hiding cursor forever by preventing proper use of
;               Show Cursor
;===============================================================

EVEN
MMCURSOFF PROC FAR
	Xor     AL,AL                 ; make AL = 0
	Cmp     B$HAVEMOUSE,AL        ; check for mouse
	Je      @f                    ; if 0 then no mouse
	Cmp     B$MOUSEON,AL          ; turn cursor off only if
	Je      @f                    ; not already off
	Mov     AX,2
	Int     33h
	Xor     AL,AL                 ; make AL = 0
	Mov     B$MOUSEON,AL          ; record MOUSE now hidden
@@:
	Ret
MMCURSOFF  Endp

;===============================================================
; DECLARE FUNCTION MMLEFTBUT%()
; Description:  Does test for Mouse existence.
;               Returns current status of left button
;               0 = no button or error
;               1 = left button pressed
;===============================================================

EVEN
MMLEFTBUT PROC FAR
	Xor     AX,AX             ; assume no button pressed
	Cmp     B$HAVEMOUSE,AL    ; check for mouse
	Je      @f                ; no, so quit with no button
	Mov     AX,3
	Int     33h
	Xor     AX,AX
	Cmp     BX,3
	JA      @f
	Mov     AX,BX
	And     AX,1
@@:
	Ret
MMLEFTBUT Endp

;===============================================================
; DECLARE FUNCTION MMRIGHTBUT%()
; Description:  Does test for Mouse existence.
;               Returns current status of right button
;               0 = no button or error
;               1 = right button pressed
;===============================================================

EVEN
MMRIGHTBUT PROC FAR
	Xor     AX,AX
	Cmp     B$HAVEMOUSE,AL    ;check for mouse
	Je      @f                ;quit with no button
	Mov     AX,3
	Int     33h
	Xor     AX,AX
	Cmp     BX,3
	JA      @F
	Mov     AX,BX
	Shr     AX,1
@@:
	Ret
MMRIGHTBUT Endp

;===============================================================
; DECLARE SUB MMLOCATE(Row%, Col%)
; Description:  Does test for Mouse existence.
;               Returns location of Mouse on text screen only.
;               Screen 0,0,0 mode 80 x 25, 80 x 43, 80 x 50 display
;===============================================================

EVEN
MMLOCATE PROC FAR
	Push    BP
	Mov     BP,SP          ;create stack frame and make SP addressable
	Xor     AL,AL
	Cmp     B$HAVEMOUSE,AL ;check for mouse, see if = AL(0)
	Je      MML_Err        ;no mouse so return with 0,0
	Mov     AX,3
	Int     33h
	Mov     AX,CX          ;store CX in AX so we can use CX for other purposes
	Mov     CX,3
	Shr     AX,CL          ;divide by 8 by shifting right 3 times
	Inc     AX             ;make it one biased
	Shr     DX,CL          ;DX has Row% value; AX has Coll% value
	Inc     DX             ;make it one biased
MML_Err1:
	Mov     BX,[BP+6]      ;get address of Col%
	Mov     [BX],AX        ;store value from AX in indirect address in BX
	Mov     BX,[BP+8]      ;get address of Row%
	Mov     [BX],DX        ;store value from DX in indirect address in BX
	Pop     BP             ;restore BP
	Ret     4              ;clean up stack by eliminating 2 variables passed
MML_Err:
	Xor     AX,AX          ;return 0,0 for no mouse
	Mov     DX,AX
	Jmp     Short MML_Err1
MMLOCATE  Endp

;===============================================================
; DECLARE SUB MMPRESSLOCATE(Button%,Times%,Row%, Coll%)
; Input  Button% 0 = Left Button
;        Button% 1 = Right Button
; Returns
;        Times% = no. of times pressed since function last called
;        Row%   = Row% location at last press
;        Coll%  = Column location at last press
; Description:  Does test for Mouse existence.
;               Returns location of Mouse on text screen only.
;               Screen 0,0,0 mode 80 x 25, 80 x 43, 80 x 50 display
;===============================================================

EVEN
MMPRESSLOCATE PROC FAR
	Push    BP
	Mov     BP,SP          ;create stack frame and make SP addressable
	Xor     AL,AL          ; make AL = 0
	Cmp     B$HAVEMOUSE,AL ;check for mouse
	Je      MMP_Err        ;no mouse so return with 0,0,0
	Mov     BX,[BP+12]
	Mov     BX,[BX]        ;get which button to test for
	And     BX,1           ;make sure value either 0 or l
	Mov     AX,5
	Int     33h
MMP_Err1:
	Mov     AX,BX          ;store BX in AX so we can use BX for other purposes
	Mov     BX,[BP+10]
	Mov     [BX],AX        ;store Times% in variable Times%
	Mov     AX,CX          ;store CX in AX so we can use CX for other purposes
	Mov     CX,3
	Shr     AX,CL          ;divide by 8 by shifting right 3 times
	Inc     AX             ;make it one biased
	Shr     DX,CL          ;AX has Col% value; DX has Row% value
	Inc     DX             ;make it one biased
	Mov     BX,[BP+6]      ;get address of Col%
	Mov     [BX],AX        ;store value from AX in indirect address in BX
	Mov     BX,[BP+8]      ;get address of Row%
	Mov     [BX],DX        ;store value from DX in indirect address in BX
	Pop     BP             ;restore BP
	Ret     8              ;clean up stack by eliminating 4 variables passed
MMP_Err:
	Xor     AX,AX          ;return all errors
	Mov     BX,AX
	Mov     CX,AX
	Mov     DX,AX
	Jmp     Short MMP_Err1
MMPRESSLOCATE  Endp

;===============================================================
; DECLARE SUB MMRELEASELOCATE(Button%,Times%,Row%, Coll%)
; Input  Button% 0 = Left Button
;        Button% 1 = Right Button
; Returns
;        Times% = no. of times pressed since function last called
;        Row%   = Row% location at last press
;        Coll%  = Column location at last press
; Description:  Does test for Mouse existence.
;               Returns location of Mouse on text screen only.
;               Screen 0,0,0 mode 80 x 25, 80 x 43, 80 x 50 display
;===============================================================

EVEN
MMRELEASELOCATE PROC FAR
	Push    BP
	Mov     BP,SP          ;create stack frame and make SP addressable
	Xor     AL,AL          ; make AL = 0
	Cmp     B$HAVEMOUSE,AL ;check for mouse
	Je      MMR_Err        ;no mouse so return with 0,0,0
	Mov     BX,[BP+12]
	Mov     BX,[BX]        ;get which button to test for
	And     BX,1           ;make sure value either 0 or 1
	Mov     AX,6
	Int     33h
MMR_Err1:
	Mov     AX,BX          ;store BX in AX so we can use BX for other purposes
	Mov     BX,[BP+10]
	Mov     [BX],AX        ;store Times% in variable Times%
	Mov     AX,CX          ;store CX in AX so we can use CX for other purposes
	Mov     CX,3
	Shr     AX,CL          ;divide by 8 by shifting right 3 times
	Inc     AX             ;make it one biased
	Shr     DX,CL          ;AX has Col% value; DX has Row% value
	Inc     DX             ;make it one biased
	Mov     BX,[BP+6]      ;get address of Col%
	Mov     [BX],AX        ;store value from AX in indirect address in BX
	Mov     BX,[BP+8]      ;get address of Row%
	Mov     [BX],DX        ;store value from DX in indirect address in BX
	Pop     BP             ;restore BP
	Ret     8              ;clean up stack by eliminating 4 variables passed
MMR_Err:
	Xor     AX,AX          ;return all errors
	Mov     BX,AX
	Mov     CX,AX
	Mov     DX,AX
	Jmp     Short MMR_Err1
MMRELEASELOCATE  Endp

;===============================================================
; DECLARE SUB MMLIMIT(BYVAL ULR%,BYVAL ULC%, BYVAL LRR%, BYVAL LRC%)
;
; Purpose:
; Restricts area in which mouse can move.
;
; Input  ULR% = Upper left row
;        ULC% = Upper left column
;        LRR% = Lower right row
;        LRC% = Lower right column
; Returns
;        Nothing
; Description:  Does test for Mouse existence.
;               limits range of mouse on text screen only.
;               if ULR=LRR or ULC=LRC then exit without doing anything
;               Screen 0,0,0 mode 80 x 25, 80 x 43 or 80 x 50 display
;       Mirrors functions 7 & 8
;===============================================================

EVEN
MMLIMIT PROC FAR
	Push    BP
	Mov     BP,SP          ;create stack frame and make SP addressable
	Xor     AL,AL
	Cmp     B$HAVEMOUSE,AL ;check for mouse
	Je      ender
	Mov     AX,[BP+12]     ;get ULR% NOTE BYVAL
	OR      AX,AX          ;make sure not zero
	JZ      ender          ;quit if zero
	Dec     AX             ;subtract 1
	SHL     AX,1
	SHL     AX,1
	SHL     AX,1           ;multiply times 8

	Mov     DX,[BP+8]      ;get LRR% NOTE BYVAL
	OR      DX,DX          ;make sure not zero
	JZ      ender          ;quit if zero
	Dec     DX             ;subtract 1
	SHL     DX,1
	SHL     DX,1
	SHL     DX,1           ;multiply times 8
	Cmp     AX,DX          ;if value is the same
	JE      ender          ;quit if equal

	Mov     CX,AX          ;store AX in CX
	Mov     AX,8
	Int     33h

	Mov     AX,[BP+10]     ;get ULC% NOTE BYVAL
	OR      AX,AX          ;make sure not zero
	JZ      ender          ;quit if zero
	Dec     AX             ;subtract 1
	SHL     AX,1
	SHL     AX,1
	SHL     AX,1           ;multiply times 8

	Mov     DX,[BP+6]      ;get LRC% NOTE BYVAL
	OR      DX,DX          ;make sure not zero
	JZ      ender          ;quit if zero
	Dec     DX             ;subtract 1
	SHL     DX,1
	SHL     DX,1
	Shl     DX,1           ;multiply times 8
	Cmp     AX,DX          ;if value is the same
	JE      ender          ;quit

	Mov     CX,AX          ;store AX in CX
	Mov     AX,7
	Int     33h
ender:
	Pop     BP             ;restore BP
	Ret     8              ;clean up stack by eliminating 4 variables passed
MMLIMIT  Endp

;==========================================================================
; DECLARE SUB MMSETLOCATION(BYVAL Row%, BYVAL Col%)
;
; Purpose:
; Move Mouse to fixed text row location.
;
; Input  Row% = Text Mode Row%
;        Col% = Text Mode Col%
; Returns
;        nothing, sub fails if Row or Col% = 0
; Description:  Does test for Mouse existence.
;===========================================================================

EVEN
MMSETLOCATION PROC FAR
	Push    BP
	Mov     BP,SP          ;create stack frame and make SP addressable
	Xor     AL,AL
	Cmp     B$HAVEMOUSE,AL ;check for mouse
	JE      Halt_pgm       ;if none, then quit
	Mov     DX,[BP+8]      ;get row% NOTE BYVAL
	OR      DX,DX          ;quit if zero
	JZ      Halt_pgm
	Dec     DX             ;subtract 1
	SHL     DX,1           ;multiply by 2
	SHL     DX,1           ;multiply by 4
	SHL     DX,1           ;multiply by 8  (faster than using CL)

	Mov     CX,[BP+6]      ;get column% NOTE BYVAL
	JCXZ    Halt_pgm       ;if CX = 0 then quit
	Dec     CX             ;subtract 1
	SHL     CX,1           ;multiply by 2
	SHL     CX,1           ;multiply by 4
	SHL     CX,1           ;multiply by 8  (faster than using CL)

	Mov     AX,4           ;select function 4
	Int     33h            ;DX has row, CX has column
Halt_pgm:
	Pop     BP             ;restore BP
	Ret     4              ;clean up stack by eliminating 2 variable passed
MMSETLOCATION Endp

;===============================================================
; DECLARE FUNCTION MMORKEY% ()
; Description:
;		Tests if a key has been pressed, if no, then tests
;               if a mouse key has been pressed.  To use in a loop
;               the keyboard must be cleared or the function will
;               respond to "stale" keys.  Function does clear
;               the key from the keyboard buffer if it finds it.
;
; Returns:
;       -1 if a MOUSE press or key press occurred
;        0 if nothing found
;
; Theory:
;        This makes the loop run inside QBasic where it can be
;        interrupted for ON ... GOSUB's etc.
;===============================================================

EVEN
MMORKEY PROC    FAR
Begin_Loop:
	Mov     AH,1            ; sets Zero flag if no character is
	Int     16h             ; pending
	JZ      Try_Mouse
	Xor     AH,AH           ; clear key from buffer
	Int     16h             ; so buffer will not overflow
	Jmp	Short Have_Key
Try_Mouse:
	Xor     AL,AL           ; make AL = 0
	Cmp     B$HAVEMOUSE,AL  ; check for mouse
	JE      No_Key          ; is not installed if value = 0
Test_Button:
	Mov     AX,5            ; get button press information
	Xor     BX,BX           ; was left button pressed since last test?
	Int	33h
	Or      BX,BX           ; any presses?
	JNZ     Have_Key        ; yep, report it and quit
	Mov     AX,5            ; get button press information
	Mov     BX,1            ; was right button pressed since last test?
	Int     33h
	Or      BX,BX           ; any presses?
	JNZ     Have_Key        ; yep, report it and quit
No_Key:                         ; nope, so quit
	Xor     AX,AX           ; report no key or button found & quit
	Jmp     Short MM_Finis
Have_Key:
	Mov     AX,-1           ; report a key or mouse button found
MM_Finis:
	Ret
MMORKEY	ENDP

;===============================================================
; DECLARE SUB MMTCURSOR (BYVAL Char%)
; CALL MMTCURSOR(Char%)
; Description:  Does test for Mouse existence.
;       Allows user to change text cursor shape, but not color.
; Range of input: 0 to 255.  Color is intense white on black
; Parallel function 10
;===============================================================

EVEN
MMTCURSOR  PROC FAR
	Push    BP
	Mov     BP,SP           ; prepare stack frame
	Xor     AL,AL           ; make AL = 0
	Cmp     B$HAVEMOUSE,AL  ; check for mouse
	Je      @F              ; no mouse so quit
	Mov     AX,10           ; set function
	Xor     BX,BX           ; use software cursor
	Xor     CX,CX           ; clear original char & color
	Mov     DX,[BP+6]       ; get character (BYVAL)
	Mov     DH,00001111b    ; make intense on black w/ new char
	Int     33h
@@:
	Pop     BP              ; restore registers
	Ret     2               ; remove one parameter
MMTCURSOR  ENDP

;===============================================================
; DECLARE SUB CLRMOUSEKEY
; Purpose:  Clears keyboard, & resets all mouse counters
;      This gives us the freedom of clear buffers for mouse button
;      presses and freedom from stale keystrokes.
;
; Returns:  Nothing
;===============================================================

EVEN
CLRMOUSEKEY  PROC FAR
; Clear the keyboard first
	Xor     AX, AX
	Mov     ES, AX
	Assume  ES:NOTHING
; Mouse section
	Xor     AL,AL
	Cmp     B$HAVEMOUSE,AL        ; check for mouse (is it zero?)
	Je      ClearKBD              ; no mouse so quit
	Mov     AX,5                  ; reset mouse press left button
	Xor     BX,BX                 ; BX = 0
	Int     33h
	Mov     AX,5                  ; reset mouse press right button
	Mov     BX,1                  ; BX = 1
	Int     33h
	Mov     AX,6                  ; reset mouse release left button
	Xor     BX,BX                 ; BX = 0
	Int     33h
	Mov     AX,6                  ; reset mouse release right button
	Mov     BX,1                  ; BX = 1
	Int     33h
	Mov     AX,11                 ; reset mouse motion counter
	Int     33h
ClearKBD:                             ; clear keyboard
	CLI                           ; clear interrupts for a moment
	Mov     AL, ES:[041Ch]        ; Refer to actual addresses
	Mov     ES:[041Ah], AL        ; using ES: form
	STI                           ; allow interrupts
	Ret
CLRMOUSEKEY  ENDP

;===============================================================
; DECLARE SUB MMSPEED (BYVAL Value%)
; Input:
;       A number between 0 & 3
;       0 = slow, 1 = default, 2 = fast, 3 = maximum fast
;===============================================================

EVEN
MMSPEED PROC  FAR
	Push    BP                ;allow stack addressing
	Mov     BP,SP
	Xor     AL,AL
	Cmp     B$HAVEMOUSE,AL    ;check for mouse (is it 0?)
	Je      @F                ;if 0 then no mouse
	Mov     AX,[BP+6]         ;note BYVAL, get value
	And     AX,3              ;keep within range
	Inc     AX                ;make number between 1 & 4
	Mov     BL,25             ;multiply by 25
	Mul     BL                ;make value 25, 50, 75 or 100
	Mov     BX,AX             ;store value in BX
	Mov     CX,AX             ;in CX too
	Mov     AX,26             ;select function 26
	Mov     DX,50             ;use default double speed value
	Int     33h
@@:
	Pop     BP                ;reset BP
	Ret     2                 ;remove variable pushed on stack
MMSPEED ENDP
End

