;RM82TS8,16,24,32,40,48
; updated 12/10/90

;============================================================================
;   Copyright (C) Copr. 1990 by Sidney J. Kelly
;           All Rights Reserved.
;           Sidney J. Kelly
;           150 Woodhaven Drive
;           Pittsburgh, PA 15228
;           home phone 412-561-0950 (7pm to 9:30pm EST)
;============================================================================

;==============================================================================
;A selection of mouse driver routines for MicroSoft Mouse
;Primarily designed for text mode use w/ a standard 80 x 25 mode screen
;==============================================================================

DOSSEG
.model medium
.code
public  MMCURSON, MMCURSOFF, MMLEFTBUT, MMRIGHTBUT, MMLOCATE, MMPRESSLOCATE, MMRELEASELOCATE, MMLIMIT, MMDOUBLECLICK, MMSETLOCATION, MMOVED, MMVISIBLE, MMINVIZ, MMBUFSIZE, MMRESET, MMORKEY

.code

Hide_Stat       db  1        ; Assume Mouse cursor hidden

;If mouse cursor visible, then Hide_Stat = 0
;If mouse cursor not visible then Hide_Stat <> 0

;===============================================================
; DECLARE SUB MMCURSON()
; Description:  Does not test for Mouse existence.
;               Merely turns existing Mouse Cursor On
;               Keeps its own status to prevent multiple calls from
;               hiding cursor forever by preventing proper use of
;               Show Cursor.  Otherwise multiple calls to cursor off
;               would have be followed by equally many calls to cursor
;               on, unless the mouse were reset.
;===============================================================

EVEN
MMCURSON  Proc Far
	Mov     AX,1
	Int     33h
	Xor     AL,AL
	Mov     Hide_Stat,AL      ;turn hidden off
	Ret
MMCURSON  Endp

;===============================================================
; DECLARE SUB MMCURSOFF()
; Description:  Does not test for Mouse existence.
;               Merely turns existing Mouse Cursor OFF.
;               Keeps its own status to prevent multiple calls from
;               hiding cursor forever by preventing proper use of
;               Show Cursor
;===============================================================

EVEN
MMCURSOFF  Proc  Far
	Cmp     Hide_Stat,0		; turn cursor off only if
	Jne     @f			; not already off
	Mov     AX,2
	Mov     Hide_Stat,AL            ; record hidden on
	Int     33h
@@:
	Ret
MMCURSOFF  Endp

;===============================================================
; DECLARE FUNCTION MMLEFTBUT%()
; Description:  Does not test for Mouse existence.
;               Returns current status of left button
;               0 = no button or error
;               1 = left button pressed
;===============================================================

EVEN
MMLEFTBUT Proc  Far
	Mov     AX,3
	Int     33h
	Xor     AX,AX
	CMP     BX,3
	JA      @F
	Mov     AX,BX
	And     AX,1
@@:
	Ret
MMLEFTBUT Endp

;===============================================================
; DECLARE FUNCTION MMRIGHTBUT%()
; Description:  Does not test for Mouse existence.
;               Returns current status of right button
;               0 = no button or error
;               1 = right button pressed
;===============================================================

EVEN
MMRIGHTBUT Proc Far
	Mov     AX,3
	Int     33h
	Xor     AX,AX
	CMP     BX,3
	JA      @F
	Mov     AX,BX
	Shr     AX,1
@@:
	Ret
MMRIGHTBUT Endp

;===============================================================
; DECLARE SUB MMLOCATE(Row%, Col%)
; Description:  Does not test for Mouse existence.
;               Returns location of Mouse on text screen only.
;               Screen 0,0,0 mode 80 x 25, 80 x 43, 80 x 50 display
;===============================================================

EVEN
MMLOCATE  Proc    Far
	Push    BP
	Mov     BP,SP          ;create stack frame and make SP addressable
	Mov     AX,3
	Int     33h
	Mov     AX,CX          ;store CX in AX so we can use CX for other purposes
	Mov     CX,3
	Shr     AX,CL          ;divide by 8 by shifting right 3 times
	Inc     AX             ;make it one biased
	Shr     DX,CL          ;DX has Row% value; AX has Coll% value
	Inc     DX             ;make it one biased
	Mov     BX,[BP+6]      ;get address of Col%
	Mov     [BX],AX        ;store value from AX in indirect address in BX
	Mov     BX,[BP+8]      ;get address of Row%
	Mov     [BX],DX        ;store value from DX in indirect address in BX
	Pop     BP             ;restore BP
	Ret     4              ;clean up stack by eliminating 2 variables passed
MMLOCATE  Endp

;===============================================================
; DECLARE SUB MMPRESSLOCATE(Button%,Times%,Row%, Coll%)
; Input  Button% 0 = Left Button
;        Button% 1 = Right Button
; Returns
;        Times% = no. of times pressed since function last called
;        Row%   = Row% location at last press
;        Coll%  = Column location at last press
; Description:  Does not test for Mouse existence.
;               Returns location of Mouse on text screen only.
;               Screen 0,0,0 mode 80 x 25, 80 x 43, 80 x 50 display
;===============================================================

EVEN
MMPRESSLOCATE  Proc    Far
	Push    BP
	Mov     BP,SP          ;create stack frame and make SP addressable
	Mov     BX,[BP+12]
	Mov     BX,[BX]        ;get which button to test for
	And     BX,1           ;make sure value either 0 or l
	Mov     AX,5
	Int     33h
	Mov     AX,BX          ;store BX in AX so we can use BX for other purposes
	Mov     BX,[BP+10]
	Mov     [BX],AX        ;store Times% in variable Times%
	Mov     AX,CX          ;store CX in AX so we can use CX for other purposes
	Mov     CX,3
	Shr     AX,CL          ;divide by 8 by shifting right 3 times
	Inc     AX             ;make it one biased
	Shr     DX,CL          ;AX has Col% value; DX has Row% value
	Inc     DX             ;make it one biased
	Mov     BX,[BP+6]      ;get address of Col%
	Mov     [BX],AX        ;store value from AX in indirect address in BX
	Mov     BX,[BP+8]      ;get address of Row%
	Mov     [BX],DX        ;store value from DX in indirect address in BX
	Pop     BP             ;restore BP
	Ret     8              ;clean up stack by eliminating 4 variables passed
MMPRESSLOCATE  Endp

;===============================================================
; DECLARE SUB MMRELEASELOCATE(Button%,Times%,Row%, Coll%)
; Input  Button% 0 = Left Button
;        Button% 1 = Right Button
; Returns
;        Times% = no. of times pressed since function last called
;        Row%   = Row% location at last press
;        Coll%  = Column location at last press
; Description:  Does not test for Mouse existence.
;               Returns location of Mouse on text screen only.
;               Screen 0,0,0 mode 80 x 25, 80 x 43, 80 x 50 display
;===============================================================

EVEN
MMRELEASELOCATE  Proc    Far
	Push    BP
	Mov     BP,SP          ;create stack frame and make SP addressable
	Mov     BX,[BP+12]
	Mov     BX,[BX]        ;get which button to test for
	And     BX,1           ;make sure value either 0 or 1
	Mov     AX,6
	Int     33h
	Mov     AX,BX          ;store BX in AX so we can use BX for other purposes
	Mov     BX,[BP+10]
	Mov     [BX],AX        ;store Times% in variable Times%
	Mov     AX,CX          ;store CX in AX so we can use CX for other purposes
	Mov     CX,3
	Shr     AX,CL          ;divide by 8 by shifting right 3 times
	Inc     AX             ;make it one biased
	Shr     DX,CL          ;AX has Col% value; DX has Row% value
	Inc     DX             ;make it one biased
	Mov     BX,[BP+6]      ;get address of Col%
	Mov     [BX],AX        ;store value from AX in indirect address in BX
	Mov     BX,[BP+8]      ;get address of Row%
	Mov     [BX],DX        ;store value from DX in indirect address in BX
	Pop     BP             ;restore BP
	Ret     8              ;clean up stack by eliminating 4 variables passed
MMRELEASELOCATE  Endp

;===============================================================
; DECLARE SUB MMLIMIT(BYVAL ULR%,BYVAL ULC%, BYVAL LRR%, BYVAL LRC%)
;
; Purpose:
; Restricts area in which mouse can move.
;
; Input  ULR% = Upper left row
;        ULC% = Upper left column
;        LRR% = Lower right row
;        LRC% = Lower right column
; Returns
;        Nothing
; Description:  Does not test for Mouse existence.
;               limits range of mouse on text screen only.
;               if ULR=LRR or ULC=LRC then exit without doing anything
;               Screen 0,0,0 mode 80 x 25, 80 x 43 or 80 x 50 display
;       Mirrors functions 7 & 8
;===============================================================

EVEN
MMLIMIT  Proc    Far
	Push    BP
	Mov     BP,SP          ;create stack frame and make SP addressable

	Mov     AX,[BP+12]     ;get ULR% NOTE BYVAL
	OR      AX,AX          ;make sure not zero
	JZ      ender          ;quit if zero
	Dec     AX             ;subtract 1
	SHL     AX,1
	SHL     AX,1
	SHL     AX,1           ;multiply times 8

	Mov     DX,[BP+8]      ;get LRR% NOTE BYVAL
	OR      DX,DX          ;make sure not zero
	JZ      ender          ;quit if zero
	Dec     DX             ;subtract 1
	SHL     DX,1
	SHL     DX,1
	SHL     DX,1           ;multiply times 8
	CMP     AX,DX          ;if value is the same
	JE      ender          ;quit if equal

	Mov     CX,AX          ;store AX in CX
	Mov     AX,8
	Int     33h

	Mov     AX,[BP+10]     ;get ULC% NOTE BYVAL
	OR      AX,AX          ;make sure not zero
	JZ      ender          ;quit if zero
	Dec     AX             ;subtract 1
	SHL     AX,1
	SHL     AX,1
	SHL     AX,1           ;multiply times 8

	Mov     DX,[BP+6]      ;get LRC% NOTE BYVAL
	OR      DX,DX          ;make sure not zero
	JZ      ender          ;quit if zero
	Dec     DX             ;subtract 1
	SHL     DX,1
	SHL     DX,1
	Shl     DX,1           ;multiply times 8
	CMP     AX,DX          ;if value is the same
	JE      ender          ;quit

	Mov     CX,AX          ;store AX in CX
	Mov     AX,7
	Int     33h
ender:
	Pop     BP             ;restore BP
	Ret     8              ;clean up stack by eliminating 4 variables passed
MMLIMIT  Endp

;==========================================================================
; DECLARE FUNCTION MMDOUBLECLICK% (Button%)
; Input  Button% 0 = Left Button
;        Button% 1 = Right Button
; Returns
;        0 if only one click in time period
;       -1 if two or more clicks in time period
; Description:  Does not test for Mouse existence.
;===========================================================================

EVEN
MMDOUBLECLICK  Proc    Far
	Push    BP
	Mov     BP,SP          ;create stack frame and make SP addressable
	Mov     BX,[BP+6]
	Mov     BX,[BX]        ;get which button to test for
	And     BX,1           ;make sure value less than 2
	Push    BX             ;save so can use again
	Mov     AX,5
	Int     33h
	call    DELAY
	Mov     AX,5
	Pop     BX             ;get BX
	Int     33h
	Xor     DX,DX          ;clear DX so could call function as long too
	Xor     AX,AX          ;assume will fail test
	CMP     BX,2           ;check if we got less than 2 clicks
	jb      @f             ;we got less than 2 clicks so quit
	Mov     AX,-1          ;success, so store -1 in AX
@@:
	Pop     BP             ;restore BP
	Ret     2              ;clean up stack by eliminating 1 variable passed
MMDOUBLECLICK  Endp

;=====================================================================
;DELAY  - 1/2 SECOND DELAY
; FROM PC MAGAZINE, NOV, 28, 1989 PAGE 361
;=====================================================================

EVEN
DELAY   Proc    NEAR             ; USES SYSTEM CLOCK
	Xor     AX, AX
	Mov     ES, AX
	Mov     BX, ES:[046Ch]
	CMP     BX, 0FFF0h       ; THIS IS EQUAL TO 10000H LESS
	JB      Delay2
Delay1:
	CMP     WORD PTR ES:[046Ch],+00
	JLE     Delay1
Delay2:
	ADD     BX, 0Fh        ;THIS VALUE. 0FH  1*18.2 or # of ticks
Delay3:
	CMP     BX, ES:[046Ch]
	JA      Delay3
	Ret
DELAY   Endp

;==========================================================================
; DECLARE SUB MMSETLOCATION(BYVAL Row%, BYVAL Col%)
;
; Purpose:
; Move Mouse to fixed text row location.
;
; Input  Row% = Text Mode Row%
;        Col% = Text Mode Col%
; Returns
;        nothing, sub fails if Row or Col% = 0
; Description:  Does not test for Mouse existence.
;
;===========================================================================

EVEN
MMSETLOCATION Proc Far
	Push    BP
	Mov     BP,SP          ;create stack frame and make SP addressable
	Mov     DX,[BP+8]      ;get row% NOTE BYVAL
	OR      DX,DX          ;quit if zero
	JZ      Halt_pgm
	Dec     DX             ;subtract 1
	SHL     DX,1           ;multiply by 2
	SHL     DX,1           ;multiply by 4
	SHL     DX,1           ;multiply by 8  (faster than using CL)

	Mov     CX,[BP+6]      ;get column% NOTE BYVAL
	JCXZ    Halt_pgm       ;if CX = 0 then quit
	Dec     CX             ;subtract 1
	SHL     CX,1           ;multiply by 2
	SHL     CX,1           ;multiply by 4
	SHL     CX,1           ;multiply by 8  (faster than using CL)

	Mov     AX,4           ;select function 4
	Int     33h            ;DX has row, CX has column
Halt_pgm:
	Pop     BP             ;restore BP
	Ret     4              ;clean up stack by eliminating 2 variable passed
MMSETLOCATION Endp

;===========================================================================
; DECLARE SUB MMOVED (RowDist%, ColDist%)
;
; Gives direction of movement as a signed integer since last time
; this routine was called.  The direction of motion is all that is valuable
; because the micky count is worthless
;===========================================================================

EVEN
MMOVED Proc FAR
	Push    BP
	Mov     BP,SP          ;create stack frame and make SP addressable
	Mov     AX,11
	Int     33h
	Mov     BX,[BP+6]      ;get address of Col%
	Mov     [BX],CX        ;store value from AX in indirect address in BX
	Mov     BX,[BP+8]      ;get address of Row%
	Mov     [BX],DX        ;store value from DX in indirect address in BX
	Pop     BP             ;restore BP
	Ret     4              ;clean up stack by eliminating 2 variables passed
MMOVED  Endp

;===============================================================
; DECLARE FUNCTION MMVISIBLE%()
; Description:  Does not test for Mouse existence.
;               Returns current status of mouse cursor so long as
;               MMCURSON was used to set it.
;    Return Values:
;               0 = mouse cursor not visible
;              -1 = mouse cursor visible
;===============================================================

EVEN
MMVISIBLE Proc Far
	Xor     AX,AX                   ; assume not visible
	Cmp     Hide_Stat,0
	Jne     @f
	Mov     AX,-1                   ; return that it is on
@@:
	Ret
MMVISIBLE Endp

;===============================================================
; DECLARE SUB MMINVIZ (BYVAL ULR, BYVAL ULC, BYVAL LRR, BYVAL LRC)
; Shuts off the mouse cursor, if and only if inside window measured by
; ULR = Upper Left Row of a rectangle box  (1 to 50)
; ULC = Upper Left Column of a rectangle box (1 to 80)
; LRR = Lower Right Row of a rectangle box  (1 to 50)
; LRC = Lower Right Column of a rectangle box  (1 to 80)
;
; Use Function 1 to turn cursor back on.
;
; IF ULR, ULC, LRR or LRC = 0 then quits doing nothing
;
; Returns:   Nothing
;	Mirrors Mouse function 26
;===============================================================
EVEN
MMINVIZ  PROC   FAR
	Push    BP              ; set up stack frame
	Mov     BP,SP           ; point to stack
	Push    SI              ; save necessary registers
	Push    DI
	Cmp     Hide_Stat,0     ; is mouse cursor off <> 0
	JE      @f              ; nope, so continue
	Jmp     Short   Finis4  ; else quit
@@:
	Mov     DX,[bp+12]      ; get ULR
	Or      DX,DX           ; store ULR in DX
	JZ      Finis4          ; if ULR (DX) = 0 then quit
	Dec     DX              ; make it zero biased
	SHL     DX,1            ; multiply by 2
	SHL     DX,1            ; by 4
	SHL     DX,1            ; by 8

	Mov     CX,[bp+10]      ; get ULC
	JCXZ    Finis4          ; if ULC (CX) = 0 then quit
	Dec     CX              ; make it zero biased
	SHL     CX,1            ; multiply by 2
	SHL     CX,1            ; by 4
	SHL     CX,1            ; by 8

	Mov     DI,[bp+8]       ; get LRR in DI
	Or      DI,DI
	JZ      Finis4          ; if LRR (DI) = 0 then quit
	Dec     DI              ; make it zero biased
	SHL     DI,1            ; multiply by 2
	SHL     DI,1            ; by 4
	SHL     DI,1            ; by 8

	Mov     SI,[bp+6]       ; get LRC in SI
	Or      SI,SI
	JZ      Finis4          ; if LRC (SI) = 0 then quit
	Dec     SI              ; make it zero biased
	SHL     SI,1            ; multiply by 2
	SHL     SI,1            ; by 4
	SHL     SI,1            ; by 8

	Mov     AX,16
	Mov     Hide_Stat,AL    ; say cursor hidden
	Int     33h

Finis4:
	Pop     DI
	Pop     SI
	Pop     BP
	Ret     8               ; remove 4 * 2 parameters
MMINVIZ ENDP

;===============================================================
; DECLARE FUNCTION MMBUFSIZE%()
; Returns size of buffer needed to save current Mouse state
; Does check for mouse interrupt existence.   Can be used as a
; test of Mouse existence.  If MMBUFSIZE% = 0 then no mouse
;===============================================================
EVEN
MMBUFSIZE  PROC   FAR
	Mov     AX, 3533h               ; Get Int 33H vector by calling
	Int     21h                     ; Int 21H. Reply in ES:BX
	Mov     AX, ES                  ; Check segment and offset of
	OR      AX, BX                  ; Int 33H.  If both 0 then driver
	JZ      Finis2                  ; is not installed. (Hidden DOS
					; version check.  DOS before 3.x
					; didn't initialize an INT 33h vector)
	Xor     BX,BX                   ; make sure BX = 0
	Mov     AX,21                   ; get DRIVER STATE STORAGE SIZE
	Int     33h
	Mov     AX,BX                   ; store buffer size in AX

Finis2:
	Ret
MMBUFSIZE  ENDP

;===============================================================
; DECLARE SUB MMRESET ()
; Description:  Does not test for Mouse existence.
;		Resets mouse to default state and records that mouse
;             	cursor is no longer visible.
; Returns
;	Nothing
;===============================================================

EVEN
MMRESET	PROC	FAR
	Mov     AX,33                   ; perform software reset, very fast
	Int     33h
	Mov     AX,14                   ; turn off light pen emulation
	Mov     Hide_Stat,AL		; record mouse cursor hidden
	Int     33h                     ; for speed (since function is
					; is useless on a VGA, and only
					; useful inside GWBASIC with PEN)
	Ret
MMRESET	ENDP

;===============================================================
; DECLARE FUNCTION MMORKEY% ()
; Description:
;		Tests if a key has been pressed, if no, then tests
;		if a mouse key has been pressed
;
; Returns:
;       -1 if a MOUSE press or key press occurred
;        0 if nothing found
; Theory:
;        This makes the loop run inside QBasic where it can be
;        interrupted for ON ... GOSUB's etc.
;===============================================================

EVEN
MMORKEY	PROC	FAR
	Mov     AX,0C00h        ; clears the keyboard buffer
	Int     21h             ; using DOS, prevents wrong keys
				; from backing up in buffer
Begin_Loop:
	Mov     AH,1            ; sets Zero flag if no character is
	Int     16h             ; pending
	JZ      Try_Mouse
	Jmp	Short Have_Key

Try_Mouse:
	Mov     AX, 3533h       ; Get Int 33H vector by calling
	Int     21h             ; Int 21H. Reply in ES:BX
	Mov     AX, ES          ; Check segment and offset of
	OR      AX, BX          ; Int 33H.  If both 0 then driver
	JZ      No_Key          ; is not installed.
	Xor     BX,BX           ; make sure BX = 0
	Mov     AX,21           ; get DRIVER STATE STORAGE SIZE
	Int     33h
	Or	BX,BX		; see is storage size is 0
	JZ      No_Key          ; if it is 0 then no mouse and no key
	
Test_Button:
	Xor	BX,BX		; clear register for testing
	Mov	AX,3
	Int	33h
	And     BL,3            ; keep BX w/in range of 0 - 3
	OR      BL,BL           ; BL = 0 if no button was pressed
	JNZ     Have_Key        ; if BL <> 0 then a Mouse button was pressed

No_Key:
	Xor     AX,AX           ; report no key or button found & quit
	Jmp     Short MM_Finis

Have_Key:
	Mov     AX,-1           ; report a key or mouse button found

MM_Finis:
	Ret
MMORKEY	ENDP
End
