Unit Mouse;
Interface
Uses DOS;
{MOUSE ROUTINES}

{See PC Mouse tech ref for details}

const        {functions}
  ResetMouse     = 0;
  ShowMouse      = 1;
  HideMouse      = 2;
  ButtonStatus   = 3;
  MousePosition  = 3;
  SetCursor      = 4;
  ButtonPressed  = 5;
  ButtonReleased = 6;
  SetMinMaxHoriz = 7;
  SetMinMaxVert  = 8;
  SetGrafCursor  = 9;
  SetTextCursor  = 10;
  ReadMotion     = 11;
  SetSubAddr     = 12;
  LightPenOn     = 13;
  LightPenOff    = 14;
  SetMickey      = 15;

             {cursors}

  CheckCursor     = 1;
  HandCursor      = 2;
  XCursor         = 3;
  CrossCursor     = 4;
  HourGlassCursor = 5;
  ArrowCursor     = 6;
  WatchCursor     = 7;

  Cursor : array[1..7,0..1,0..15] of word =

{Check}   ((($fff0,$ffe0,$ffc0,$ff81,$ff03,$0607,$000f,$001f,
             $c03f,$f07f,$ffff,$ffff,$ffff,$ffff,$ffff,$ffff),
            ($0000,$0006,$000c,$0018,$0030,$0060,$70c0,$1d80,
             $0700,$0000,$0000,$0000,$0000,$0000,$0000,$0000)),

{hand}     (($f3ff,$edff,$e1ff,$e1ff,$e1ff,$e049,$e000,$e000,
             $8000,$0000,$0000,$0000,$0000,$0000,$0000,$8001),
            ($0c00,$1200,$1200,$1200,$1200,$13b6,$1249,$1249,
             $7249,$9001,$9001,$9001,$8001,$8001,$8001,$7ffe)),

{x}        (($07e0,$0180,$0000,$c003,$f00f,$c003,$0000,$0180,
             $07e0,$ffff,$ffff,$ffff,$ffff,$ffff,$ffff,$ffff),
            ($0000,$700e,$1c38,$0660,$03c0,$0660,$1c38,$700e,
             $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000)),

{Cross}    (($fc3f,$fc3f,$fc3f,$fc3f,$0000,$0000,$0000,$fc3f,
             $fc3f,$fc3f,$fc3f,$ffff,$ffff,$ffff,$ffff,$ffff),
            ($0000,$0180,$0180,$0180,$0180,$7ffe,$0180,$0180,
             $0180,$0180,$0000,$0000,$0000,$0000,$0000,$0000)),

{Hour}     (($0000,$0000,$0000,$8001,$c003,$e007,$f00f,$e007,
{Glass}      $c003,$8001,$0000,$0000,$0000,$0000,$0000,$0000),
            ($0000,$7ffe,$6006,$300c,$1818,$0c30,$0660,$03c0,
             $0660,$0c30,$1998,$33cc,$67e6,$7ffe,$0000,$0000)),

{Arrow}    (($3fff,$1fff,$0fff,$07ff,$03ff,$01ff,$00ff,$007f,
             $003f,$001f,$01ff,$10ff,$30ff,$f87f,$f87f,$fc3f),
            ($0000,$4000,$6000,$7000,$7800,$7c00,$7e00,$7f00,
             $7f80,$7fc0,$7c00,$4600,$0600,$0300,$0300,$0180)),

{Watch}    (($e007,$e007,$e007,$e007,$c003,$8001,$8001,$8001,
             $8001,$8001,$c003,$e007,$e007,$e007,$e007,$e007),
            ($0ff0,$0ff0,$0ff0,$0ff0,$1008,$2084,$2084,$2086,
             $2f86,$2004,$2004,$1008,$0ff0,$0ff0,$0ff0,$0ff0)));

Type
  MouseResult = Record                               {function Number}
                  Installed     : boolean;                { 0   }
                  NumberButtons : integer;                { 0   }
                  Button1       : boolean;                { 3   }
                  Button2       : boolean;                { 3   }
                  Button3       : boolean;                { 3   }
                  ButtonArray   : Array[1..3] of boolean; { 3   }
                  AnyButton     : boolean;                { 3   }
                  HPos          : integer;                { 3   }
                  VPos          : integer;                { 3   }
                  CharRow       : integer;                { 3   }
                  CharCol       : integer;                { 3   }
                  ButtonMask    : integer;                { 3   }
                  Button        : boolean;                { 5,6 }
                  ButtonCount   : integer;                { 5,6 }
                  HMotion       : integer;                { 11  }
                  VMotion       : integer;                { 11  }
                end;

var
  MouseReturn : MouseResult;

procedure InitMouse;
procedure Hide;  {Hides the mouse cursor}
procedure Show;  {Shows the mouse cursor}
procedure SetGraphCursor(CursorType : byte);  {Sets shape}
function  MouseX                              : integer;  {Returns x coord}
function  MouseY                              : integer;
function  MouseButton                         : word;     {returns button status}
procedure MouseXY(Var x, y : word);            {Returns XY}
procedure SetMouseXY(x, y : word);
procedure KeepMouseIn(x1,y1,x2,y2 : word);     {keeps in window}
function  MousePositionIn(x1,y1,x2,y2 : word) : boolean; {True if in bounds}
function  XYPos(x1,y1,x2 : word)              : boolean; {True if on y text coord and in x coords}
procedure SetMickeys(Mickeys : word);    {mouse sensitivity}
procedure ProtectCursor(x1,y1,x2,y2 : word);  {not used.  hides when in bounds (text)}
procedure ProtectCursorXY(x1,y1,x2,y2 : word);{not used.  hides when in bounds (graph)}


implementation
var
  regs : registers;

  procedure InitReturn;
  Var
    I : integer;
  begin
    With MouseReturn do
    begin;
      Installed        := False;
      NumberButtons    := 0;
      Button1          := False;
      Button2          := False;
      Button3          := False;
      For I := 1 to 3 do
        ButtonArray[I] := False;
      AnyButton        := False;
      HPos             := 0;
      VPos             := 0;
      ButtonMask       := 0;
      Button           := False;
      ButtonCount      := 0;
      HMotion          := 0;
      VMotion          := 0;
      CharRow          := 0;
      CharCol          := 0;
    end;
  end;


procedure InitMouse;
begin
  regs.AX := 0;
  Intr($33,regs);
  InitReturn;
  If regs.AX = $ffff then
    MouseReturn.Installed := True
  Else
    MouseReturn.Installed := False;
  If MouseReturn.Installed = False Then
  begin
    WriteLn;
    WriteLn('Error: Mouse driver not installed!');
    Halt(1);
  end;
end;

procedure Hide;
begin
  regs.AX := HideMouse;
  Intr($33,regs);
end;

procedure Show;
begin
  regs.AX := ShowMouse;
  Intr($33,regs);
end;

function MouseX;
var
  regs : Registers;
begin
  regs.AX := 3;
  Intr($33,regs);
  MouseReturn.HPos := regs.CX;
  MouseReturn.VPos := regs.DX;
  MouseReturn.ButtonMask := regs.BX and 7;
  MouseReturn.AnyButton := (regs.BX and 7 <> 0);
  MouseX := regs.CX;
end;

function MouseY;
var
  regs : Registers;
begin
  regs.AX := 3;
  Intr($33,regs);
  MouseReturn.HPos := regs.CX;
  MouseReturn.VPos := regs.DX;
  MouseReturn.ButtonMask := regs.BX and 7;
  MouseReturn.AnyButton := (regs.BX and 7 <> 0);
  MouseY := regs.DX;
end;

function MouseButton;
var
  regs : Registers;
begin
  regs.AX := ButtonStatus;
  Intr($33,regs);
  MouseReturn.HPos := regs.CX;
  MouseReturn.VPos := regs.DX;
  MouseButton := regs.BX;
end;

procedure MouseXY;
Var
  regs : Registers;
begin
  regs.AX := MousePosition;
  Intr($33,regs);
  x := regs.CX;
  y := regs.DX;
  With MouseReturn do
  begin
    HPos := x;
    VPos := y;
    ButtonMask := regs.BX;
    If regs.BX and 1 = 1 then
    begin
      Button1 := True;
      ButtonArray[1] := True;
    end;
    If regs.BX and 2 = 2 then
    begin
      Button2 := True;
      ButtonArray[2] := True;
    end;
    If regs.BX and 4 = 4 then
    begin
      Button3 := True;
      ButtonArray[3] := True;
    end;
    If regs.BX and 7 <> 0 then
      AnyButton := True
    Else
      AnyButton := False;
    regs.AX := $0f00;
    Intr($10,regs);
    If regs.AX = $10 then
      CharRow := VPos div 14 + 1
    Else
      CharRow := VPos shr 3 + 1;
  end;
end;

procedure SetMouseXY;
begin
  regs.AX := SetCursor;
  regs.CX := x;
  regs.DX := y;
  Intr($33,regs);
  MouseReturn.HPos := x;
  MouseReturn.VPos := y;
end;

procedure SetGraphCursor;
var
  regs : Registers;
begin
  regs.AX := SetGrafCursor;
  regs.DX := Ofs(Cursor[CursorType,0,0]);
  regs.ES := Seg(Cursor);
  Case CursorType of
    CheckCursor        : begin
                           regs.BX := 6;
                           regs.CX := 8;
                         end;
    HandCursor         : begin
                           regs.BX := 5;
                           regs.CX := 0;
                         end;
    XCursor            : begin
                           regs.BX := 7;
                           regs.CX := 4;
                         end;
    CrossCursor        : begin
                           regs.BX := 7;
                           regs.CX := 7;
                         end;
    HourGlassCursor    : begin
                           regs.BX := 7;
                           regs.CX := 7;
                         end;
    ArrowCursor        : begin
                           regs.BX := 0;
                           regs.CX := 0;
                         end;
    WatchCursor        : begin
                           regs.BX := 7;
                           regs.CX := 7;
                         end;
  end;
  Intr($33,regs);
end;

procedure KeepMouseIn;
var
  regs : Registers;
begin
  regs.AX := 7;
  regs.CX := x1;
  regs.DX := x2;
  Intr($33,regs);
  regs.AX := 8;
  regs.CX := y1;
  regs.DX := y2;
  Intr($33,regs);
end;

function MousePositionIn; {Graphic Mouse Position}
                                                           {Returns true if the cursor is within those coordinates}
var
  x,y : word;
begin
  With MouseReturn do begin
    MouseXY(x,y);
    If (x >= x1) and (x <= x2) and (y >= y1) and (y <= y2) then
      MousePositionIn := True
    Else
      MousePositionIn := False;
  end;
end;

function XyPos; {Text Mouse Position Used for menu}
Var
  y2 : integer;
  x,y : word;
begin
  x1 := Pred(x1) * 8;
  y1 := Pred(y1) * 14;
  x2 := Pred(Pred(x2) * 8);
  y2 := y1 + 13;
  MouseXY(x,y);
  If (x >= x1) and (x <= x2) and (y >= y1) and (y <= y2) then
    XyPos := True
  Else
    XyPos := False;
end;

procedure SetMickeys;
var
  regs : Registers;
begin
  regs.AX := SetMickey;
  regs.CX := Mickeys;
  regs.DX := Mickeys;
  Intr($33,regs);      {Set mickeys per pixel  See mouse tech ref.}
end;

procedure ProtectCursor;
var
  regs : registers;
begin
  regs.AX := 16;
  regs.CX := Pred(x1)*8;
  regs.DX := Pred(y1)*14;
  regs.SI := x2*8-1;
  regs.DI := y2*14-1;
  Intr($33,regs);
end;

procedure ProtectCursorXY;
var
  regs : registers;
begin
  regs.AX := 16;
  regs.CX := x1;
  regs.DX := y1;
  regs.SI := x2;
  regs.DI := y2;
  Intr($33,regs);
end;

end.
