/* --------- phonedir.c ---------- */
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <string.h>
#include <mem.h>
#include <ctype.h>
#include "window.h"
#include "entry.h"
#include "help.h"
#include "modem.h"

#define DIRECTORY "phone.dir"
#define MAX_ENTRIES 50
#define WRITEDIR F2
#define MODIFYDIR F3

void phdirectory(void);
char scriptfile[13];
static void get_directory(void);
static void put_directory(void);
static int dirproc(int, int);
static void bld_default(int);
static void build_dir(int);
static int enter_directory(int);
static void select_directory(int);
char *prompt_line(char *, int, char *);
void reset_prompt(char *, int);
static int edit_directory(void);
static int direrror(int);
static void field_terminate(FIELD *fld, int termchar);

extern int PARITY,STOPBITS,WORDLEN,BAUD;
extern char PHONENO[];
extern char spaces[];
extern struct wn wkw;   /* the directory window structure */
extern void (*phone_directory)() = phdirectory;
/* -------- phone directory file record ------------ */
struct {
    char ol_name[21];    /* callee's name             */
    char ol_phone[24];   /* phone number              */
    char ol_parity[8];   /* none/odd/even             */
    char ol_stopbits[4]; /* 1 or 2                    */
    char ol_wordlen[3];  /* 7 or 8                    */
    char ol_baud[6];     /* 110,150,300,600,1200,2400 */
    char ol_script[9];   /* name of script file       */
} pd;
static char hdr[] =
    " Name                 "
    "Phone Number           "
    "Parity Stop Len Baud  Script";
static char select_prompt[] =
    "\030\031\021\304\331:Select Esc:Return "
    "F2:Write Directory F3:Modify "
    "Ins:Insert Del:Delete";
static char enter_prompt[] =
    " F2:Write Changes to Directory    "
    " Esc:Ignore Entry   F1:Help";
static char *pds[MAX_ENTRIES+1];
static int pct;
static FILE *fp;
static char nmmask[] = "____________________";
static char phmask[] = "____________________";
static char prmask[] = "____";
static char sbmask[] = "_";
static char wlmask[] = "_";
static char bdmask[] = "____";
static char scmask[] = "________";
/* ------- data entry template for the directory ------- */
FIELD directory_template[] = {
    {3, 16, 1, pd.ol_name,     nmmask, "name"},
    {4, 16, 1, pd.ol_phone,    phmask, "phone"},
    {5, 16, 1, pd.ol_parity,   prmask, "parity"},
    {6, 16, 1, pd.ol_stopbits, sbmask, "stopbits"},
    {7, 16, 1, pd.ol_wordlen,  wlmask, "wordlen"},
    {8, 16, 1, pd.ol_baud,     bdmask, "baud"},
    {9, 16, 1, pd.ol_script,   scmask, "script"},
    {0}
};
/* -------- data entry error messages --------- */
static char *ermsgs[] = {
    "Parity must be None, Odd, or Even",
    "Stop Bits must 1 or 2",
    "Word Length must be 7 or 8",
    "Baud Rate must be 110,150,300,600,1200,2400"
};
/* ------ manage the telephone directory ------ */
void phdirectory(void)
{
    int s = 1;
    char *ttl, *sel;
    set_help("directry");
    ttl = prompt_line(hdr, 1, NULL);
    sel = prompt_line(select_prompt, 25, NULL);
    establish_window(1,2,80,24,TEXTFG,TEXTBG,TRUE);
    get_directory();
    text_window(pds, 1);
    while (pct &&
          (s=select_window(s,SELECTFG,SELECTBG,dirproc))!=0)
        if (pct && pds[s-1] != spaces+1)    {
            select_directory(s-1);
            break;
        }
    delete_window();
    reset_prompt(sel, 25);
    reset_prompt(ttl, 1);
}
/* -------- select the directory entry for the dialer ------- */
static void select_directory(int n)
{
    char *cp = scriptfile;
    movmem(pds[n], &pd, sizeof pd);
    strncpy(PHONENO, pd.ol_phone, 20);
    BAUD     = atoi(pd.ol_baud);
    STOPBITS = *pd.ol_stopbits - '0';
    WORDLEN  = *pd.ol_wordlen  - '0';
    PARITY   = (*pd.ol_parity == 'N' ? 0 :
                *pd.ol_parity == 'O' ? 1 : 2);
    establish_window(30,11,50,13,HELPFG,HELPBG,TRUE);
    gotoxy(2,2);
    cputs("Initializing Modem");
    initmodem();
    delete_window();
    setmem(scriptfile, sizeof scriptfile, '\0');
    strncpy(scriptfile, pd.ol_script, 8);
    while (*cp && *cp != ' ')
        cp++;
    strcpy(cp, ".scr");
}
/* ------ read the phone directory ----------- */
static void get_directory(void)
{
    if (pct == 0 && (fp = fopen(DIRECTORY, "r")) != NULL)   {
        while (fread(&pd, sizeof pd, 1, fp) != 0)   {
            build_dir(pct++);
            if (pct == MAX_ENTRIES)
                break;
        }
        pds[pct++] = spaces+1;
        pds[pct] = NULL;
        fclose(fp);
    }
    if (pct == 0)
        dirproc(INS, 1);
}
/* ------- build a default phone directory entry -------- */
static void bld_default(int n)
{
    static char *prs[] = {"None", "Odd ", "Even"};
    setmem(&pd, sizeof pd-1, ' ');
    strncpy(pd.ol_parity, prs[PARITY], 4);
    *pd.ol_stopbits = STOPBITS + '0';
    *pd.ol_wordlen  = WORDLEN + '0';
    sprintf(pd.ol_baud, "%4d", BAUD);
    pd.ol_baud[4] = ' ';
    build_dir(n);
}
/* --------- build a directory entry for display ----------- */
static void build_dir(int n)
{
    if ((pds[n] = malloc(sizeof pd)) != NULL)
        movmem(&pd, pds[n], sizeof pd);
}
/* ------- write the phone directory ---------- */
static void put_directory(void)
{
    int i;
    fp = fopen(DIRECTORY, "w");
    for (i = 0; i < pct; i++)
        if (pds[i] != spaces+1)
            fwrite(pds[i], sizeof pd, 1, fp);
    fclose(fp);
}
/* ---------- process a directory entry ------------- */
static int dirproc(int c, int lineno)
{
    int i, j;
    switch (c)  {
        case DEL:
            if (pds[lineno-1] != spaces+1)  {
                free(pds[lineno-1]);
                for (j = lineno-1; j < pct; j++)
                    pds[j] = pds[j+1];
                if (--pct)  {
                    text_window(pds, wkw.wtop);
                    for (i = pct+2; i <= wkw.wtop+wkw.ht; i++)
                        writeline(2, i, spaces+1);
                    if (lineno-1 == pct)
                        --lineno;
                }
                else
                    clear_window();
            }
            break;
        case INS:
            if (pct == MAX_ENTRIES)
                break;
            i = pct;
            if (i)
                while (i >= lineno) {
                    pds[i] = pds[i-1];
                    --i;
                }
            bld_default(i);
            pct++;
        case MODIFYDIR:
            if (pds[lineno-1] != spaces+1)  {
                movmem(pds[lineno-1], &pd, sizeof pd);
                enter_directory(lineno-1);
            }
            break;
        case WRITEDIR:
            put_directory();
            break;
    }
    wkw.wy = lineno - wkw.wtop + 1;
    return (pct == 0);
}
/* ------- data entry for a directory record ---------- */
static int enter_directory(int lineno)
{
    int s = 1;
    char *p = prompt_line(enter_prompt, 25, NULL);
    establish_window(20,5,56,15,ENTRYFG,ENTRYBG,TRUE);
    window_title(" Telephone Directory Entry ");
    gotoxy(3,3), cputs("Name:");
    gotoxy(3,4), cputs("Phone:");
    gotoxy(3,5), cputs("Parity:");
    gotoxy(3,6), cputs("Stop Bits:");
    gotoxy(3,7), cputs("Word Length:");
    gotoxy(3,8), cputs("Baud Rate:");
    gotoxy(3,9), cputs("Script:");
    field_terminate(directory_template, '\0');
    while (s != WRITEDIR && s != ESC)   {
        s = data_entry(directory_template, FALSE, s);
        if (s == WRITEDIR)
            s = edit_directory();
    }
    field_terminate(directory_template, ' ');
    *(((char *)(&pd)) + sizeof pd - 1) = '\0';
    delete_window();
    reset_prompt(p, 25);
    if (s == WRITEDIR)  {
        movmem(&pd, pds[lineno], sizeof pd);
        put_directory();
    }
    text_window(pds,wkw.wtop ? wkw.wtop : 1);
    return (s != ESC);
}
/* -------- validate the directory entry -------- */
static int edit_directory(void)
{
    int i;
    static int bds[] = {110,150,300,600,1200,2400};
    *pd.ol_parity = toupper(*pd.ol_parity);
    if (*pd.ol_parity != 'N' &&
            *pd.ol_parity != 'O' &&
            *pd.ol_parity != 'E')
        return direrror(3);
    if (*pd.ol_stopbits != '1' && *pd.ol_stopbits != '2')
        return direrror(4);
    if (*pd.ol_wordlen != '7' && *pd.ol_wordlen != '8')
        return direrror(5);
    for (i = 0; i < 6; i++)
        if (atoi(pd.ol_baud) == bds[i])
            break;
    if (i == 6)
        return direrror(6);
    return WRITEDIR;
}
/* ------- post a directory entry error ---------- */
static int direrror(int n)
{
    error_message(ermsgs[n-3]);
    return n;
}
/* -------- set field terminators to null or space ------- */
static void field_terminate(FIELD *fld, int termchar)
{
    for (;fld->frow;fld++)
        *(fld->fbuff+strlen(fld->fmask)) = termchar;
}
