Comment |                PROTDEMO.ASM

                         ACKNOWLEDGEMENTS

A good part of the code in this module was taken from the book "80386
Technical Reference", Edmund Strauss, Brady Books.  I highly recommend
this reference and, in fact, it may be required to understand some of
what follows.
|

.sall  ;suppress macro exp in listing
.model small
.386P         ;placed here makes 16 bit segments

MESSAGE macro msg
 local xx
.Data
xx db "&msg",0
.Code
    mov si,offset xx
    call OutSt
EndM

MESSAGECRLF macro msg
    MESSAGE <msg>
    call CrLf
EndM

;Flag bits
VMFL   equ 2H
IOPLFL equ 3000h
IFL    equ 0200H
TRAPFL equ 0100h

               ;Gdt Selectors
MAIN_TSS_SEL equ 08H
TMP_TSS_SEL equ 10H
FLAT_SEL equ 18H
REAL_MODE_SEL equ 20h
CODE_SEL  equ 28H
DATA_SEL equ 30H
FULL_TEXT_SEL equ 38h

.Data
Include TABLES.INC  ;contains the GDT, IDT, TSS's, etc.

IntFlag db 0
RealIDT  df  03FFh
SaveSP   dw ?

     dw 256 dup (?)
SupIntTos label word     ;Stack for level switch to level 0 (interrupts)
     dw 512 dup (?)
SupTos label word        ;Stack for supervisory mode use
     dw 128 dup (?)
vm_int_tos label word    ;Stack used when using virtual mode to do interrupts
     dw 256 dup(?)
Tos label word           ;Stack used here when in real mode

.Code

;----------------Begin
   assume CS:_TEXT, DS:DGROUP, ES:nothing, SS:DGROUP
Begin:                  ;this is where we start when called
      mov ax,DGROUP
      mov DS,ax
      mov SS,ax
      mov sp, OFFSET Tos

      call SeeIf386   ;make sure this is a 386 CPU
      cmp ax,0
        jne short vm7
      MESSAGECRLF <CPU is not 80386>
      jmp Exit

vm7:
      smsw ax     ;Check the protect mode bit of the machine status word
      test ax,1
        jz short vm8
      MESSAGECRLF <Already in protected mode>
Exit: mov ax,4C01h
      int 21h

vm8:  sidt [RealIDT] ;save the real mode IDT
;setup tables
      call InitBases ;convert segment:offsets in various tables to linear addr
      call InitIDT   ;fill in the IDT to direct interrupts to the jump table

      lidt IDT_Fword ;load the IDT and GDT registers
      lgdt GDT_Fword

      mov bx,TMP_TSS_SEL  ;get ready to set up a temporary task
      cli                 ;interrupts off here
      mov eax,1       ;set protect mode bit
      mov cr0,eax     ;enable protect mode

;We're now in protect mode.  The instructions in the prefetch queue are no
;longer correct, so flush the queue by doing a jump

      jmp short flush
flush:                ;in protected mode level 0

      ltr bx          ;load task register with TMP_TSS_SEL, a temporary task

;I don't believe the temporary task is actually necessary.  But it does
;provide a neat way to initiate the main task by doing a task switch.

      jmp  cs:dword ptr [TaskSw]    ;This is a task switch to MainTSS

TaskSw  label dword
      dw    0,MAIN_TSS_SEL  ;to the Main task

Comment |
Now that the task switch is made, initiation has been accomplished according
to MainTSS.  In particular:
    SS:esp = DATA_SEL:SupTos
    CS:eip = CODE_SEL:BeginSupervisory
    Eflags have I/O priveledge level set to 3 to allow virtual mode to be
      able to use the sti,cli,int,iret,pushf,popf instructions.
|
BeginSupervisory:
      sti               ;interrupts can go on now
      mov ax,DATA_SEL   ;this could have been done in MainTSS
      mov DS,ax         ;we're using selectors now, not real mode segments

;Software interrupts are permitted here with the restriction that they
;cannot pass information in the segment registers.  In the following,
;Int 10h and Int 16h are used to provide console I/O.  In addition,
;hardware interrupts are active.  Interrupt processing is largely
;done in virtual mode by the normal interrupt routines.  See the discussion
;at 'Reflect' for more info.

      MESSAGECRLF <Now in Supervisory mode--type ESC to exit>

Back: MESSAGE <Type a character-- >
      call GetChar
      cmp al,1Bh         ;Escape?
        je ReturnToReal  ;yes
      call OutCh
      call CrLf
      jmp Back

ReturnToReal:
    call CrLf
;to return to real mode, all the segment registers must be loaded with
;selectors whose descriptors match a typical real mode conterpart.  In
;particular, REAL_MODE_SEL, must be a read/write data segment with a limit
;of 0FFFFh.
    mov ax,REAL_MODE_SEL
    mov ES,ax
    mov FS,ax
    mov GS,ax
    mov DS,ax
    mov SS,ax
;the same must be true for CS.  In this case, FULL_TEXT_SEL is a descriptor
;for an execute/readable code segment with limit of 0FFFFh.  To change the
;CS selector, do an intersegment jump.

    jmp dword ptr CS:[FullSegm]
FullSegm dw offset FSeg, FULL_TEXT_SEL

FSeg:
    cli                ;interrupts off now
    xor eax,eax
    mov cr0,eax        ;reset the protect bit

;now in real mode again.  At his point, a _direct_ far jump will flush the
;instruction cache and make the CS segment writable.  Important--do not use
;an indirect far jump here (as in jmp dword ptr [Blah]) as the code segment
;will be left writeprotected.

    jmp far ptr FCache

FCache label far
    mov ax,DGROUP    ;must set up the segm registers using real mode segments
    mov DS,ax
  assume DS:DGROUP
    mov SS,ax        ;set up a stack before turning interrupts on!
    mov sp,offset tos
    lidt fword ptr [RealIDT]  ;load the IDT register with appropriate real
                              ;mode value

    sti                       ;now the interrupts can go on again
    MESSAGECRLF <Back in real mode.  Exiting...>

    mov ax,4C00h
    Int 21h             ;terminate
  assume DS:Nothing

INCLUDE  REFLECT.INC   ;the interrupt jump tables

;Define some stack displacements.  TRerr is an error code that gets pushed
;for certain exceptions.  While it's not used in this program, BP is aligned
;at the point where it would normally be.

TRgs     equ   Word Ptr [bp] +36
TRfs     equ   Word Ptr [bp] +32
Trds     equ   Word Ptr [bp] +28
TRes     equ   Word Ptr [bp] +24
TRss     equ   Word Ptr [bp] +20
TRsp     equ   Word Ptr [bp] +16
TRflag   equ   Word Ptr [bp] +12
TRcs     equ   Word Ptr [bp] +8
TRip     equ   Word Ptr [bp] +4
TRerr    equ   Word Ptr [bp] +0   ;only present for exceptions

TReflag  equ   Dword Ptr [bp]+12

;--------------Reflect
;Most interrupts come here by way of the jump table in REFLECT.INC.  BP has
;been pushed and now contains the interrupt number.  Interrupts are off.
    assume DS:nothing

Reflect:
      push eax
      mov ax,bp  ;transfer int num to ax
      mov bp,sp
      add bp,2      ;to get proper allignment. BP pts to TRerr
      push ebx
      test TRflag+2,VMFL   ;see if we're in Virtual mode when int occured
        jz LocalInt        ;no it happened in supervisory mode

Comment |
MainTSS defines two stacks, the one at DATA_SEL:SupTos is used at task
initialization.  The other at DATA_SEL:SupIntTos is used when a switch
from a lower level to supervisory level is made.  Therefore, if the interrupt
occurs when in virtual mode, a stack switch is made to DATA_SEL:SupIntTos and
the stack will look like that below (left).  Here the virtual mode segment
registers and stack pointer are saved as well as the eflags and return
address.  We will modify the stack to look like that on the right so that
when an IretD is issued, a return will be made directly to the interrupt
service routine.  Processing of the interrupt will then be done in virtual
mode by the usual interrupt routine.

In addition, the virtual mode stack will be modified by adding a stack
frame containing the original return address so that at the Iret in the
interrupt routine, return will go directly to the interrupted instruction.

        Stack on entry                     Modified stack
SupIntTos ->..
            GS          +36                    GS
            FS          +32                    FS
            DS          +28                    DS
            ES          +24                    ES
            SS          +20                    SS
            (old)esp    +16                    newesp=oldesp+6
            eflags      +12                    eflags  (interrupt off)
            return CS   +8                     int vector segment
            return IP   +4                     int vector IP
BP (TRerr)->  ..

        New stack frame added to caller's stack
oldesp ->    ..
            flags
            return CS
newesp ->   return IP
---------------------|

      push ax     ;save int type
      mov bx,DATA_SEL
      mov DS,bx
   assume DS:DGROUP
      mov ax, FLAT_SEL   ;can address all of memory with this selector
      mov FS,ax

      movzx eax,TRsp     ;change top of caller's stack to add frame
      sub ax, 3*2
      mov TRsp,ax

      movzx ebx,TRss
      shl ebx,4
      add ebx,eax     ;linear address of caller's stack frame

      mov ax,TRip         ;install new frame for caller
      mov FS:[ebx],ax     ;put in IP
      mov ax,TRcs
      mov FS:[ebx+2],ax   ;put in CS
      mov ax,TRflag
      mov FS:[ebx+4],ax   ;put in flags

      pop bx       ;the int number
      shl bx,2     ;change return on this stack to go to interrupt routine
      mov ax,FS:[bx]  ;get ip of int vector
      mov TRip,ax
      mov ax,FS:[bx+2] ;get cs of int vector
      mov TRcs,ax
      and TRflag, NOT (IFL or TRAPFL) ;don't want interrupt on there

      pop ebx
      pop eax
      pop bp
      iretD       ;to normal interrupt routine

LocalInt:       ;an interrupt occuring when in Supervisory mode
     ;Note: Software interrupts may be used in protected mode providing that
     ;the segment registers are not needed for parameter passing.  Info may be
     ;transfered in all registers but the segment regs and bp--
     ;the latter are preserved.

Comment |
If the interrupt occurs when in supervisory mode, no stack switch will take
place--the pushed values go on the same stack.  Since the interrupt is to
be processed in virtual mode, a stack frame must be added so that an IretD
will cause execution to proceed in virtual mode at the start of the interrupt
routine.  In addition, a virtual mode stack is set up in a way  that at the
interrupt routine's iret, execution will resume at 'Int63Addr' in this module.
Here an Int 63h is executed causing a return to supervisory mode at
'ReturnFromInt'.

In this case, though, the Int 63h causes a level change from virtual to
supervisory which does cause a stack change to the wrong stack.  Therefore,
it's necessary to switch back to the supervisory stack.

Note the following:

  The virtual mode segment register (also bp) values provided in the stack
  frame addition are dummies.  This is not important for hardware interrupts
  but means that software interrupt calls that use these registers can't
  be used in supervisory mode.

  While the interrupt is being processed in virtual mode, other interrupts
  may occur.  Since these occur in virtual mode they will be processed in
  the code above and cause no problem.

        Stack on entry
SupTos ->   ..
              .          <- Supervisory stack in use
            ..
            eflags      +12
            return CS   +8
            return IP   +4
BP (TRerr)->  ..


        Stack frame added to above
                .        <- saved registers
SaveSP ->     ..
            virtual GS
            virtual FS
            virtual DS
            virtual ES
            SS = DGROUP
            esp = vm_int_tos
            eflags (Virtual mode bit set, interrupts off)
            interrupt vector segment (CS)
            interrupt vector offset  (IP)


        Virtual stack setup
               flags
               Segment Int63Addr
vm_int_tos ->  Offset Int63Addr
|
   ;bp, eax, ebx have been pushed. bp points to TRerr and ax has the interrupt
   ;number
      push ds  ;these haven't been saved (no level switch), so save them
      push es
      push fs
      push gs
      movzx ebx,ax   ;save the interrupt no
      mov ax,DATA_SEL
      mov DS,ax
      mov ax,FLAT_SEL
      mov FS,ax
   assume DS:DGROUP
;this is the stack offset we'll want to switch back to in RetFromInt

      mov word ptr [SaveSP],sp

      xor eax,eax
      mov ax,DGROUP     ;set up the stack for an Iretd
      push eax     ;GS  dummy values
      push eax     ;FS
      push eax     ;DS
      push eax     ;ES
      push eax     ;SS
      mov word ptr [vm_int_tos],offset Int63Addr  ;Int63's address to virtual
      mov word ptr [vm_int_tos+2], Seg Int63Addr  ; stack
      push 0         ;high order esp
      mov ax,offset vm_int_tos  ;top of stack
      push ax        ;low order esp
      mov eax,TReflag     ;get eflags
      test ax,IFL
        setnz [IntFlag]   ;save status of interrupt on entry
      and ax,not (IFL or TRAPFL)     ;turn off interrupt and trap flags
      mov word ptr [vm_int_tos+4],ax ;put the flags on virtual stack
      or eax,VMFL*10000h             ;set virtual mode bit to return to virtual
      push eax                       ;put the flags on this stack
      mov eax,FS:[4*ebx]    ;get the interrupt vector
      sub esp,8             ;make room for cs,eip
      movzx ebx,ax          ;IP
      mov [esp],ebx
      shr eax,16
      movzx ebx,ax
      mov [esp+4],ebx     ;CS
      mov ebx,[bp-6]  ;restore eax,ebx as they were on entry
      mov eax,[bp-2]

      IretD      ;and we're off to the interrupt service routine


;The interrupt service routine then returns to Int63Addr (executes an Int 63h)
;which returns us here.

ReturnFromInt:        ;return here from an interrupt from Supervisory mode
  assume DS:DGROUP
      push DATA_SEL
      pop DS
;throw away the useless stuff on stack from the Int 63 call. Restore the stack
; position to that saved in LocalInt
      mov SP,word ptr [SaveSP]

      pushf     ;save the flags on entry--these are the ones to return
      pop bp              ;the flags on entry
      test [IntFlag],0    ;were interrupts on originally?
        je short RI1
      or bp,IFL  ;if so, see that they stay on

RI1:  pop GS     ;restore the segment registers
      pop FS
      pop ES
      pop DS
      add sp,8   ;throw away eax,ebx stored temporarily at entrance to Reflect
      mov [esp+10],bp  ;fix up the flags on the stack
      pop bp     ;the original bp at interrupt
      IretD
   assume DS:Nothing, ES:Nothing


;---------------Int63Addr
;Interrupts generated while in supervisory mode are done in virtual mode and
;return here to get back to supervisory mode.
Int63Addr:
      Int 63h  ;no return


;------------------InitBases
;Many items in the various tables require linear addresses.  The linear
;addresses can't be calculated until run time. BaseTable itemizes the offsets
;that have to be changed together with the real mode segments which are
;here converted to linear addresses.
InitBases   proc near
   Assume DS:DGROUP
      mov si, offset BaseTable
      mov cx, BaseCount
      cld
BaseFill:
      lodsw        ;offset of fixup location
      mov bx,ax
      lodsw        ;segment to add on
      movzx edx,ax ;need 32 bit register
      shl edx,4    ;segment to linear
      shld eax,edx,16 ;high byte to eax
      add [bx],dx  ;add onto the offset already in [bx]
      adc [bx+2],al
      loop BaseFill

      ret
InitBases ENDP

;---------------------InitIDT
InitIDT  proc near
Comment | InitIDT fills in the IDT (Interrupt Descriptor Table) with interrupt
gates as:

  dw  bits 0..15 of offset to start of interrupt routine (rf0, rf1,...rf255)
  dw  code selector for start of routine (CODE_SEL)
  dw  0EE00h  code for interrupt gate accessible by virtual mode.
  dw  bits 16..31 of offset to start--here always 0.
  |

      mov ax,DGROUP
      mov ES,ax
      cld
      mov dx, offset rf0   ;start of jump code.  See REFLECT.INC
      mov di, offset IDT
      mov cx,256
IDTFill:
      mov ax,dx
      add dx,SIZE_RF       ;spacing in jump code between items
      stosw
      mov ax,CODE_SEL
      stosw
      mov ax,0EE00h
      stosw
      xor ax,ax
      stosw
      loop IDTFill
      ret
InitIDT ENDP

SeeIf386 Proc near   ;check to see if 386 cpu
      xor ax,ax      ;get ready to return
      push sp
      pop bx
      cmp bx,sp      ;don't know what this test is for
        jnz short SI1
      mov bx,7000h
      push bx
      popf
      pushf
      pop bx
      and bx,7000h
        jz short SI1
      inc ax         ;it's a 386
SI1:  ret
SeeIf386 EndP

OutCh proc      ;Output the char in al to the console.
     mov ah,14  ;TTY function
     mov bx,0   ;page 0
     int 10h
     ret
OutCh EndP

CrLf proc       ;Output a CR, LF
    mov al,0Dh
    call OutCh
    mov al,0Ah
    call OutCh
    ret
CrLf EndP

OutSt proc      ;DS:si pts to ASCIIZ string (null terminated) to be output
     lodsb
     cmp al,0
       je short os1
     call OutCh
     jmp OutSt
os1: ret
OutSt EndP

GetChar proc      ;get a char from the keyboard, returned in al
     mov ah,0
     int 16h
     cmp al,0     ;Don't accept function keys
       je GetChar
     ret
GetChar EndP

CodeEnd label byte   ;last code item

.Data
DataEnd label byte   ;last data item

END  Begin
