/*
**
**  Q.C - Queue functions in 'C'
**
**
** The functions in this program are for academic purposes in the discussion
** of queue implementation.  The Queues implemented are simple character
** circular queues that follow Queue rules:
**
**  1) If the Head and Tail of a Queue are equal, then that Queue is empty
**  2) The Head is the location of the next data item to be removed from the queue
**  3) The Tail is to be the location of the next data item added to the queue.
**  4) Data is inserted and removed in a FIFO format.
**  5) If in inserting an item into a Queue the Tail pointer is incremented so
**        that it now equals the head, an overflow has (or will) occur(ed).  Since
**        Rule 1 states this is an 'empty' queue, it must be an error (inserting
**        data into a queue can not empty).  These routines will not insert data
**        if this problem arises (much like the keyboard buffer will not insert
**        any more data into it's queue or buffer if it is full.
**
**  CreateQ() Creates a Queue and returns a pointer to it's structure info
**  DestroyQ() Removes from memory a Queue created with the CreateQ function
**  GetFromQ() Returns a value from the Queue specified
**  InserIntoQ() Inserts passed value into specified Queue
**
**
** To compile:   TCL or QCL  -c Q.C
**
** (See QTEST.C for testing)
** Mario Giannini
*/
#include <stdio.h>
#include <malloc.h>
#include "Q.H"

int QueStat=0;  /* The global status for last Que operation */

Que *CreateQ(int Elements)
{
	Que *Tmp;

	if( (Tmp=malloc(sizeof(Que)))==NULL)
	{
		QueStat=OUTOFMEM;
		return(NULL);
	}
	if( (Tmp->BufferBeg=malloc(Elements))==NULL)
	{
		QueStat=OUTOFMEM;
		free(Tmp);
		return(NULL);
	}
	Tmp->Head=Tmp->Tail=Tmp->BufferBeg;
	Tmp->BufferEnd=Tmp->BufferBeg+Elements;
	QueStat=ALLOK;
	return(Tmp);
}

void DestroyQ(Que *Q2Destroy)
{
	free(Q2Destroy->BufferBeg);
	free(Q2Destroy);
}

int GetFromQ(Que *Que2GetFrom)
{
	int i;
	
	if(!Empty(Que2GetFrom))
	{
		i=*Que2GetFrom->Head++;
		if(Que2GetFrom->Head==Que2GetFrom->BufferEnd)
			Que2GetFrom->Head=Que2GetFrom->BufferBeg;
	}
	else
		i=QueStat=UNDERFLOW;
	return(i);
}

InsertIntoQ(Que *Q2Add2, int Value)
{
	if(Q2Add2->Tail+1==Q2Add2->Head || 
		Q2Add2->Tail==Q2Add2->BufferEnd && Q2Add2->Head==Q2Add2->BufferBeg)
		return(QueStat=OVERFLOW);
	*Q2Add2->Tail++=Value;
	if(Q2Add2->Tail==Q2Add2->BufferEnd)
		Q2Add2->Tail=Q2Add2->BufferBeg;
	return(QueStat=ALLOK);
}
