'*** QB3-Demo.Bas *********************************************************
'
'              ***  P-Screen QuickBASIC 3.0 Demo Program  ***
'
'    Information:   Rob Smetana          (415) 863-0530
'                   132 Alpine Terrace   San Francisco, CA  94117
'
'    Purpose:       Within QuickBASIC 3.0, demonstrate how to display
'                   P-Screen library screens from your QB 3.0 programs
'
'    Compatibility: QB 3.0 (and maybe 2.x but we haven't tested it)
'
'    To run this:   - Be sure P-Screen.Psl is on the "current" drive/directory
'                   - Run under QB 3.x like this:
'
'                     qb qb3-demo /l QB3-Demo.exe   QB3-Demo.exe is REQUIRED,
'                                                   unless you added the routines
'                                                   mentioned below to your own user lib.
'
'    Notes:         To display P-Screen library screens from QB 3.x programs:
'
'                   - Link the following subroutines to your programs:
'                       Int86.obj, QB3PScrn.obj, QB3LdScr.obj

'                       Int86.obj comes from Microsoft.  It's on your QB 3 disk.
'                       It contains PTR86, which we call to locate our screen array in memory.
'                       (This isn't needed in QB 4.5.)
'
'                   - CALL 3 routines (we supply some, PTR86 comes from Microsoft):
'                       QB3LoadScrn     (opens library, loads screen into array)
'                       PTR86           (locate screen array in memory)
'                       QB3CompRest     (displays FULL WIDTH compressed/normal screens)
'                          -or-
'                       QB3CompRestPlus (REGISTERED users:  display ANY size compressed/normal screen)
'
'                   Because P-Screen compresses screens, use ONLY our
'                   subroutines to display them.  These routines will
'                   ALSO display uncompressed screens -- so you may use
'                   them for ANY screen-display purposes.

' ArraySize:        CRITICAL!  Notice it's in parentheses when we call QB3LoadScrn.
'                      - Parentheses ensure QB does NOT change it
'                      - Here, we need it unchanged for EACH screen display

'                   When calling QB3LoadScrn:  ArraySize = size of array you allocated.
'                      - We check the size of each Screen against ArraySize to
'                        ensure you allocated an array large enough.  If ArraySize
'                        was too small, the assembler routine (Load Binary)
'                        would crash your program (would write over who-knows-what).
'                   On Exit from QB3LoadScrn: ArraySize = size of screen (unless you
'                        wrap ArraySize in parentheses, in which case it won't change).
'                      - If you need to know ScreenSize, initialize ArraySize
'                        before EACH call to QB3LoadScrn.

'*** QB3-Demo.Bas *********************************************************

Defint a-z                      'ALL numeric variables MUST be Integers

LibName$ = "P-SCREEN"           'BE SURE P-Screen.PSL is on the "current"
				'drive before going any further.  We check below.

On error goto SomeError         '--- run with /e AND /x ON (On Error/Resume Next)
ErrorNumber = 0

'---let's see if our screen library is here
Open LibName$+".Psl" for input as #1
    If ErrorNumber Then
       Cls:Locate 12,2
       Print " Sorry, you must run this with ";LibName$;".Psl on the current drive/directory."
       end
    end if
Close

'--- If we got this far without an error, our screen library must be around.
'    Let's display some screens.


ArraySize = 2010                'ArraySize is CRITICAL!!!  See notes.
				'2000 is enough for 80 col. by 25 row screens
				' +10 for extra margin of safety

DIM Array%(ArraySize)           'allocate screen-display array


b! = TIMER                      'time the screen displays

'--- display some full screens

ScreenName$ = "TOORDER": GOSUB FullScreen
ScreenName$ = "PS-MAIN": GOSUB FullScreen
ScreenName$ = "TOORDER": GOSUB FullScreen
ScreenName$ = "PS-MAIN": GOSUB FullScreen

'--- pause
x = 4: d$="FULL": Gosub Pause

CLS

b! = TIMER                      'time the screen displays

'--- display some partial screens

ScreenName$ = "PSCRTOP1": GOSUB PartialScreen
ScreenName$ = "PSCRFILE": GOSUB PartialScreen
ScreenName$ = "PSCRDRAW": GOSUB PartialScreen
ScreenName$ = "PSCRBLOK": GOSUB PartialScreen
ScreenName$ = "PSCREDIT": GOSUB PartialScreen
ScreenName$ = "PSCROPTN": GOSUB PartialScreen
ScreenName$ = "PSCRHELP": GOSUB PartialScreen

'--- pause
x = 7: d$="PARTIAL": Gosub Pause

END

'-------------------------------------------------------------------------
Pause:
'-------------------------------------------------------------------------

Color 23,0
    LOCATE 20, 11: PRINT "Displaying"; x; d$; " screens FROM DISK took -->"; Timer - b!; "seconds"
    LOCATE 22, 32: PRINT "press a key . . ."
Color 15,0

'...pause
d$=Input$(1)

Return
'-------------------------------------------------------------------------
FullScreen:      'Calls QB3CompRest --- displays FULL WIDTH screens only.
'-------------------------------------------------------------------------

'--- NOTE:  parentheses around ArraySize, so QB / QB3LoadScrn won't change it

CALL QB3LoadScrn(Array%(), LibName$, ScreenName$, Desc$, TopRow, TopCol, BotRow, BotCol, (ArraySize), ErrCode)

IF ErrCode < 0 THEN
   BEEP
ELSE
   CALL PTR86(Segment%, OffSet%, VARPTR(Array%(1)))
   CALL QB3CompRest(TopRow, BotRow, Segment%, OffSet%)
END IF

RETURN

'-------------------------------------------------------------------------
PartialScreen:    'Calls QB3CompRestPlus to display ANY size screen.
		  '      QB3CompRestPlus is sent to REGISTERED users.
'-------------------------------------------------------------------------

'--- NOTE:  parentheses around ArraySize, so QB / QB3LoadScrn won't change it

CALL QB3LoadScrn(Array%(), LibName$, ScreenName$, Desc$, TopRow, TopCol, BotRow, BotCol, (ArraySize), ErrCode)

IF ErrCode < 0 THEN
   BEEP
ELSE
   CALL PTR86(Segment%, OffSet%, VARPTR(Array%(1)))
   CALL QB3CompRestPlus(TopRow, TopCol, BotRow, BotCol, Segment%, OffSet%)
END IF

RETURN

'-------------------------------------------------------------------------
SomeError:        '--- returns err # if screen library not found
'-------------------------------------------------------------------------
    ErrorNumber = Err: Beep: Resume Next

'*** End of QB3-Demo.Bas **************************************************
