DECLARE FUNCTION GetTime$ ()
DECLARE SUB DivideLine (text$)
DECLARE SUB PrintLine (text$)
DECLARE SUB PrintHeader ()
'----------------------------------------------------------------------------
'                            QB_LIST.BAS, v. 1.0
'                            -------------------
'A program to list QB programs better than the QB editor lists them.
'Copyright (c) 1993 by Donald E. West, all rights reserved.
'Version 1.0 completed on June 25, 1993.
'Compiled with Microsoft QuickBASIC command line compiler and /O switch.
'----------------------------------------------------------------------------
DEFINT A-Z
COLOR 7, 1: CLS
DIM SHARED numlines, pagenum, now$, filespec$
CONST FORMFEED = 12
CONST MARGIN = 10
LPRINT CHR$(27) + CHR$(58)      'Set printer to 12 cpi.
WIDTH LPRINT 255

now$ = GetTime$
numlines = 0: pagenum = 0
filespec$ = COMMAND$

LOCATE 5, 20: PRINT "Welcome to QB_LIST, ver. 1.0"
LOCATE 7, 20: PRINT "Setting up to list "; filespec$; "."

OPEN filespec$ FOR INPUT AS #1

CALL PrintHeader
 
'---Program lines equal to or less than 78 characters are printed as written.
    'This number was chosen because 78 characters fill the QB editor screen.
    'Program lines greater than 78 characters long are divided before
    'being printed.
DO WHILE NOT EOF(1)
    LINE INPUT #1, text$
    IF LEN(text$) <= 78 THEN
        CALL PrintLine(text$)
    ELSE
        CALL DivideLine(text$)
    END IF   'LEN(text$) <= 78
LOOP

CLOSE #1

LPRINT CHR$(18)     'Reset printer to 10 cpi.

LOCATE , 20: PRINT "PROCESSING COMPLETE."
LOCATE , 20: PRINT
LOCATE , 20: PRINT "PRINTING WILL BE COMPLETE SOON."
PRINT

END   'Main program

'----------------------------------------------------------------------------
                          SUB DivideLine (text$)
'----------------------------------------------------------------------------
'Program lines arriving here are longer than 78 characters.  This procedure
'"breaks off" the first 75-character segment. If this segment was indented,
'the indent is calculated and saved.  Then the right-most blank space in that
'segment is searched for.  When found, all characters to the
'left of the final blank are printed, and characters to the right are
'replaced in the program line.  The process continues until the entire
'program line is printed.  If no blank is found in a segment,
'all 75 (less any indent) characters are printed.
'
'75 characters were chosen to allow for an identification mark to be
'placed at the beginning of lines that have been segmented.  When the
'subdivided line has been printed, the i. d. marks appear as a "[" symbol
'to the left of the group.

flag = 0        'flag = 0 signifies first pass through the DO...LOOP.
numchars = LEN(text$)       'Total number of characters in text$.
indent = 0                  'Leading blank spaces in text$.

'---Find number of leading spaces.
FOR i = 1 TO numchars
    z$ = MID$(text$, i, 1)
    IF z$ = CHR$(32) THEN
        indent = indent + 1
    ELSE
        EXIT FOR
    END IF   'z$ = CHR$(32)
NEXT

numchars = numchars - indent        'Update numchars for leading blanks.

DO WHILE LEN(text$) > 0
    IF flag = 0 THEN                         'On first pass,
        text1$ = LEFT$(text$, 75)            '"Break off" 75 characters.
        text1$ = MID$(text1$, indent + 1)    'Throw indent away.
        text$ = MID$(text$, 76)              'Save remainder of text$.
    ELSE                                     'On subsequent passes,
        text1$ = LEFT$(text$, 75 - indent)   'Break off (75 - indent) chars.
        text$ = MID$(text$, 76 - indent)     'Save remainder of text$.
    END IF   'flag = 0

    FOR i = (75 - indent) TO 1 STEP -1        'Find rightmost blank.
        x$ = MID$(text1$, i, 1)
        IF x$ = CHR$(32) THEN EXIT FOR
    NEXT

    IF i = 0 THEN                   'There are no blanks,
        text2$ = text1$             'so print the whole segment.
    ELSEIF indent + numchars <= 75 THEN    'All remaining text will fit on
        text2$ = text1$             'one line, so print it.
    ELSE                            'Otherwise, print the blank and all
        text2$ = LEFT$(text1$, i)   'characters to its left
        text$ = MID$(text1$, i + 1) + text$  'Put all characters to the right
    END IF   'i = 0                          'of the blank back into text$.

    numchars = numchars - LEN(text2$)   'Number of characters remaining to
                                        'be printed AFTER this pass.

   


    IF flag = 0 THEN                               'First time through loop.
        CALL PrintLine(SPACE$(indent) + CHR$(218) + " " + text2$)
    ELSEIF flag = 1 AND numchars = 0 THEN          'Last time through loop.
        CALL PrintLine(SPACE$(indent) + CHR$(192) + " " + text2$)
    ELSE                                           'Other times through loop.
        CALL PrintLine(SPACE$(indent) + CHR$(179) + " " + text2$)
    END IF   'flag = 0
   
    flag = 1
   
LOOP

END SUB   'DivideLine

'----------------------------------------------------------------------------
                          FUNCTION GetTime$ STATIC
'----------------------------------------------------------------------------
t$ = LEFT$(TIME$, 5)

IF VAL(t$) = 0 THEN
    t$ = "12" + RIGHT$(t$, 3) + " A.M."
ELSEIF VAL(t$) < 12 THEN
    t$ = RIGHT$(STR$(VAL(t$)), 2) + RIGHT$(t$, 3) + " A.M."
ELSEIF VAL(t$) = 12 THEN
    t$ = t$ + " P.M."
ELSE
    t$ = RIGHT$(STR$(VAL(t$) - 12), 2) + RIGHT$(t$, 3) + " P.M."
END IF

GetTime$ = t$

END FUNCTION   'GetTime$

'----------------------------------------------------------------------------
                          SUB PrintHeader STATIC
'----------------------------------------------------------------------------
pagenum = pagenum + 1
LOCATE , 20: PRINT "Processing page"; pagenum

LPRINT TAB(MARGIN); "Listed on "; DATE$;
LPRINT TAB(90 - LEN(filespec$)); UCASE$(filespec$)
LPRINT TAB(MARGIN); "       at "; now$;
LPRINT TAB(82); "Page"; pagenum
LPRINT : LPRINT : LPRINT

END SUB   'PrintHeader

'----------------------------------------------------------------------------
                          SUB PrintLine (text$)
'----------------------------------------------------------------------------
LPRINT TAB(MARGIN); text$
numlines = numlines + 1
IF numlines MOD 55 = 0 THEN
    LPRINT CHR$(FORMFEED)
    CALL PrintHeader
END IF   'numlines MOD 55 = 0

END SUB   'PrintText

