;SBIT.INC - 17 May 1991 - Knight Software
;-----------------------------------------------------------
;Entry:   DX = Count - number bytes to write
;Return:  N/A
;Destroy: None

WriteClear PROC  NEAR
	PUSH	ES
	PUSH	DI
	PUSH	CX
	PUSH	AX
	OR	DX,DX		   ;if nothing to write stop this
	JZ	@WriteClearDone
	MOV	DI,[PixelAddress]  ;destination offset address
	MOV	ES,[VideoSegment]  ;video is at segment 0a000h
	MOV	AL,[InitColor]     ;preload init color
	MOV	CX,DX		   ;copy length in bytes to CX
	SHR	CX,1		   ;and convert to words
	JNC	@WriteClearEven    ;if odd count do the odd byte
	STOSB			   ;ES:DI = dest array		  
	JZ	@WriteClearDone    ;nothing else to write if done
@WriteClearEven:
	MOV	AH,AL		   ;extend color for word write
	REP	STOSW		   ;fast write to the screen
@WriteClearDone:
	POP	AX
	POP	CX
	POP	DI
	POP	ES
	RET
WriteClear ENDP

;-----------------------------------------------------------
;Entry:   DX = Count - number bytes to write
;Return:  N/A
;Destroy: None

WriteFillLine PROC  NEAR
	PUSH	DS
	PUSH	ES
	PUSH	SI
	PUSH	DI
	PUSH	DX
	PUSH	CX
	PUSH	BX
	PUSH	AX
	OR	DX,DX
	JZ	@WriteFillLineDone    ;if nothing to write stop this

	MOV	DI,CS:[PixelAddress]  ;destination offset address
	MOV	ES,CS:[VideoSegment]  ;video is at segment 0a000h
	LEA	SI,FillPattern        ;point to fill pattern table
	MOV	AX,CS:[PixelY]
	AND	AX,07H
	ADD	SI,AX		      ;can we do it fast?
	MOV	AL,CS:[FillForeColor] ;preload foreground color
	MOV	AH,CS:[SI]	      ;get pattern to use
	INC	AH		      ;if pattern = 0FFH, do fast
	JZ	@WriteFillLineFast
	MOV	AL,CS:[FillBackColor] ;preload background color
	DEC	AH		      ;if pattern = 00H, do fast
	JZ	@WriteFillLineFast

	MOV	CX,CS:[PixelX]	      ;gotta do it slow 
	MOV	CH,8		      ;get PixelX location
	AND	CL,07H		      ;and sync pattern to it
	SUB	CH,CL
	MOV	AH,CS:[SI]
	SHL	AH,CL
	MOV	CL,AH		      ;copy pattern to CL
	MOV	AL,CS:[FillForeColor] ;preload foreground color
	MOV	AH,CS:[FillBackColor] ;preload background color
	MOV	BX,CS:[FillPixelProc] ;load pixel calling routine

@WriteAltFillLineLp:
	TEST	CL,80H		;set flags for call
	CALL	BX		;via table proc pointer
	INC	DI		;and loop till done
	DEC	DX
	JZ	@WriteFillLineDone
	SHL	CL,1
	DEC	CH		;do next pattern bit
	JNZ	@WriteAltFillLineLp
	MOV	CL,CS:[SI]	;reload pattern byte
	MOV	CH,8		;and count length
	JMP	SHORT @WriteAltFillLineLp ;and do again

@WriteFillLineFast:
	MOV	CX,DX		   ;copy length in bytes to CX
	SHR	CX,1		   ;and convert to words
	JNC	@WriteFillLineEven ;if odd count do the odd byte
	STOSB			   ;ES:DI = dest array		  
	JZ	@WriteFillLineDone ;nothing else to write if done
@WriteFillLineEven:
	MOV	AH,AL		   ;extend color for word write
	REP	STOSW		   ;fast fill write to the screen
@WriteFillLineDone:
	POP	AX
	POP	BX
	POP	CX
	POP	DX
	POP	DI
	POP	SI
	POP	ES
	POP	DS
	RET
WriteFillLine ENDP

;-----------------------------------------------------------
;Entry:    DX = Count - number bytes to read
;	   ES:SI = Pointer to cpu memory 
;Return:   N/A
;Destorys: None

ReadBitMap PROC  NEAR
	PUSH	DS
	PUSH	SI
	PUSH	DI
	PUSH	CX
	MOV	CX,DX
	OR	CX,CX
	JZ	@ReadBitMapDone	     ;if nothing to read stop this
	MOV	DI,SI
	MOV	SI,CS:[PixelAddress] ;DS:SI = source array
	MOV	DS,CS:[VideoSegment] ;video is at segment 0a000h
	SHR	CX,1		     ;ES:DI = dest array
	JNC	@ReadBitMapEven
	MOVSB			  ;if odd count do the odd byte
	JZ	@ReadBitMapDone	  ;nothing else to draw if done
@ReadBitMapEven:
	REP	MOVSW		  ;read it from the screen
@ReadBitMapDone:
	POP	CX
	POP	DI
	POP	SI
	POP	DS
	RET
ReadBitMap ENDP

;-----------------------------------------------------------
;Entry:  DX = Count - number bytes to write
;	 ES:SI = Pointer to cpu memory 
WriteBitMap PROC  NEAR
	PUSH	DS
	PUSH	ES
	PUSH	SI
	PUSH	DI
	PUSH	DX
	PUSH	CX
	PUSH	BX
	PUSH	AX
	MOV	CX,DX
	OR	CX,CX
	JZ	@WriteBitMapDone     ;if nothing to write stop this
	MOV	AH,CS:[BitMapPixelWriteMode]
	MOV	BX,CS:[BitMapPixelProc]
	MOV	DI,CS:[PixelAddress] ;destination offset address
	PUSH	ES
	POP	DS		      ;DS:SI = source array
	MOV	ES,CS:[VideoSegment]  ;video is at segment 0a000h
	OR	AH,AH
	JZ	@WriteBitMapFast      ;if mode zero, use fast move
	MOV	AH,CS:[BitMapBackColor] ;preload background color
@WriteAltBitMapLp:
	MOV	AL,DS:[SI]	  ;get next pixel from buffer
	CMP	AL,AH		  ;is this bitmap background color?
	CALL	BX		  ;if other style write mode
	INC	SI		  ;use long call process
	INC	DI		  ;via table proc pointer
	LOOP	@WriteAltBitMapLp ;loop til done
	JMP	SHORT @WriteBitMapDone
@WriteBitMapFast:
	SHR	CX,1		 ;ES:DI = dest array
	JNC	@WriteBitMapEven ;DS:SI = source array
	MOVSB			 ;if odd count do the odd byte
	JZ	@WriteBitMapDone ;if nothing else to write all done
@WriteBitMapEven:
	REP	MOVSW		;fast block write to the screen
@WriteBitMapDone:
	POP	AX
	POP	BX
	POP	CX
	POP	DX
	POP	DI
	POP	SI
	POP	ES
	POP	DS
	RET
WriteBitMap  ENDP

;-------------------------------------------------------
;Bit map read/write entry code
;BitMapProc ptr = pointer to read/write procedure to call
;To write blockfill, call as: LEA DI,WriteFillLine; CALL DoBitMap
;To write bit map, call as:   LEA DI,WriteBitMap;   CALL DoBitMap
;To Read bit map, call as:    LEA DI,ReadBitMap;    CALL DoBitMap
;Enter:    ES:SI = cpu memory pointer
;	   CS:DI = operation procedure pointer
;          CX = start X location
;          DX = start Y location
;          AX = image width (x size)
;          BX = image height (y size)
;Return:   N/A
;Destroys: None

DoBitMap PROC NEAR
	PUSH	ES
	PUSH	DI
	PUSH	SI
	PUSH	DX
	PUSH	CX
	PUSH	BX
	PUSH	AX

	MOV	[BitMapProc],DI ;save proc ptr
	MOV	[PixelX2],AX
	MOV	[PixelX1],CX	;CX= PixelX1
	MOV	DI,DX		;DI= PixelY1
	MOV	CX,BX

@DoBitMapLp:
	MOV	[PixelY],DI		;get this scan line #
	MOV	AX,[PixelX1]
	MOV	[PixelX],AX	   	;start at left edge 

	MOV	DX,[PixelX2]
	CALL	GetPixelAddress		;locate the start adr
	CMP	DX,[PixelSegmentLength] ;if larger than segment
	JC 	@SingleBitScan		;split the scan line op
	MOV	DX,[PixelSegmentLength]	;do the first part
	CALL	WORD PTR CS:[BitMapProc];of the scan line
	MOV	AX,[PixelSegmentLength]	;start where we left off
	ADD	[PixelX],AX		;on the screen
	MOV	DX,[PixelX2]		;reduce count by previous
	SUB	DX,AX			;amount processed
	ADD	SI,AX			;adj cpu memory pointer
	CALL	GetPixelAddress		;get the new start adr
@SingleBitScan:
	CALL	WORD PTR CS:[BitMapProc];process the scan line
	ADD	SI,DX			;adj cpu memory pointer
	INC	DI			;adj PixelY pointer
	LOOP	@DoBitMapLp		;loop til all are done

	POP	AX
	POP	BX
	POP	CX
	POP	DX
	POP	SI
	POP	DI
	POP	ES
	RET
DoBitMap ENDP


