;
; file : ScrnSave.asm
;
;		This TSR, when activated with ALT-S, will save SHO files to a file
;	called 'test.sho' in the current directory. '.SHO' files consist of
; a 768 byte MCGA palette (as read from the DAC registers), width of
; the image / 8, height of the image, and 64000 bytes of screen data.
;   To display these images, just load the palette into the DACs, and
; use Microsoft's _putimage() to place the picture on the screen (since
; it's in Microsoft _putimage() format. i.e. image width / 8, height,
; and image data ).
;
;   This TSR was made to learn a little about programming a TSR that calls
; DOS functions. As far as I can see, it don't destroy anything... However
; you agree to use this code at your own risk! I will not be responsible
; for any damage, loss of data, or anything else that might go wrong!
;   This code is has been placed in the public domain, and you may do
; with it what you want.
;
;   Some of the things I would like to see are:
;
;    1) Don't allow it to run more than once (?)
;    2) Increment filenames (save as 'name00.sho', 'name01.sho' etc...)
;    3) Make sure we are in MCGA mode
;
;   What I would like to know is:
;
;    1) Am I really doing this correctly? Is there still a chance we may
;      be In-Dos or the Critical Error handler?
;
;   If you can answer this, or change this TSR to make it better, please
; upload it and let me know you changed it!
;
;   I found it REAL HARD to find information on programming TSR's that
; call DOS!! I would like to see this used as a example for other users
; who are searching for the same thing. So please help me by correcting
; any mistakes I may have made. When doing so, add your name to the
; history below, and be sure to use plenty of comments!
;
; Thank's,
;		Troy Barlow
;
;
;	Created By Troy Barlow  on 1/11/91
; Compuserve ID: 75416,3364
;
;	History:
;					Fixed bug allowing me to interrupt DOS  1/12/91, Troy Barlow
;         Started checking dos crit. error flag 2/22/91, Troy Barlow
;					Started saving as .SHO file 2/22/91 Troy Barlow & Will Boyce
;
INT_NUMBER			=		09H    				; The interrupt we will steal (Keyboard)
HOT_KEY					=   1f00H					; Hotkey scancode = Alt-S
;
R_B_DATA		SEGMENT	AT	40H				; ROM BIOS Data Area segment
	ORG					1AH									; Offest into data area for keyboard stuff
	Head		DW   	?									; Keyboard buffer HEAD
	Tail		DW		?									; Keyboard buffer TAIL
	Buffer  DW	16 DUP(?)						;	Keyboard buffer itself
	Buf_End	LABEL	WORD							; Mark the end of the buffer
R_B_DATA		ENDS

CODE_SEG	SEGMENT  'CODE'
					ASSUME	CS:CODE_SEG
					ORG			100H						; This is a .COM file

FIRST:	jmp	LOAD_PROG							; Go start the TSR

	Old_Int			LABEL	WORD
	Old_K_Int   DD		?							; Place for old Keyboard int
	Crit				LABEL	WORD
	Crit_Err		DD		?							;	Place for address of DOS crit. err. flag
	IDos				LABEL	WORD
	In_Dos			DD		?							;	Place for INDOS flag
	Notice			DB		'ScreenSave, (Alt-S)',0dH,0aH
							DB		'a TSR by Troy Barlow',0dH,0aH,'$'
	FName				DB		'test.sho',0  ; Filename to save as
	Hand				DW		?             ; Place to save the filehandle
	Palette     DB    768 DUP(?)    ; Palette buffer
	SizeX       DW    0A00H         ; Screen width * Bits per pixel
	SizeY       DW    0C8H          ; Screen height
PROG			PROC	NEAR							; This is the new keyboard handler
					ASSUME	CS:CODE_SEG

				push	ax									; Save registers
				push	bx
				push	cx
				push	dx
				push	di
				push	si
				push	ds
				push	es
				push	bp
				pushf											; Save the flags
				call	Old_K_Int						; Call the old interrupt
					ASSUME	DS:R_B_DATA
				mov		bx,R_B_DATA					; Examine the key just put in the buffer
				mov		ds,bx
				mov		bx,Tail 						; Point to current tail
				cmp		bx,Head         		;	If it's at the head, the old int deleted
				jne		NOEXIT							;	So leave
				jmp   EXIT
NOEXIT:
				sub		bx,2								;	Point to the char just read in
				cmp		bx,OFFSET	Buffer	  ; Did we undershoot the buffer?
				jae		NO_WRAP							; Nope..
				mov		bx,OFFSET	Buf_End		; Yep. Move to top of buffer
				sub		bx,2								; Now point to char just read in
NO_WRAP:
				mov		dx,[bx]							; Get that char into dx
				cmp		dx,HOT_KEY					;	Is it our hot key's scancode?
				jne		EXIT								; Nope, we gotta get out of this place...
				mov		Tail,bx							; Yep. Remove the key from buffer
				;
				;	program code goes here! Remember that DS still points to R_B_DATA!
				;
				push	cs                  ;	Set ds and es back to our code & data
				pop 	ds
					ASSUME	DS:CODE_SEG

				les		si,[In_Dos]					; See if DOS is being called
				mov		al,es:[si]          ; If the flag is 0, it's not in use
				cmp		al,0
				jnz		EXIT                ; DOS in use, exit

				les   si,[Crit_Err]				;	See if we are in the crit error handler
				mov		al,es:[si]
				cmp   al,0
				jnz   EXIT

				mov   ax,ds               ; Copy DS into ES
				mov   es,ax
				mov   dx, OFFSET Palette  ; Get color registers into ES:DX
				mov   cx,100H             ; Read 256 color registers from the DAC's
				xor   bx,bx               ; Start at color location 0
				mov   ax,1017H            ; Call read dac's
				int   10H

				mov		ah,3cH							;	Create and open file. Name in  DS:DX
				xor		cx,cx               ; Normal attributes
				mov		dx,OFFSET FName
				int		21H
				jc    EXIT

				mov		[Hand],ax           ; Save filehandle
				mov		bx,ax               ;	Move filehandle into BX
				mov		ah,40H							; Dos Write the palette and size
				mov   cx,772              ; 772 bytes
				mov   dx, OFFSET Palette  ; @ ds:dx
				int 	21H

				mov   bx,[Hand]           ; Move filehandle into BX
				mov   ah,40H              ; Dos Write the screen data
				mov		cx,0FA00H						;	64000 bytes
				push	ds                  ; @ ds:dx
				mov		dx,0A000H
				mov		ds,dx								;	MCGA Screen segment
				xor		dx,dx               ; offset 0 (top left corner)
				int		21H
				pop		ds

				mov		ah,3eH							;	DOS close
				mov		bx,[Hand]
				int 	21H

				mov		ax, 0E07H						; Sound off a BELL since we are done
				xor   bx,bx
				int		10H

				;
				; Program ends here
				;
EXIT:
				pop		bp
				pop		es									;	Restore registers
				pop		ds
				pop		si
				pop		di
				pop		dx
				pop		cx
				pop		bx
				pop		ax
				iret
PROG				ENDP

LOAD_PROG		PROC	NEAR
				mov		ah,09H							; Output a string at DS:DX
				lea		dx,Notice
				int		21H
				mov		ah,35H							; Get old int vector into ES:BX
				mov		al,INT_NUMBER
				int		21H
				mov   Old_Int,bx					; Now save it
				mov		Old_Int[2],es
				;
				; UNDOCUMENTED INT's!!, Only used in DOS 3.1+
				push	ds                  ;	Save ds
				mov   ax,5D06H						; Get address of crit. error flag to DS:SI
				int		21H
				mov		bx,ds               ;	Transfer ds to bx so we can get ds back
				pop		ds                  ;	Get ds back
				mov   Crit,si   					;	Save address
				mov		Crit[2],bx
				mov		ax, 3400H						; Get address of INDOS flag to ES:BX
				int		21H
				mov  	IDos,bx
				mov		IDos[2],es
				;
				;
				push	ds
				pop		es
				mov		ah,25H							; Set up our new interrupt
				mov		al,INT_NUMBER
				lea		dx,PROG             ; Address of new interrupt
				int		21H
				mov		dx,OFFSET	LOAD_PROG	; Set up everything but LOAD_PROG to
				int		27H									;	Stay resident
LOAD_PROG		ENDP
CODE_SEG		ENDS
				END	FIRST

