;SFLOOD.INC - 25 May 1991 - Knight Software
;---------------------------------------------------------
;Draw flood fill line on screen
;Enter:   DX = LineY
;	  BX = SeedX
;Returns: BX = NewLeftX 
;         CX = NewRightX
;	  DX = unchanged
DoFloodLine PROC NEAR
	MOV	AX,[StatBlock.BytesPerScanLine] ;get whole scan line
	SUB	SP,AX		;allocate array on stack
	CMP	SP,64
	JNC	@FloodStackOK
	JMP	@FloodStackError;if ran out of stack
@FloodStackOK:
	MOV	SI,SS
	MOV	ES,SI		;ES:SI points to array
	MOV	SI,SP

;read in scan line from screen
@ReadFloodLine:
	PUSH	BX		;(reg AX still has length)
	MOV	BX,1		;read only one scan line
	MOV	CX,0		;start at left edge
	LEA	DI,ReadBitMap	;on scan line Y (reg DX)
	CALL	DoBitMap	;returns scan line in ES:SI
	POP	BX

;find left edge of flood line
@FindFloodLeft:
	MOV	AL,ES:[SI+BX]	  ;find left edge for fill
	CMP	AL,CS:[SeedColor] ;if not seed, go to the right
	JNZ	@NotFFLseed	  ;until we find the seed color
	MOV	CX,BX		  ;else move pointer to left
@FFLloop:
	DEC	BX		  ;CX = Right mark
	CMP	BX,[ClipX1]       ;BX = Left mark
	JL	@FFLexit	  ;over the left edge?
	MOV	AL,ES:[SI+BX]	  ;move to the left until
	CMP	AL,CS:[SeedColor] ;we find the new left edge
	JZ	@FFLloop	  ;or left edge of screen found
@FFLexit:
	INC	BX
	JMP	@FindFloodRight	  ;all done

@NotFFLseed:
	INC	BX		  ;move to the right until
	MOV	AL,ES:[SI+BX]	  ;we find the new left edge
	CMP	AL,[SeedColor]    ;or bump into right boundry
	JZ	@FFLexit2
	CMP	BX,[RightXmark]
	JLE	@NotFFLseed
@FFLexit2:
	MOV	CX,BX		;CX= New Right mark

;find right edge of flood line
@FindFloodRight:
	CMP	BX,[RightXmark]	;if BX > right border, exit 
	JG	@FloodLineDone  ;without doing any flood fill
	CMP	BX,[ClipX2]
	JGE	@FloodLineDone	;@@@

	XCHG	BX,CX		;CX= New Right mark
@FFRloop:
	INC	BX
	MOV	AL,ES:[SI+BX]
	CMP	AL,[SeedColor]
	JNZ	@FFRexit
	CMP	BX,[ClipX2]
	JL	@FFRloop
@FFRexit:
	DEC	BX
	XCHG	BX,CX	;CX= New Right X mark, BX = Left X mark

;write the flood line to the screen
@WriteFloodLine:
	PUSH	BX
	PUSH	CX
	ADD	SI,BX		;adj memory pointer to start loc
	SUB	CX,BX		;write between BX and CX
	INC	CX
	MOV	AX,CX		;AX = width 
	MOV	CX,BX		;start at left edge mark
	MOV	BX,1		;write only one scan line
	LEA	DI,WriteFillLine;on scan line Y (reg DX)
	CALL	DoBitMap
	POP	CX
	POP	BX
	JMP	SHORT @FloodLineDone

@FloodStackError:
	MOV	CL,grNoFloodMem ;mark error and exit
	MOV	CS:[StatBlock.stat],CL
	MOV	CX,BX

	MOV	byte ptr CS:[FillForeColor],1	;@@@@@@@@@@

@FloodLineDone:
	ADD	SP,[StatBlock.BytesPerScanLine]	;deallocate array from stack
	RET
DoFloodLine ENDP

;-------------------------------------------------------------
;Recursive flood fill procedure
ReFloodFill PROC NEAR
LeftXmark	EQU [BP+10]
RightXmark	EQU [BP+8]
MasterLeftX	EQU [BP+6]
MasterRightX	EQU [BP+4]
;NewLeftX  = BX
;NewRightX = CX
;LineY	   = DX
;Direction = DI

	PUSH	BP
	MOV	BP,SP		;copy SP to BP pointer
	MOV	BX,[LeftXmark]	;start from seed point
@ReFloodFillLoop:
	PUSH	DI
	CALL	DoFloodLine
	POP	DI
	CMP	BX,CX	;BX=NewLeftX ;CX=NewRightX
	JGE	@ReFloodFillExit

	PUSH	BX	;NewLeftX
	PUSH	CX	;NewRightX
	PUSH	BX
	PUSH	CX
	SUB	DX,DI	;LineY-Dir
	CMP	DX,CS:[ClipY1]
	JL	@ReOverFlood1
	CMP	DX,CS:[ClipY2]
	JG	@ReOverFlood1
	CALL	ReFloodFill
@ReOverFlood1:
	ADD	DX,DI	;restore LineY
	POP	CX	;pop two levels off stack
	POP	BX

	ADD	DX,DI			;LineY + Dir
	NEG	DI			;Neg Dir
	CMP	DX,CS:[ClipY1]		;if beyond screen
	JL	@ReOverFlood3		;don't do anymore
	CMP	DX,CS:[ClipY2]
	JG	@ReOverFlood3

	MOV	AX,[MasterLeftX]
	SUB	AX,2		;if MasterLeftX-2 > NewLeftX
	CMP	BX,AX		;don't do this
	JG	@ReOverFlood2
	PUSH	BX		;NewLeftX
	PUSH	AX		;MasterLeftX-2
	CALL	ReFloodFill
	POP	AX		;pop two levels off stack
	POP	BX
	POP	CX		;restore CX
	PUSH	CX
@ReOverFlood2:

	MOV	AX,[MasterRightX]
	ADD	AX,2		;if MasterRightX+2 > NewRightX
	CMP	AX,CX		;don't do this
	JG	@ReOverFlood3
	PUSH	AX		;MasterRightX+2
	PUSH	CX		;NewRightX
	CALL	ReFloodFill
	POP	CX		;pop two levels off stack
	POP	AX

@ReOverFlood3:
	POP	CX	;pop other two levels off stack
	POP	BX
	NEG	DI	;Restore Dir
	SUB	DX,DI	;Restore LineY

	MOV	BX,CX	;RightX = NewRightX + 2
	ADD	BX,2
	JMP	@ReFloodFillLoop

@ReFloodFillExit:
	POP	BP
	RET
ReFloodFill ENDP

