;SHARD.INC - 17 May 1991 - Knight Software
; History:
;  17 May 1991 - first release
;  14 Jul 1991 - Corrected default line draw from MoveByte to ForeByte
;**********************************************************************
;Misc init stuff this is common VGA initialization code
;It sets up various variables and tables in the system

MiscInit PROC NEAR
	PUSH	ES
	PUSH	SI
	PUSH	DI
	PUSH	DX
	PUSH	CX
	PUSH	BX
	PUSH	AX

	MOV	BX,0FFH			;Init bank select reg
	MOV	[PixelSegment],BX

	MOV	AL,0
	MOV	[ModeXflag],0		;clr modex flag
	MOV	AL,[ModeSelect]
	AND	AX,7
	LEA	BX,StatusPtrTable	;copy mode status table
	ADD	BX,AX
	ADD	BX,AX
	MOV	SI,[BX]
	MOV	CX,StatBlockSize
	LEA	DI,StatBlock
	PUSH	CS
	POP	ES
	REP	MOVSB

	LEA	BX,MoveByte
	MOV	[FillPixelProc],BX	;preset proc ptrs
	MOV	[BitMapPixelProc],BX
	LEA	BX,ForeByte
	MOV	[LinePixelProc],BX
	MOV	[TextPixelProc],BX
	MOV	AL,0
	MOV	[FillPixelWritemode],AL	;preset mode selects
	MOV	[BitMapPixelWritemode],AL
	MOV	AL,8
	MOV	[LinePixelWritemode],AL
	MOV	[TextPixelWritemode],AL

	MOV	AL,[StatBlock.ctblf]	;init colors
	MOV	[DrawForeColor],AL
	MOV	[FillForeColor],AL
	MOV	AL,0
	MOV	[DrawBackColor],AL
	MOV	[FillBackColor],AL
	MOV	[TextBackColor],AL
	MOV	[BitMapBackColor],AL

	CALL	GetTextInfo	;Load text info from BIOS

	MOV	AX,1012H
	MOV	BX,0		;Set default palette table
	MOV	CX,256
	PUSH	CS
	POP	ES
	LEA	DX,PaletteTable
	INT	10H

	POP	AX
	POP	BX
	POP	CX
	POP	DX
	POP	DI
	POP	SI
	POP	ES
	RET
MiscInit ENDP

;**********************************************************************
;Create mode string from mode name and card name
;Entry:  CL = Mode number for name
;Return: ES:BX = pointer to mode string
;        NC = valid mode; C= bad mode number
GetModeName PROC NEAR
	PUSH	SI
	PUSH	DI
	PUSH	CX
	PUSH	DX

	PUSH	CS	;return a pointer to a pascal string ES:BX
	POP	ES	;of the mode name for the mode passed in CX
	LEA	SI,ModeNameError
	MOV	DH,grInvalidMode ;Beyond mode number allowed?
	MOV	CH,0
	CMP	CX,[NumberModes]
	JNC	@GetModeName2
	MOV	DH,0
	LEA	SI,ModeName
	MOV	AL,ModeNameSize  ;get table entry size
	MUL	CL
	ADD	SI,AX		 ;index into table for name
@GetModeName2:
	MOV	CH,0
	MOV	CL,[SI]		 ;get mode name length
	INC	SI
	LEA	DI,ModeNameBuffer
	INC	DI		 ;Point to target buffer
	MOV	DL,CL		 ;save count
	REP	MOVSB		 ;copy mode name to buffer
	MOV	SI,[CardNamePtr] ;get card name buffer address
	MOV	CL,[SI]		 ;get length of card name
	INC	SI
	ADD	DL,CL		 ;add to overall name length
	REP	MOVSB		 ;add card name to mode name
	MOV	BYTE PTR [DI],0  ;tag with a null
	LEA	BX,ModeNameBuffer;return pointer to name string 
	MOV	[BX],DL		 ;set length

	POP	DX
	POP	CX
	POP	DI
	POP	SI
	RET
GetModeName ENDP


;**********************************************************************
;Take the pixel position specified by PixelX, PixelY and the
;scan line length specified by BytesPerScanLine and compute
;the long address into the display. Selects proper segment bank
;on the display and stuffs bank select into PixelSegment and 
;the Display Offset address into PixelAddress. Returns how many 
;bytes are left in the display segment from the specified address
;in PixelSegmentLength.
;Entry:   N/A (see description above)
;Return:  N/A
;Destroy: None

GetPixelAddress PROC NEAR
	PUSH	AX
	PUSH	BX
	PUSH	DX
	PUSH	SI
	MOV	AX,[StatBlock.BytesPerScanLine] ;bytes per scan line
	MOV	DX,[PixelY]           ;PixelY
	MUL	DX                    ;dx:ax=beginning of scan line
	MOV	SI,AX
	ADD	SI,[PixelX]           ;si=address
	ADC	DX,0                  ;dx=bank
	CMP	[PixelSegment],DX     ;set the bank select
	JE	@DoneBankSelect
	MOV	[PixelSegment],DX
	CALL	CS:[BankSelectProc]   ;call approprate bank sel code
@DoneBankSelect:
	MOV	[PixelAddress],SI     ;save the pixel address
	NOT	SI
	INC	SI
	JNZ	@BankSelectOK
	DEC	SI		;limit length to 65535
@BankSelectOK:
	MOV	[PixelSegmentLength],SI  ;save the segment length
	POP	SI
	POP	DX
	POP	BX
	POP	AX
	RET
GetPixelAddress ENDP


;**********************************************************************
;Standard VGA only has 320x200 which fully fits in 64K bank.
;Thus no selection needs to be done here.
VGABankSelect PROC NEAR
	RET
VGABankSelect ENDP


;----------------------------------------------------------------------
;Select standard VGA320x200 mode operation
;Entry:   N/A
;Return:  Correct display mode selected
;Destroy: AX

VGA320x200Init PROC NEAR
	MOV	AX,0013H	;select standard mode
	INT	10H		;call BIOS to set the mode
	RET
VGA320x200Init ENDP

