/***
*signal.h - definitions/declarations for signal values and routines
*
*                                                                        
*
*Purpose:
*   This file defines the signal values and declares the signal
*   routines.
*   [ANSI/System V]
*
*******************************************************************************/

typedef int sig_atomic_t;


#define NSIG 23     /* maximum signal number + 1 */

/* signal types */
/* SIGINT, SIGFPE, SIGILL, SIGSEGV, and SIGABRT are recognized on DOS 3.x */

#define SIGINT      2   /* interrupt - corresponds to DOS 3.x int 23H */
#define SIGILL      4   /* illegal instruction - invalid function image */
#define SIGFPE      8   /* floating point exception */
#define SIGSEGV     11  /* segment violation - invalid storage access */
#define SIGTERM     15  /* Software termination signal from kill */
#define SIGUSR1     16  /* User defined signal 1 */
#define SIGUSR2     17  /* User defined signal 2 */
#define SIGUSR3     20  /* User defined signal 3 */
#define SIGBREAK    21  /* Ctrl-Break sequence */
#define SIGABRT     22  /* abnormal termination triggered by abort call */


/* signal action codes */
/* SIG_DFL and SIG_IGN are recognized on DOS 3.x */

#define SIG_DFL (void (*)())0   /* default signal action */
#define SIG_IGN (void (*)())1   /* ignore */
#define SIG_SGE (void (*)())3   /* signal gets error */
#define SIG_ACK (void (*)())4   /* error if handler not set up */


/* signal error value (returned by signal call on error) */

#define SIG_ERR (void (*)())-1  /* signal error value */


/* function declarations for those who want strong type checking
 * on arguments to library function calls
 */

#ifdef LINT_ARGS    /* argument checking enabled */

#ifndef NO_EXT_KEYS /* extended keywords are enabled */
void (cdecl * cdecl signal(int, void (cdecl *)()))();
int cdecl raise(int);
#else           /* extended keywords not enabled */
void (*signal(int, void (*)()))();
int raise(int);
#endif  /* NO_EXT_KEYS */

#else           /* argument checking not enabled */

#ifndef NO_EXT_KEYS /* extended keywords are enabled */
void (cdecl * cdecl signal())();
int cdecl raise();
#else           /* extended keywords not enabled */
void (*signal())();
int raise();
#endif  /* NO_EXT_KEYS */

#endif  /* LINT_ARGS */
