/***
*stdlib.h - definitions/declarations for commonly used library functions
*
*
*Purpose:
*   This file contains the functions declarations for commonly used
*   library functions which either don't fit somewhere else, or, like
*   toupper/tolower, can't be declared in the normal place (ctype.h
*   in the case of toupper/tolower) for other reasons.
*   [ANSI]
*
*******************************************************************************/


#ifndef _SIZE_T_DEFINED
typedef unsigned int size_t;
#define _SIZE_T_DEFINED
#endif

#ifndef NO_EXT_KEYS /* extended keywords are enabled */
typedef int (cdecl * cdecl onexit_t)();
#else           /* extended keywords not enabled */
typedef int (*onexit_t)();
#endif  /* NO_EXT_KEYS */

/* extern definitions for all commonly used global variables, except for
 * those used by the ctime family of functions (see time.h)
 */

#ifndef NO_EXT_KEYS /* extended keywords are enabled */
extern int cdecl errno;         /* XENIX style error number */
extern int cdecl _doserrno;     /* MS-DOS system error value */
extern char * cdecl sys_errlist[];  /* perror error message table */
extern int cdecl sys_nerr;      /* number of entries in sys_errlist table */

extern char ** cdecl environ;   /* pointer to environment table */

extern unsigned int cdecl _psp; /* segment value of Program Segment Prefix */

extern int cdecl _fmode;        /* default file translation mode */

/* DOS major/minor version numbers and real/protect mode */

extern unsigned char cdecl _osmajor;
extern unsigned char cdecl _osminor;
extern unsigned char cdecl _osmode;
#else               /* external keywords not enabled */
extern int errno;           /* XENIX style error number */
extern int _doserrno;       /* MS-DOS system error value */
extern char *sys_errlist[]; /* perror error message table */
extern int sys_nerr;        /* number of entries in sys_errlist table */

extern char **environ;      /* pointer to environment table */

extern unsigned int _psp;   /* segment value of Program Segment Prefix */

extern int _fmode;          /* default file translation mode */

/* DOS major/minor version numbers and real/protect mode */

extern unsigned char _osmajor;
extern unsigned char _osminor;
extern unsigned char _osmode;
#endif  /* NO_EXT_KEYS */

/*
 * flag words used to determine REAL or PROTECTED mode on 80286 machines.
 */

#define REAL_MODE 0
#define DOS_MODE 0
#define PROTECT_MODE 1
#define OS2_MODE 1

/* function declarations for those who want strong type checking
 * on arguments to library function calls
 */

#ifdef LINT_ARGS    /* argument checking enabled */

#ifndef NO_EXT_KEYS /* extended keywords are enabled */
void cdecl abort(void);
int cdecl abs(int);
double cdecl atof(const char *);
int cdecl atoi(const char *);
long cdecl atol(const char *);
void * cdecl bsearch(const void *, const void *, size_t, size_t, int (cdecl *)());
void * cdecl calloc(size_t, size_t);
char * cdecl ecvt(double, int, int *, int *);
void cdecl exit(int);
void cdecl _exit(int);
char * cdecl fcvt(double, int, int *, int *);
void cdecl free(void *);
char * cdecl gcvt(double, int, char *);
char * cdecl getenv(const char *);
char * cdecl itoa(int, char *, int);
long cdecl labs(long);
char * cdecl ltoa(long, char *, int);
void * cdecl malloc(size_t);
onexit_t cdecl onexit(onexit_t);
void cdecl perror(const char *);
int cdecl putenv(char *);
void cdecl qsort(void *, size_t, size_t, int (cdecl *)());
int cdecl rand(void);
void * cdecl realloc(void *, size_t);
void cdecl srand(unsigned int);
double cdecl strtod(const char *, char **);
long cdecl strtol(const char *, char **, int);
void cdecl swab(char *, char *, int);
int cdecl system(const char *);
char * cdecl ultoa(unsigned long, char *, int);

#ifndef tolower         /* tolower has been undefined - use function */
int cdecl tolower(int);
#endif  /* tolower */

#ifndef toupper         /* toupper has been undefined - use function */
int cdecl toupper(int);
#endif  /* toupper */

#else           /* extended keywords not enabled */
void abort(void);
int abs(int);
double atof(const char *);
int atoi(const char *);
long atol(const char *);
void *bsearch(const void *, const void *, size_t, size_t, int (*)());
void *calloc(size_t, size_t);
char *ecvt(double, int, int *, int *);
void exit(int);
void _exit(int);
char *fcvt(double, int, int *, int *);
void free(void*);
char *gcvt(double, int, char *);
char *getenv(const char *);
char *itoa(int, char *, int);
long labs(long);
char *ltoa(long, char *, int);
void *malloc(size_t);
onexit_t onexit(onexit_t);
void perror(const char *);
int putenv(char *);
void qsort(void *, size_t, size_t, int (*)());
int rand(void);
void *realloc(void *, size_t);
void srand(unsigned int);
double strtod(const char *, char **);
long strtol(const char *, char **, int);
void swab(char *, char *, int);
int system(const char *);
char *ultoa(unsigned long, char *, int);

#ifndef tolower         /* tolower has been undefined - use function */
int tolower(int);
#endif  /* tolower */

#ifndef toupper         /* toupper has been undefined - use function */
int toupper(int);
#endif  /* toupper */

#endif  /* NO_EXT_KEYS */

#else           /* argument checking not enabled */

#ifndef NO_EXT_KEYS /* extended keywords are enabled */
void cdecl abort();
int cdecl abs();
double cdecl atof();
int cdecl atoi();
long cdecl atol();
void * cdecl bsearch();
void * cdecl calloc();
char * cdecl ecvt();
void cdecl exit();
void cdecl _exit(int);
char * cdecl fcvt();
void cdecl free();
char * cdecl gcvt();
char * cdecl getenv();
char * cdecl itoa();
long cdecl labs();
char * cdecl ltoa();
void * cdecl malloc();
onexit_t onexit();
void cdecl perror();
int cdecl putenv();
void cdecl qsort();
int cdecl rand();
void * cdecl realloc();
void cdecl srand();
double cdecl strtod();
long cdecl strtol();
void cdecl swab();
int cdecl system();
char * cdecl ultoa();

#ifndef tolower         /* tolower has been undefined - use function */
int cdecl tolower();
#endif  /* tolower */

#ifndef toupper         /* toupper has been undefined - use function */
int cdecl toupper();
#endif  /* toupper */

#else           /* extended keywords not enabled */
void abort();
double atof();
long atol();
void *bsearch();
void *calloc();
char *ecvt();
void exit();
void _exit();
void free();
char *fcvt();
char *gcvt();
char *getenv();
char *itoa();
long labs();
char *ltoa();
void *malloc();
onexit_t onexit();
void perror();
void qsort();
void *realloc();
void srand();
double strtod();
long strtol();
void swab();
char *ultoa();
#endif  /* NO_EXT_KEYS */

#endif  /* LINT_ARGS */
