;SVESA.INC - 17 May 1991 - Knight Software
;----------------------------------------------------------
;This file provides VESA support

;----------------------------------------------------------
;DetectCard detects whether the VESA support is out there.
;If VESA support is found, then the highest supported 
;256 color mode is determined. If no VESA support is found, 
;we default to standard VGA mode. 
DetectCard PROC NEAR
	PUSH	ES
	PUSH	DI
	PUSH	SI
	PUSH	DX
	PUSH	CX
	PUSH	BX
	PUSH	AX
	MOV	BX,0
	MOV	[VESAmode],BX    ;init VESA mode to zero
	MOV	[NumberModes],BX ;and max mode

	SUB	SP,256		;alloc 256 byte buffer on stack
	MOV	AX,SS
	MOV	ES,AX		;point ES:DI at stack buffer
	MOV	DI,SP
	MOV	AX,4F00H	;get VESA display info
	INT	10H
	CMP	WORD PTR ES:[DI],"EV"
	JNZ	@NoVESA		;error out if not VESA
	CMP	WORD PTR ES:[DI+2],"AS"
	JZ	@IsVESA
@NoVESA:
	JMP	@NotVESA

@IsVESA:
	LES	SI,ES:[DI+0EH]	;ES:SI = table of avail modes
	SUB	SI,2		;preadj pointer
@NextMode:
	ADD	SI,2
	MOV	AX,ES:[SI]	;get item from list
	CMP	AX,-1		;last item?
	JZ	@LastMode
	MOV	CX,VESAmodeSize	;CX=our table length
	LEA	DI,VESAmodes	;DS:DI = modes we use
	MOV	BX,1		;BX=max mode number
@ModeLoop:
	CMP	AX,[DI]		;Is the item in our list?
	JZ	@GoodMode
	ADD	DI,2		;try everything in our list
	INC	BX
	LOOP	@ModeLoop
	JMP	SHORT @NextMode	 ;go try next vesa mode
@GoodMode:
	MOV	BH,0
	CMP	BL,[ModeSelect]
	JNZ	@NotOurMode
	MOV	[VESAmode],AX ;Save VESA mode number
@NotOurMode:
	CMP	BX,[NumberModes] ;is mode higher than last one? 
	JL	@NextMode
	MOV	[NumberModes],BX ;update max mode if higher
	JMP	SHORT @NextMode  ;check on next vesa mode

@LastMode:
	MOV	CX,[VESAmode]
	OR	CX,CX		 ;if Zero is not a valid
	JNZ	@ModeOK		 ;VESA mode 
	CMP	BYTE PTR [ModeSelect],0 ;if mode zero
	JZ	@NotVESA	 ;is OK not to be VESA mode
	MOV	AH,grInvalidMode ;set error flag
	MOV	[StatBlock.stat],AH
	JMP	@NotVESA	 ;and go to default mode

@ModeOK:
	MOV	AX,SS
	MOV	ES,AX
	MOV	AX,4F01H	;get mode info
	MOV	DI,SP
	INT	10H
	MOV	CX,ES:[DI+4]	;Get window granularity
	OR	CX,CX		;if zero, not a valid value
	JZ	@NotVESA
	MOV	AX,64		;convert to 64K multiplier
	XOR	DX,DX		;for bank selects
	DIV	CX
	MOV	[WinGran],AX	;Save it for later
	OR	AX,AX
	JZ	@NotVESA	;bad window granularity

	LEA	DI,VESAbank	;everything looks good
	MOV	[BankSelectProc],DI ;so init pointers
	LEA	DI,VESAinit
	MOV	[InitDisplayProc],DI
	INC	WORD PTR [NumberModes] ;adj number modes
	MOV	BX,8		;Analog color display type 
	MOV	[DisplayType],BX
	LEA	DI,VESAname
	MOV	[CardNamePtr],DI
	XOR	AX,AX
	JMP	SHORT @DetectExit

;Couldn't find VESA, so fall back on standard VGA
@NotVESA:
	LEA	DI,VGABankSelect ;allow select mode 13H only
	MOV	[BankSelectProc],DI
	LEA	DI,VGA320x200Init
	MOV	[InitDisplayProc],DI
	MOV	BX,64		 ;default window granularity = 64K
	MOV	[WinGran],BX
	MOV	BX,0
	MOV	[ModeSelect],BL  ;def select mode
	INC	WORD PTR [NumberModes] ;adj number modes

	MOV	AX,1A00H
	INT	10H		;check if VGA type display
	CMP	AL,1AH
	JNZ	@NotVGA
	MOV	[DisplayType],BX
	CMP	BL,6		;allow MCGA or VGA support
	JC	@NotVGA
	CMP	BL,0AH		;but not digital MCGA
	JZ	@NotVGA

	LEA	DI,VGAcard
	MOV	[CardNamePtr],DI
	XOR	AX,AX
	JMP	SHORT @DetectExit

;Couldn't find VGA/MCGA, so just error out
@NotVGA:
	MOV	AH,grNoInitGraph ;set error flag
	MOV	[StatBlock.stat],AH
	LEA	DI,DummyCard
	MOV	[CardNamePtr],DI
	OR	AL,255
@DetectExit:
	ADD	SP,256	;restore stack pointer
	OR	AX,AX
	POP	AX
	POP	BX
	POP	CX
	POP	DX
	POP	SI
	POP	DI
	POP	ES
	RET
DetectCard ENDP

VESAmodeSize EQU 5	;number of modes in table
VESAmodes:
	 DW 100H	;1: 640x400x256
	 DW 101H	;2: 640x480x256
	 DW 103H	;3: 800x600x256
	 DW 105H	;4: 1024x768x256
	 DW 107H	;5: 1280x1024x256

MaxModes EQU 6	;maximum available modes we can use
VESAmode DW 0	;VESA mode we will be using

VESAname DB 7
	 DB 'VESA256',0

;----------------------------------------------------------------------
;Set screen mode to desired value
;Enter:   mode to select in VESAmode
;Return:  selected mode; Rets Z if selected, NZ if bad
;Destory: AX,BX

VESAinit PROC NEAR
	PUSH	AX
	PUSH	BX
	MOV	AX,04F02H
	MOV	BX,[VESAmode]
	INT	10H
	CMP	AX,004FH	;ret Z if good
	POP	BX
	POP	AX
	RET
VESAinit ENDP

;----------------------------------------------------------------------
;Enter:   DX = 64K bank to select
;Return:  Bank is selected 
;Destory: None

VESAbank PROC NEAR
	PUSH	AX
	PUSH	BX
	PUSH	DX
	MOV	AX,CS:[WinGran]
	MUL	DX
	MOV	DX,AX
	MOV	AX,04F05H
	MOV	BX,0
	INT	10H
	POP	AX
	POP	BX
	POP	DX
	RET
VESAbank ENDP

