'
' TBSAYGET.BAS: a Turbo Basic subroutine   version 1
' author .....: Vernon E. Davis  [71330,2705]
' bug fix ....: correction for insert routine - Lee M. Bernbaum 6/15/87
' purpose ....: emulates the dBASE @..SAY..GET..READ statement that
'             : allows bullet-proof string input from the keyboard.
' syntax .....: Call SAYGET( row, colunm, saystring, getstring )   where
'             :   row       = 1 to 25
'             :   colunm    = 1 to 80
'             :   saystring = string to SAY on the screen
'             :   getstring = string to GET on the screen
'             :   NOTE: the entire SAYGET combination MUST fit on one line
' returns ....: the data in getstring
' notes ......: there are four shared variables that MUST be declared.
'             :   stextcolor - the text color of the normal screen.
'             :   sbackcolor - the background color of the normal screen.
'             :   gtextcolor - the text color of the GET input line.
'             :   sbackcolor - the background color of the GET input line.
'             : the following editing keys are operational:
'             :   RETURN - returns to calling routine with changes
'             :   ESC - returns to calling routine without changes
'             :   Left Arrow - moves one character left, non-destructive
'             :   Right Arrow - moves one character right, non-destructive
'             :   HOME - moves to beginning of GET line
'             :   END - moves to end of GET line
'             :   Backspace - moves one character left, destructive. If this
'             :               key is used on the last character of the GET
'             :               line, it will not erase it. Use the DEL key.
'             :   DEL - deletes character from cursor position
'             :   INS - toggles insert on and off. INS indicator appears
'             :         at row 1, column 60 when insert is on
'
' Turbo Basic is a product from Borland International
'
' dBASE is a registered trademark of Ashton-Tate
'
Sub SAYGET( ypos%, xpos%, saystr$, getstr$ )
   Shared gtextcolor%,gbackcolor%,stextcolor%,sbackcolor%
   Local gstrlen%,getbegin%,getend%,h%,i%,j%,inskey%,ch$,t0$

   inskey% = 0                                   ' insert key initially off
   gstrlen% = Len(getstr$)                       ' get GET string length
   Locate ypos%,xpos%,1 : Print saystr$;         ' print SAY string
   getbegin% = Pos                               ' store beginning screen pos
   getend% = (getbegin% + gstrlen%) - 1          ' store end screen pos
   If getend% > 80 Then getend% = 80             ' insure string fits on line
   Color gtextcolor%,gbackcolor%                 ' change to GET colors
   Locate ypos%,getbegin%,1 : Print getstr$;     ' print GET string
   Locate ypos%,getbegin%                        ' return cursor to beginning
   Do                                            '
      Do                                         '
         ch$ = Inkey$                            ' get character from kbd
      Loop Until ch$ <> ""                       '
      Select Case ch$                            ' check for these keys ...
      Case Chr$(13)                              ' ** RETURN Key **
         getstr$ = ""                            '  clear GET string
         For i% = getbegin% To getend%           '
            h% = Screen(ypos%,i%)                '  get char. from screen
            getstr$ = getstr$ + Chr$(h%)         '  and place it in GET string
         Next i%                                 '
         Color stextcolor%,sbackcolor%           '  change normal color
         Exit Sub                                '  and return to caller
      Case Chr$(0) + Chr$(75)                    ' ** Left Arrow Key **
         If Pos = getbegin% Then                 '  if at the begin of line
            Locate ypos%,getbegin%               '   remain there
         Else                                    '  else
            Locate ypos%,Pos - 1                 '   move 1 char. left
         End If                                  '
      Case Chr$(0) + Chr$(77)                    ' ** Right Arrow Key **
         Locate ypos%,Pos + 1                    '  move 1 char. right
         If Pos >= getend% Then                  '  if at the end of line
            Locate ypos%,getend%                 '   remain there
         End If                                  '
      Case Chr$(0) + Chr$(71)                    ' ** Home Key **
         Locate ypos%,getbegin%                  '  move to begin of line
      Case Chr$(0) + Chr$(79)                    ' ** End Key **
         Locate ypos%,getend%                    '  move to end of line
      Case Chr$(0) + Chr$(83)                    ' ** Del Key **
         j% = Pos                                '  store current horiz.
         t0$ = ""                                '  clear temp string
         For i% = getbegin% To getend%           '
            h% = Screen(ypos%,i%)                '  get char. from screen
            If Pos <> i% Then                    '  if not equal to horiz.
               t0$ = t0$ + Chr$(h%)              '   place it in temp string
            End If                               '
         Next i%                                 '
         t0$ = t0$ + " "                         '   place blank at end
         Locate ypos%,getbegin% : Print t0$;     '   replace string on screen
         Locate ypos%,j%                         '   and return to horiz.
      Case Chr$(0) + Chr$(82)                    ' ** Ins Key **
         If inskey% = 0 Then                     '   if ins toggle off
            inskey% = 1                          '    turn ins toggle on
            j% = Pos                             '    store current horiz.
            Color stextcolor%,sbackcolor%        '    normal colors
            Locate 1,60,0 : Print "INS";         '    say that INS is on
            Locate ypos%,j%,1                    '    return to horiz.
            Color gtextcolor%,gbackcolor%        '    and GET colors
         Else                                    '   if ins toggle on
            inskey% = 0                          '    turn ins toggle off
            j% = Pos                             '    save current horiz.
            Color stextcolor%,sbackcolor%        '    normal colors
            Locate 1,60,0 : Print "   ";         '    say that INS is off
            Locate ypos%,j%,1                    '    return to horiz.
            Color gtextcolor%,gbackcolor%        '    and GET colors
         End If                                  '
      Case Chr$(8)                               ' ** Backspace Key **
         If Pos <> getbegin% Then                '  if not at begin of line
            Locate ypos%,Pos - 1                 '   move back one space
            Print " ";                           '   print a space
            Locate ypos%,Pos - 1                 '   and move back again
         End If                                  '
      Case Chr$(27)                              ' ** Escape Key **
         Color stextcolor%,sbackcolor%           '  normal color
         Exit Sub                                '  return w/o modification
      Case " " To "~"                            ' ** Alphanumeric Keys **
         If inskey% = 1 Then                     '  if ins toggle on
            j% = Pos                             '   store current horiz.
            t0$ = ""                             '   clear temp string
            For i% = getbegin% To getend% - 1    '
               h% = Screen(ypos%,i%)             '   get char. from screen
               If Pos = i% Then                  '   if horiz.
                  t0$ = t0$ + ch$                '    add char. in temp string
               End If                            '
               t0$ = t0$ + Chr$(h%)              '    add screen chars.
            Next i%                              '

            'correction to insert routine starts here
            IF J%=GETEND% THEN                   '  if at end of GET string
               t0$=left$(t0$,getend%-1)+ch$      '    insert replaces last
            END IF                               '    character in string
            Locate ypos%,getbegin%:Print t0$;    '   replace string on screen
            IF J%=GETEND% THEN                   '   and return to horiz.
               locate ypos%,j%                   '  cursor pos if insert at end
            ELSE                                 '  otherwise advance so insert
               locate ypos%,j%+1                 '  chars don't get put in a
            END IF                               '  reverse entry order
            'end of correction for insert routine

         Else                                    '  if ins toggle off
            Print ch$;                           '   print char.
            If Pos > getend% Then                '   if at end of line
               Locate ypos%,getend%              '    remain at end of line
            End If                               '
         End If                                  '
      End Select                                 '
   Loop Until TRUE                               ' loop always
End Sub                                          ' >>>  End of SAYGET  <<<
'
'
' ***  MAIN PROGRAM  ***
'
' the main program is only a demo for the SAYGET subroutine. It is not
' required to use the SAYGET subroutine.
'
stextcolor% = 7                                  ' color of normal text
sbackcolor% = 0                                  ' color of normal background
gtextcolor% = 0                                  ' color of GET text
gbackcolor% = 7                                  ' color of GET background
Cls
Locate 3,1
Print "Test Program for SAYGET procedure.  Written by Vernon E. Davis";
Locate 9,1
Print "RETURN - returns to calling routine with changes"
Print "ESC - returns to calling routine without changes"
Print "Left Arrow - moves one character left, non-destructive"
Print "Right Arrow - moves one character right, non-destructive"
Print "HOME - moves to beginning of GET line"
Print "END - moves to end of GET line"
Print "Backspace - moves one character left, destructive. If this"
Print "            key is used on the last character of the GET"
Print "            line, it will not erase it. Use the DEL key."
Print "DEL - deletes character from cursor position"
Print "INS - toggles insert on and off. INS indicator appears"
Print "      at row 1, column 60 when insert is on"
Do
   saygetstr$ = "         "                      ' initialize GET string
   Call SAYGET(6,1,"Enter any data or Q to quit-> ",saygetstr$)
   Print : Print "The returned string SAYGET = ";
   Print Chr$(34);
   Print saygetstr$;
   Print Chr$(34);
Loop Until saygetstr$ = "q        " OR saygetstr$ = "Q        "
Print ". Program aborted.";
End
