	page	60,132
	title	TDD modem driver

; Author's notes:
; This isn't claimed to be the best that can be done, but it works.
; Anyone with enough patience can lift the Baudot convertor and patch
; it into a more versatile comms program, but I didn't think there was
; any point.

; Mod record
; June 18th: added .INI file
; June 29th: shrunk buffers, uses MALLOC to grab 64k for RAM store
;	     Total program size is now a satisfying 3k

B	equ	byte ptr
W	equ	word ptr
Addr	equ	offset
false	equ	0
true	equ	not false

; Various values to write to the 8250 to set baud rates
baud50	equ	0900H		; "International" setting
baud455	equ	09E4H		; US setting
baud45	equ	0A00H		; this one works

Com2	equ	02f8h
Com1	equ	03f8h

sseg	segment para stack 'STACK'
	db	64 dup(0)
sseg	ends

dseg	segment para public 'DATA'

LogFileBuffer	label	byte
	db	256 dup(0)

SendFileBuffer		label	byte
	db	256 dup(0)

OutBuffer		label	byte
	db	256 dup(0)	; Baudot goes here before transmission

InBufferPointer		label	word
	dw	0		; used for putting Baudot in

OutBufferPointer	label	word
	dw	0		; used for taking it out

LogFilePointer		label	word
	dw	0

SendFilePointer		label	word
	dw	0		; this (and Count, below) used to
				; keep track when you pause and resume
SendFileCount		label	word
	dw	0

IniFileHandle		label	word
	dw	0

LogFileHandle		label	word
	dw	0

SendFileHandle		label	word
	dw	0

SendFileName		label	byte
	db	42 dup(0)

IniFileName		label	byte
	db	"TDD.INI",0

NewFile			label	byte
	db	0			; set NZ if anything changes
					; causes a rewrite of TDD.INI

; ----- Block saved/restored from .INI file -----
; The following block is loaded from a file called TDD.INI, if it
; exists in the current directory. The only reason for this is to
; be able to change the COM port without having to hack the code or
; reassemble the file.

LogFileName	label	byte
	db	0,7,"TDD.LOG",33 dup(0)

BaudRate	label	word
	dw	Baud45

ComPort		label	word
	dw	Com2

ComStatus	label	word
	dw	Com2+5

; ----- End of .INI block -----

IniFileLength	equ	$-LogFileName

ShiftState		label	byte
	db	0		; keeps track of LTRS/FIGS

MeOrYou			label	byte
	db	0		; 0 when I'm talking

CharacterCount		label	byte
	db	0		; used to insert CRLF

BcdAcc			label	byte
	db	0,0,0,0,0,0	; used to make printable date/time

; ASCII to Baudot lookup table. Bits 0-4 are Baudot data, bit 5
; is Letters Shift (LTRS), bit 6 is Figures Shift (FIGS), neither
; of which are transmitted, but if the looked-up shift disagrees
; with the current shift state a shift character is inserted in
; the output buffer. Both shifts high (60H) means the character is
; the same in both shifts (eg, Space) and supresses any change.
; TAB converts to space - hitting TAB on the keyboard switches shifts.
; Backspace is Baudot NULL
TableAtoB	label	byte
	db	00h,00h,00h,00h,00h,00h,00h,00h		; 00-07
	db	60h,64h,00h,00h,00h,68h,00h,00h		; 08-0f
	db	00h,00h,00h,00h,00h,00h,00h,00h		; 10-17
	db	00h,00h,00h,00h,00h,00h,00h,00h		; 18-1f
	db	64h,4dh,51h,54h,49h,00h,5ah,45h		; 20-27
	db	4fh,52h,00h,00h,4ch,43h,5ch,5bh		; 28-2f
	db	56h,57h,53h,41h,4ah,50h,55h,47h		; 30-37
	db	46h,58h,4eh,5eh,00h,00h,00h,59h		; 38-3f
	db	00h,23h,39h,2eh,29h,21h,2dh,3ah		; 40-47
	db	34h,26h,2bh,2fh,32h,3ch,2ch,38h		; 48-4f
	db	36h,37h,2ah,25h,30h,27h,3eh,33h		; 50-57
	db	3dh,35h,31h,00h,00h,00h,00h,00h		; 58-5f
	db	00h,23h,39h,2eh,29h,21h,2dh,3ah		; 60-67
	db	34h,26h,2bh,2fh,32h,3ch,2ch,38h		; 68-6f
	db	36h,37h,2ah,25h,30h,27h,3eh,33h		; 70-77
	db	3dh,35h,31h,00h,00h,00h,00h,60h		; 77-7f

; Baudot to ASCII lookup table. If LTRS is active, the codes 20-3F
; are used, else the codes 00-1F. NULL is Backspace.
TableBtoA	label	byte
	db	08h,33h,0ah,2dh,20h,27h,38h,37h		; figs 0-31
	db	0dh,24h,34h,2ch,2ch,21h,3ah,28h
	db	35h,22h,29h,32h,23h,36h,30h,31h
	db	39h,3fh,26h,2fh,2eh,00h,3bh,00h
	db	08h,45h,0ah,41h,20h,53h,49h,55h		; ltrs 0-31
	db	0dh,44h,52h,4ah,4eh,46h,43h,4bh
	db	54h,5ah,4ch,57h,48h,59h,50h,51h
	db	4fh,42h,47h,00h,4dh,58h,56h,00h

Credit$		label byte
	db	1bh,"[2JTDD Modem driver version 1.04, "
	db	"copyright (c) 1992, Peter E. Hand"
	db	0dh,0ah,"Conditions of use: give it "
	db	"to anyone who wants it, but don't ask for money."
	db	0dh,0ah,"TAB changes shift if line noise scrambles"
	db	" the incoming text."
	db	0dh,0ah,"Control-T sends a text file, after prompting "
	db	"for a filename .."
	db	0dh,0ah,"Control-S pauses it (on-off), ESC aborts it."
	db	1bh,"[0m"
	db	0
Dedicate$	label byte
	db	0dh,0ah,0ah,"For Debbie Jo "
	db	1bh,"[1;31m",03h,1bh,"[0m"
	db	0dh,0ah,0
Prompt$		label	byte
	db	0dh,0ah,"File to send? ",0
MemLog$		label	byte
	db	0dh,0ah,"Logging to Memory",0
NoMem$		label	byte
	db	0dh,0ah,"Not enough memory for log.",0
LogAsk$		label	byte
	db	0dh,0ah,1bh,"[0mSave log to disk (Y/N) ? ",0
LogPrompt$	label	byte
	db	0dh,0ah,"Log File Name - <Enter> for ",0
lp1$		label	byte
	db	" ... ",0	
NoLog$		label	byte
	db	0dh,0ah,"Log is OFF",0dh,0ah,0
IsLog$		label	byte
	db	0dh,0ah,"Log is ON",0dh,0ah,0
Nofile$		label	byte
	db	0dh,0ah,"Cannot open file",0dh,0ah,0
Endfile$	label	byte
	db	0dh,0ah,"File Sent",0dh,0ah,0
Abort$		label	byte
	db	0dh,0ah,"File cancelled",0dh,0ah,0
Pause$		label byte
	db	0dh,0ah,"File Paused",0dh,0ah,0
Resume$	label	byte
	db	0dh,0ah,"Resuming file ..",0dh,0ah,0
Info1$		label	byte
	db	0dh,0ah,"Modem on COM ",0
Info2$		label	byte
	db	", baud rate ",0
Info3$		label	byte
	db	"45 (normal)",0
Info4$		label	byte
	db	"45.5",0
Info5$		label	byte
	db	"50 (international)",0
Reverse$	label	byte
	db	1bh,"[7m",0		; ANSI reverse video
Mon$		label	byte
	db	"Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec "
Day$		label	byte
	db	"Sun Mon Tue Wed Thu Fri Sat "

dseg	ends

cseg	segment	para public 'CODE'
	assume	cs:cseg,ds:dseg

start	proc	near
	mov	ax,dseg
	mov	ds,ax
	call	PrintTitle
	call	ReadIniFile
	call	ShowInfo
	call	SetBaud
	call	CreateLogFile
	call	PrintString
	call	GetDateTime
	call	MainLoop
	call	GetDateTime
	call	CloseLog
;	call	WriteIniFile		; no, let's not do this ..
	mov	ax,00ebh		; reset COM to 9600 8 N 1
	mov	dx,0
	cmp	[ComPort],com2
	jnz	notcom2
	mov	dx,1
notcom2:
	int	14h
	mov	ah,4ch
	int	21h
start	endp

; Announce us
PrintTitle	proc	near
	mov	dx,Addr Credit$
	call	PrintString
	mov	dx,Addr Dedicate$
	call	PrintString
	ret
PrintTitle	endp

; Show com port, baud rate on startup
ShowInfo	proc	near
	mov	dx,Addr Info1$
	call	PrintString
	mov	dl,'1'
	cmp	[ComPort],Com1
	jz	sinf1
	mov	dl,'2'
sinf1:	mov	ah,2
	int	21h
	mov	dx,Addr Info2$
	call	PrintString
	mov	ax,[BaudRate]
	mov	dx,Addr Info5$
	cmp	ax,Baud50
	jz	sinf2
	mov	dx,Addr Info4$
	cmp	ax,Baud455
	jz	sinf2
	mov	dx,Addr Info3$
sinf2:	call	PrintString
	ret
ShowInfo	endp

; Set baud rate, 5 bits, no parity, 1.5 stop bits
SetBaud		proc	near
	mov	dx,[ComPort]
	push	dx
	inc	dx
	inc	dx
	inc	dx
	mov	al,80h
	out	dx,al
	pop	dx
	mov	ax,[BaudRate]		; divisor for 8250
	out	dx,al
	inc	dx
	mov	al,ah
	out	dx,al
	inc	dx
	inc	dx
	mov	al,00000100b
	out	dx,al
	inc	dx
	mov	al,0			; RTS, DTR true
	out	dx,al
	inc	dx
	mov	[ComStatus],dx
	ret
SetBaud		endp

MainLoop	proc	near
	call	TrySerialOut		; send anything in buffer
	mov	ah,1			; any keyboard character?
	int	16h			; BIOS int 16H
	jz	TrySerialIn		; nothing there
	mov	ah,0			; if so get it
	int	16h
	cmp	al,1bh			; Esc?
	je	EscapeFound
	cmp	al,14h			; Control-T?
	jz	TransmitFile
	cmp	al,13h			; Control-S?
	jz	ResumeFile
	cmp	al,09			; Tab?
	jnz	NotSpecial
	mov	ah,[ShiftState]
	xor	ah,60h			; swap shifts
	mov	[ShiftState],ah
	jmp	MainLoop
NotSpecial:
; insert CR on first space after 64th char
	inc	B[CharacterCount]
	cmp	al,' '			; space?
	jnz	NotSpace
	cmp	B[CharacterCount],64	; time for CRLF yet?
	jc	NotSpace		; nope
	mov	al,0dh			; else do local CRLF
	call	DisplayCharacter
	mov	al,' '
NotSpace:
	push	ax
	call	ConvertAscii		; put Baudot in OutBuffer
	pop	ax
	jz	MainLoop		; no equivalent
	call	DisplayMyCharacter	; local echo
	jmp	MainLoop
TrySerialIn:
	mov	dx,[ComStatus]		; anything in the receive reg?
	in	al,dx
	and	al,1
	jz	MainLoop		; nope
	mov	dx,[ComPort]		; yes, read it
	in	al,dx
	and	al,01fh			; strip to 5 bits
	call	ConvertBaudot		; convert to ASCII
	jz	MainLoop		; zero, unrecognised character
	call	DisplayYourCharacter	; display and stack in buffer
	jmp	MainLoop
EscapeFound:
	mov	al,0dh			; exit with CRLF
	call	DisplayCharacter
	ret
TransmitFile:
	call	DoSendFile
	jmp	MainLoop
ResumeFile:
	test	[SendFileHandle],0ffffh
	jz	MainLoop		; not open, do nothing
	mov	dx,addr Resume$
	call	PrintString
	mov	bx,[SendFilePointer]
	mov	cx,[SendFileCount]
	call	ContinueSendFile
	jmp	MainLoop
MainLoop	endp

; If there's any character waiting in OutBuffer, and the 8250
; transmit buffer register is empty, send it
TrySerialOut	proc	near
	mov	ax,[InBufferPointer]	; are these the same?
	sub	ax,[OutBufferPointer]
	jnz	CharWaiting		; if not, there's a character
	ret
CharWaiting:
	mov	dx,[ComStatus]		; is the transmit buffer empty?
	in	al,dx
	and	al,20h
	jnz	SendChar		; seems so
	ret				; not ready to send
SendChar:
	mov	bx,[OutBufferPointer]
	mov	si,addr OutBuffer
	mov	al,[bx+si]		; get the (next) character
	inc	bl			; inc pointer mod 256
	mov	[OutBufferPointer],bx
	mov	dx,[ComPort]
	or	al,20h			; set 6th bit high (redundant)
	out	dx,al			; send the character
	and	al,al			; set NZ flag
	ret
TrySerialOut	endp

; Convert ASCII in AL into Baudot in OutBuffer, inserting shifts a/r
ConvertAscii	proc	near
	and	al,7fh
	mov	bx,addr TableAtoB
	xlat				; get char from table
	and	al,al			; make sure it's not zero
	jnz	CharOutOK
	ret
CharOutOK:
	mov	bx,[InBufferPointer]	; get ready to stuff it
	mov	si,addr OutBuffer
	mov	ah,[ShiftState]		; but first check the shift
	and	ah,al			; AND Baudot in AL with Shift
	jnz	NoNeedToShift		; same shift, ok
	mov	ah,1bh			; figures shift
	test	al,20h			; letters shift needed?
	jz	SendFigs
	mov	ah,1fh			; letters shift
SendFigs:
	mov	[bx+si],ah		; put shift char in buffer
	inc	bl
	mov	ah,al			; the real char to send
	and	ah,60h			; extract shifts
	mov	[ShiftState],ah
NoNeedToShift:
	and	al,1fh			; strip off shift bits
	mov	[bx+si],al		; save char for tx
	inc	bl
	mov	[InBufferPointer],bx
	and	ah,ah			; reset non-zero in case BL wrapped
	ret
ConvertAscii	endp

; Convert Baudot in AL to ASCII. All Baudot letters convert to
; Upper Case. Null converts to BackSpace
ConvertBaudot	proc	near
	and	al,1fh			; strip to 5 bits
	cmp	al,1fh			; check for letters shift ..
	jz	LtrsShift
	cmp	al,1bh			; then figures shift ..
	jz	FigsShift		; handle these seperately
	mov	ah,[ShiftState]		; else add the current shift
	and	ah,20h			; to the Baudot in AL
	or	al,ah			; so if it's ltrs, AL=AL+20H
	mov	bx,addr TableBtoA	; and if it's figs, AL=AL
	xlat				; look it up
	and	al,al			; and see if it's zero
	ret
LtrsShift:
	mov	[ShiftState],20h
	jmp	SaveShift
FigsShift:
	mov	[ShiftState],40h
SaveShift:
	xor	al,al			; return zero, shifts don't print
	ret
ConvertBaudot	endp

; Two Display routines, so that we can throw CRLF when it changes over.
; Makes the output more readable.
DisplayMyCharacter	proc	near
	test	[MeOrYou],0ffh		; if 0, we're in command
	jz	StillMe
	mov	[MeOrYou],0
	push	ax
	mov	al,0dh
	call	DisplayCharacter
	pop	ax
StillMe:
	call	DisplayCharacter
	ret
DisplayMyCharacter	endp	

DisplayYourCharacter	proc	near
	test	[MeOrYou],0ffh		; if 0, we're in command
	jnz	StillYou
	mov	[MeOrYou],0ffh
	push	ax
	mov	al,0dh
	call	DisplayCharacter
	pop	ax
StillYou:
	call	DisplayCharacter
	ret
DisplayYourCharacter	endp	

; DisplayCharacter writes AL to the screen, expands CR to CRLF, and
; stuffs the output buffer too, if we're using one.
DisplayCharacter	proc	near
	cmp	al,0ah			; ignore LF
	jnz	dspc0
	ret
dspc0:	cmp	al,0dh			; CR?
	jnz	dspc1			; no
	call	dspc1			; if so send CR first,
	mov	al,0ah			; then LF
	call	dspc1
	mov	[CharacterCount],0	; mark beginning of line
	mov	al,0dh			; restore CR before we go
	ret
dspc1:	push	ax			; all chars come here
	call	StuffBuffer		; try it in the buffer
	mov	ah,2
	mov	dl,al
	int	21h
	pop	ax
	ret
DisplayCharacter	endp

; Put the character in AL either into the RAM buffer or, if there
; isn't one, see if the log file is open and if so put it there.
; If the log file buffer fills up, dump it to disk. Because I made
; a short file buffer, this happens often; unless it's using a
; RAM disk a few characters may be lost if they're incoming at this
; time. But almost inevitably the RAM buffer will be working. Disk
; logging is only for emergencies. Note: if the RAM buffer fills up
; the text goes reverse video with a few hundred characters left.
; You have to exit to save it, but heck, you've been on the line for
; at least three hours when it happens.
StuffBuffer	proc	near
	push	ax
	mov	ax,es			; are we RAM logging?
	and	ax,ax			; ie, is ES zero?
	pop	ax
	jz	stfb0			; no, try disk
	stosb				; = mov es:di,al; inc di
	cmp	di,0fc00h		; are we near the end?
	jz	SetWarning		; yeah, better warn the user
	ret
SetWarning:
	push	ax
	push	dx
	mov	dx,Addr Reverse$	; switch to reverse video
	call	PrintString
	pop	dx
	pop	ax
	ret
stfb0:	test	[LogFileHandle],0ffffh	; file open?
	jnz	stfb1
	ret				; nope
stfb1:	mov	si,Addr LogFileBuffer
	mov	bx,[LogFilePointer]
	mov	[bx+si],al		; add char to buffer
	inc	bx
	mov	[LogFilePointer],bx
	and	bx,00ffh		; pointer mod 100H
	jnz	stfb2			; if it didn't overflow
	push	ax			; but if it did,
	mov	cx,100h
	call	WriteLogFile		; dump buffer to file
	pop	ax
stfb2:	ret
StuffBuffer	endp

; Send a file on Control-T
DoSendFile	proc	near
	mov	dx,Addr Prompt$		; ask for file name
	call	PrintString
	mov	bx,Addr SendFileName
	call	GetFileName		; then get it
	jnc	FileNameOk
	mov	al,0dh
	call	DisplayCharacter
	ret
FileNameOk:
	call	OpenSendFile		; open the file
	jc	SendFileError		; if we can ..
	mov	al,0dh			; start with CRLF
	call	DisplayMyCharacter
SendFileLoop1:
	call	ReadSendFile		; read a bufferful
	jc	SendFileError
	test	ax,0ffffh
	jz	EndOfFile
	mov	cx,ax			; number of bytes read	
	mov	bx,Addr SendFileBuffer
ContinueSendFile	label	near	; it's a call entry so say near
	mov	al,[bx]			; next byte from buffer
	and	al,7fh
	inc	bx
	push	bx
	push	cx
	cmp	al,0ah			; don't send LF
	jz	CharacterSent
	push	ax
	call	ConvertAscii		; character to Baudot
	pop	ax			; in OutBuffer
	jz	CharacterSent
	call	DisplayMyCharacter	; local echo the file
SendFileLoop3:
	call	TrySerialOut		; send the char if we can
	mov	bx,[InBufferPointer]
	sub	bx,[OutBufferPointer]
	jnz	SendFileLoop3		; wait until it goes
CharacterSent:
	pop	cx
	pop	bx
	mov	ah,1
	int	16h			; any user key?
	jz	NoKeyDown
	mov	ah,0			; if so get it
	int	16h
	cmp	al,1bh			; Esc?
	jz	UserAbort
	cmp	al,13h			; Control-S?
	jz	UserPause
NoKeyDown:
	loop	ContinueSendFile		; count down CX
	jmp	SendFileLoop1		; then refill buffer and repeat
UserPause:
	mov	[SendFilePointer],bx
	mov	[SendFileCount],cx
	mov	dx,addr Pause$
	call	PrintString
	ret
SendFileError:
	mov	dx,Addr Nofile$
	jmp	ReportClose
UserAbort:
	mov	dx,Addr Abort$
	jmp	ReportClose
EndOfFile:
	mov	dx,Addr Endfile$
ReportClose:
	call	PrintString
	call	CloseSendFile
	mov	bx,[InBufferPointer]
	mov	[OutBufferPointer],bx
	ret
DoSendFile	endp

; Get a file name from the user - name buffer in BX
GetFileName	proc	near
	mov	al,40			; allow 40 chars inc. path
	mov	[bx],al
	push	bx
	mov	dx,bx			; file name buffer to DX
	mov	ah,10			; buffered input to [DX]
	int	21h
	pop	bx			; start of buffer
	inc	bx			; # of characters entered
	mov	al,[bx]
	and	al,al			; is it > 0?
	jnz	gsfn0			; yep
	stc
	ret				; nothing entered
gsfn0:	inc	bx			; check the name
gsfn1:	mov	al,[bx]			; look for the CR at the end
	cmp	al,0dh			; and convert to upper case
	jz	gsfn2			; while we're doing so
	cmp	al,'a'
	jc	gsfn1a
	cmp	al,'z'+1
	jnc	gsfn1a
	sub	al,20h
	mov	[bx],al
gsfn1a:	inc	bx
	jmp	gsfn1
gsfn2:	mov	al,0			; when we find it,
	mov	[bx],al			; terminate name with 00H
	clc
	ret
GetFileName	endp

; Try and open the requested file for transmission
OpenSendFile	proc	near
	mov	[SendFileHandle],0	; clear handle first
	mov	dx,Addr SendFileName+2
	mov	ax,3d00h		; open for read
	int	21h
	jc	opnf1			; can't be done
	mov	[SendFileHandle],ax	; else save handle
opnf1:	ret
OpenSendFile	endp

CloseSendFile	proc	near
	mov	bx,[SendFileHandle]
	call	CloseFile
	mov	[SendFileHandle],0
	ret
CloseSendFile	endp

; Try and open INI file; read it if we succeed
ReadIniFile	proc	near
	mov	[IniFileHandle],0	; clear handle first
	mov	dx,Addr IniFileName
	mov	ax,3d00h		; open for read
	int	21h
	jc	rif1			; can't be done
	mov	[IniFileHandle],ax	; else save handle
	mov	bx,ax			; file handle
	mov	cx,IniFileLength
	mov	dx,Addr LogFileName
	mov	ah,3fh			; read it
	int	21h
rif1:	mov	bx,[IniFileHandle]
	call	CloseFile
	ret
ReadIniFile	endp

WriteIniFile	proc	near
	test	[NewFile],0ffh		; is anything changed?
	jnz	wif0			; if not, don't bother
	ret
wif0:	mov	dx,Addr IniFileName
	mov	cx,0
	mov	[IniFileHandle],cx
	mov	ah,3ch			; create/truncate
	int	21h
	jc	wif1			; failed
	mov	[IniFileHandle],ax
	mov	bx,ax
	mov	dx,Addr LogFileName
	mov	cx,IniFileLength
	mov	ah,40h			; write data
	int	21h
wif1:	mov	bx,[IniFileHandle]
	call	CloseFile
	ret
WriteIniFile	endp

CloseFile	proc	near
	and	bx,bx			; don't close unless open
	jnz	clf0			; (ie handle |= 0)
	ret
clf0:	mov	ah,3eh			; close file
	int	21h
	ret
CloseFile	endp

; Read in 256 byte blocks - the length of the buffer
ReadSendFile	proc	near
	mov	bx,[SendFileHandle]
	mov	cx,256
	mov	dx,Addr SendFileBuffer
	mov	ah,3fh
	int	21h
	ret
ReadSendFile	endp

; Open a memory log - if this fails, open and clear a disk file,
; though it's almost inconceivable that there isn't 64k handy
CreateLogFile	proc	near
	mov	[LogFileHandle],0
	call	GrabMemory		; ask for 64k
	jnc	MemLogOn		; we got it

AskLogFile	label	near		; used later to save mem log
	mov	dx,Addr LogAsk$		; do you want to?
	call	PrintString
	call	GetYesNo
	jc	creax			; no you don't
	mov	dx,Addr LogPrompt$	; yes you do, so
	call	PrintString		; ask for a name
	mov	dx,Addr LogFileName+2	; offer the default name
	call	PrintString
	mov	dx,Addr lp1$
	call	PrintString
	mov	al,[LogFileName].2	; we have a default, so
	push	ax			; save its length
	mov	[NewFile],0ffh		; mark new in anticipation
	mov	bx,Addr LogFileName
	call	GetFileName		; will this name do, or will
	pop	ax			; you enter one?
	jnc	crea0			; one was entered
	mov	[LogFileName].2,al	; nothing entered so restore length
	mov	[NewFile],0		; of default name and mark not new
crea0:	mov	dx,Addr LogFileName+2	; try and open <filename>
	mov	ah,3ch
	mov	cx,0
	int	21h
	jnc	crea1			; it opened ok
	mov	dx,Addr NoFile$		; it failed
	call	PrintString
	jmp	CreateLogFile		; then ask again
crea1:	mov	[LogFileHandle],ax	; save handle
	mov	ax,0			; initialise pointer
	mov	[LogFilePointer],ax
	mov	dx,Addr IsLog$		; say log is on
	clc				; (after we return)
	ret
creax:	mov	dx,Addr NoLog$		; say log is off
	stc
	ret
MemLogOn:
	mov	dx,addr MemLog$
	ret
CreateLogFile	endp

; If the log file is open, flush and close it. If the memory log
; is open, ask if user wants to save it, if no, exit; if yes, get
; log file name and do it.
CloseLog	proc	near
	mov	bx,[LogFileHandle]	; is the log file open?
	and	bx,bx			; don't close unless it is
	jz	CloseMem
	call	CloseLogFile
	ret
CloseMem:				; close the memory log
	push	ax
	mov	ax,es			; are we RAM logging?
	and	ax,ax			; ie, is ES zero?
	pop	ax
	jnz	clsm1			; we're logging
	ret
clsm1:	call	AskLogFile		; ask if we should save it
	jnc	clsm2			; yeah, we should
	ret
clsm2:	mov	bx,[LogFileHandle]	; so inspect the log file
	and	bx,bx			; it should be open now
	jnz	clsm3			; if the handle's zero
	ret				; something went wrong
clsm3:	push	bx
	mov	dx,0
	mov	cx,di			; LogBufferPointer
	push	ds
	push	es			; copy ES to DS temporarily
	pop	ds
	mov	ah,40h			; write CX bytes from DS:DX
	int	21h
	pop	ds			; restore DS
	pop	bx			; and handle
	mov	ah,3eh
	int	21h			; close the file
	ret
CloseLog	endp

CloseLogFile	proc	near
	mov	cx,[LogFilePointer]
	and	cx,cx			; anything to save?
	jz	clsf1			; nope, just close
	call	WriteLogFile		; cx already has the count
clsf1:	mov	ah,3eh			; close file
	int	21h
	ret
CloseLogFile	endp

; Write [CX] bytes to the log file from its buffer
WriteLogFile	proc	near
	mov	bx,[LogFileHandle]
	and	bx,bx			; is it open?
	jnz	wrf1
	ret
wrf1:	mov	dx,Addr LogFileBuffer
	mov	ah,40h
	int	21h
	mov	ax,0
	mov	[LogFilePointer],ax
	ret
WriteLogFile	endp

; Get a Y/N response from the keyboard and ignore everything else
GetYesNo	proc	near
	mov	ah,0
	int	16h
	and	al,5fh
	cmp	al,"Y"
	jz	GotYes
	cmp	al,"N"
	jz	GotNo
	jmp	GetYesNo
GotYes:	clc
	ret
GotNo:	stc
	ret
GetYesNo	endp

; Read the system date and time, convert to displayable form, print
; to screen and put it in the buffer
GetDateTime	proc	near
	mov	al,0dh		; start with CRLF
	call	DisplayCharacter
	mov	ah,2ah
	int	21h
	push	cx		; year
	push	dx		; month/day
	mov	bx,Addr Day$	; list of days in English
	xor	ah,ah		; day number 0-6 in AX
	add	ax,ax
	add	ax,ax
	add	bx,ax		; point to day
	call	Stuff4		; print the day
	pop	dx		; recover day/month
	mov	ax,dx
	xor	ah,ah
	call	BinaryBCD	; numeric Day hex to decimal
	call	Stuff2		; print it
	mov	al,' '
	call	DisplayCharacter
	mov	al,dh		; month
	xor	ah,ah
	dec	al
	add	ax,ax
	add	ax,ax
	mov	bx,Addr Mon$	; list of months in English
	add	bx,ax
	call	Stuff4		; print it
	pop	ax		; pop year into AX
	call	BinaryBCD
	mov	bx,Addr BcdAcc+1
	call	Stuff4
	mov	al,','
	call	DisplayCharacter
	mov	al,' '
	call	DisplayCharacter
; now do time
	mov	ah,2ch
	int	21h
	mov	al,ch		; hours first
	xor	ah,ah
	call	BinaryBCD	; convert to decimal
	call	Stuff2		; print it
	mov	al,':'
	call	DisplayCharacter
	mov	al,cl		; then minutes
	xor	ah,ah
	call	BinaryBCD
	call	Stuff2		; print it
	mov	al,0dh		; CRLF to end up
	call	DisplayCharacter	
	ret
GetDateTime		endp

; Move two characters (date, time etc) from BCDACC to screen/buffer
Stuff2		proc	near
	mov	al,[BcdAcc].3
	call	DisplayCharacter
	mov	al,[BcdAcc].4
	call	DisplayCharacter
	ret
Stuff2		endp

; Move four characters (day, month etc) from [BX] to screen/buffer
Stuff4		proc	near
	push	cx
	mov	cx,4
s41:	mov	al,[bx]
	push	bx
	call	DisplayCharacter
	pop	bx
	inc	bx
	loop	s41
	pop	cx
	ret
Stuff4		endp

; Binary to BCD routine - takes 16-bit number in AX, converts to
; 5 printable decimal digits in BcdAcc. Used to convert time and date.
BinaryBCD	proc	near
	push	bx
	push	cx
	push	dx
	mov	bx,Addr BcdAcc		; for bcd output
	mov	cx,10000		; first digit divisor
	mov	dx,0			; zero top 16 bits
	div	cx			; 10 thousands
	or	al,'0'			; convert to ASCII
	mov	[bx],al			; in BcdAcc
	inc	bx
	mov	ax,dx			; remainder to AX
	xor	dx,dx
	mov	cx,1000
	div	cx			; thousands
	or	al,'0'
	mov	[bx],al
	inc	bx
	mov	ax,dx			; remainder to AX
	mov	cl,100			; byte from here on
	div	cl			; hundreds
	or	al,'0'
	mov	[bx],al
	inc	bx
	mov	al,ah
	xor	ah,ah
	mov	cl,10			; tens
	div	cl
	or	al,'0'
	or	ah,'0'
	mov	[bx],al			; tens in AL
	inc	bx
	mov	[bx],ah			; units in AH
	pop	dx
	pop	cx
	pop	bx
	ret
BinaryBCD	endp

; Print a string terminated with zero
PrintString	proc	near
	push	bx
	mov	bx,dx
ps1:	mov	dl,[bx]		; next char to DL
	inc	bx
	and	dl,dl		; is it zero (00H)?
	jz	psx		; yes, we're done
	mov	ah,2		; else send it to CON
	push	bx
	int	21h
	pop	bx
	jmp	ps1
psx:	pop	bx
	ret
PrintString	endp

GrabMemory	proc	near
	mov	ax,0
	mov	es,ax			; set ES to zero .. then later
	mov	di,ax			; if it's NZ we know we're
	mov	ah,48h			; logging to memory
	mov	bx,1000h		; ask for 64k
	int	21h
	jc	NotEnoughMemory		; can't believe this will happen
	mov	es,ax			; ES:DI is now the log pointer
	ret
NotEnoughMemory:
	mov	dx,addr nomem$
	call	PrintString
	stc
	ret
GrabMemory	endp

cseg	ends
	end	start

; To assemble:

;	MASM TDD;
;	LINK /CP:1 TDD;
; Note: the /CP:n switch tells the linker to ask for "n" 16-byte paragraphs
; of memory at execution time. If it isn't explicitly told what to do, it
; asks for the whole of available memory. That means there isn't enough
; left over to satisfy the GrabMemory function, unless we first execute a
; ReleaseMemory function. /CP:1 is a lazy-man cheat which avoids having
; to do that .. it also means we don't have to work out how much memory
; the program really needs. It's less than is needed to load the code, so
; MSDOS only allocates enough for the .EXE file and no more. Our
; subsequent request for extra memory now works.
