      DEFINT A-Z

' Demonstrates the use of the DirInfo MASM function.  You'll need to create a
' .QLB file appropriate to your particular QB version.  There are two sets of
' files, DIRINFO and DIRINF7.  DIRINFO is for use with QB4/4.5/BC6 and will
' also work with BC7 using only NEAR strings.  DIRINF7 is for use with BC7 FAR
' or NEAR strings and for the QBX IDE.  I haven't tested any of this using QBX
' with extended or expanded memory, so I'd appreciate any feedback you might
' have on them.
'
' It's interesting to change the ATTR passed to DirInfo, and to uncomment the
' lines which check for an exact match.  By searching for subdirectories, it's
' possible to start at the root directory and read each directory on a hard
' disk into a separate array.  For floppies, it's wise to use ON ERROR, since
' DirInfo contains no Critical Error handler.
'
' Please read the .DOC file and the .ASM comments for more information.
'
' For comments or complaints, contact me on CompuServe MSSYS.
'
' Jim Mack, CIS ID 76630,2012
'

      DECLARE FUNCTION DirInfo% (spec$, ATTR%, Array() AS ANY)
      DECLARE FUNCTION GiveDate$ (a%)
      DECLARE FUNCTION GiveTime$ (a%)
      DECLARE FUNCTION GiveSize$ (s&)

      TYPE DirInfoType

        atr    AS    INTEGER
        tim    AS    INTEGER
        dat    AS    INTEGER
        siz    AS    LONG
        nam    AS    STRING * 12

      END TYPE

      DECLARE SUB SortDir (Arry() AS DirInfoType, number%)

      DIM Temp AS DirInfoType
      REDIM DirArray(3 TO 102) AS DirInfoType

      DO
         PRINT
         LINE INPUT "File Spec ? "; spec$
         IF LEN(spec$) = 0 THEN EXIT DO

         ATTR = 0  'or maybe &H3F, etc
         Result = DirInfo(spec$, ATTR, DirArray())

         IF Result > 0 THEN
            St = LBOUND(DirArray)
            En = St + Result - 1

            'SortDir DirArray(), Result
            'see this routine for an example of how the TIME and DATE fields
            '  can be useful even if you don't display them.
           
            TotSiz& = 0
            TotTim& = 0
            Matched = 0
            PRINT
            FOR k = St TO En
               Temp = DirArray(k)
               'IF Temp.atr AND &H1F THEN

' uncomment and modify the line above and its companion below if you want to
' experiment with locating files by specific attributes.

                  Matched = Matched + 1
                  j = Temp.tim
                  TotTim& = TotTim& + j - (65536 * (j < 0))
                  'not strictly correct, but close enough
                  TotSiz& = TotSiz& + Temp.siz
                  PRINT LCASE$(Temp.nam), GiveSize(Temp.siz),
                  PRINT GiveDate(Temp.dat), GiveTime(Temp.tim)
               'END IF
            NEXT
            PRINT
            IF Matched THEN
               PRINT Matched; "files,", GiveSize(TotSiz&); " bytes, avg. time",
               TotTim& = TotTim& \ Matched
               IF TotTim& > 32767 THEN
                  TotTim& = TotTim& - 65536
               END IF
               j = TotTim&
               PRINT GiveTime(j)   'a bit of whimsy: talk about useless info!
            ELSE
               PRINT "No matching files"
            END IF
         ELSEIF Result = 0 THEN
            PRINT
            PRINT "No files matching '"; spec$; "'"
         ELSE
            PRINT
            Result = -Result
            SELECT CASE Result
            CASE 3
               PRINT "Path '"; spec$; "' not located"
            CASE 128
               PRINT "Illegal file spec '"; spec$; "'"
            CASE 256
               PRINT "Too many files for array"
            CASE 512
               PRINT "Wrong type of array passed"
            CASE ELSE
               PRINT "Code"; Result; "returned"
            END SELECT
         END IF
      LOOP

      END

FUNCTION GiveDate$ (a)

' This shows one way you might format the date for display purposes.
' The date value is computed by DOS as:
'
'            ((Year - 80) * 512) + (Month * 32) + Day
'
' ...so just do this in reverse to extract the information you want.

CONST months = "-Jan--Feb--Mar--Apr--May--Jun--Jul--Aug--Sep--Oct--Nov--Dec-"

      d$ = LTRIM$(STR$((a \ 512) + 80))     'year
      k = a MOD 512                         'month and day
      m = k \ 32                            'month
      m = m - (m = 0) - 1                   'correct for "month 0" errors
      d$ = MID$(months, m * 5 + 1, 5) + d$  '"-Jan-80" so far
      m = k MOD 32                          'now only day remains
      m = m - (m = 0)                       'in case day is zero

      GiveDate$ = RIGHT$(STR$(m), 2) + d$   'RSET day number, as " 2-Sep-89"
 
END FUNCTION

FUNCTION GiveSize$ (s&)

      t$ = SPACE$(10)            'allow for really big file size
      RSET t$ = STR$(s&)         'right justify the digits
      GiveSize = t$

END FUNCTION

FUNCTION GiveTime$ (a)

' This shows one way the time data might be formatted for the display.
' I ignore the seconds here.  The time value is computed by DOS as:
'
'           (Hours * 2048) + (Minutes * 32) + (Seconds \ 2)
'
' ...so just do the opposite to extract the information of interest.

      k& = a - (65536 * (a < 0))  'time integer is negative sometimes
      hrs = k& \ 2048             'isolate hours
      t = (k& MOD 2048) \ 32      'isolate minutes
      t$ = ":00 am"               'assume AM...
      IF hrs > 11 THEN            'but if not, adjust to correct range
         hrs = hrs - 12
         MID$(t$, 5) = "p"
      END IF
      IF hrs = 0 THEN             '00 hrs is 12 o'clock
         hrs = 12
      END IF
      t$ = RIGHT$(STR$(hrs), 2) + t$            'RSET the hrs, but for mins..
      MID$(t$, 4 - (t < 10)) = LTRIM$(STR$(t))  '...allow a leading zero

      GiveTime$ = t$

END FUNCTION

SUB SortDir (Ary() AS DirInfoType, number%)

' a simple (and slow) sort by decreasing DATE to show
' that these fields can be useful even just as integers

      St = LBOUND(Ary)
      En = St + number% - 1
     
      PRINT "Sorting..."
      FOR k = St TO En - 1
         FOR m = k TO En
            'IF Ary(k).siz > Ary(m).siz THEN 'for increasing SIZE instead
            IF Ary(k).dat < Ary(m).dat THEN
               SWAP Ary(k), Ary(m)
            END IF
         NEXT
      NEXT

END SUB

