'=======================================================================
' Copyright Copr. (C) 1991, Sidney J. Kelly, All rights reserved
' This makes a help window that uses a Keyboard, Quick letters & Mouse
'
' The LEFT click of the Mouse on Quit exits the window.
' ESC or "Q" also quits the menu
' A LEFT or RIGHT click on the two hot spots moves the window up and down
'
' Updated 10/27/91
'=======================================================================
DEFINT A-Z
REM $INCLUDE: 'VERTM.INC'

'From this module
 ' Prints the Help Box
DECLARE SUB HelpMenu (Msg%, Init%, HorizMenuChange%)
 ' This sets up the array of help messages
DECLARE SUB InitHelp (Msg%, Init%, HelpMenus() AS HelpType)
 ' Dimenstions the Help Window based on the Message Number
DECLARE SUB SetHelpSize (Msg%)

'-----------From other modules

'Selects the Border% Elements based on KeyPressed of Border%
'Listed by Border% Number
	'No Border                                       =Border  0
	'Double Line Border% (the default for CASE else) =Border% 1
	'Single Line Border%                             =Border% 2
	'Double Horizontal Single Vertical Border%       =Border% 3
	'Double Vertical Single Horizontal Border%       =Border% 4
	'Solid Border%                                   =Border% 5
	'Double Line Border for Dialog Box               =Border% 6
'add OR 256 if want a background shadow
DECLARE SUB NormalWindow (ULR%, ULC%, LRR%, LRC%, Attr%, Border%, Label$)

'======================================================================
'Prints the Help menu
'Right Button gives superfast scrolling
'Left Button does it one at a time
'
'HotSpots:
'    Upper right for Up Arrow or Scrollup
'    Lower right for Down Arrow or Scrolldown
'    Lower Left for "[QUIT]" box
' Note: the scroll bar is visual and not operative, you must use the
'    Arrow hot spots to get something to happen
'
'Input:
'    Msg = selects a help menu
'    Init% = initializes that menu
'
'Output:
'    HorizMenuChange = if a horizontal motion key was used to exit
'
'This is a specialized Menu
'======================================================================
SUB HelpMenu (Msg%, Init%, HorizMenuChange%) STATIC

	STATIC Inits%, CurrentTop%, CurrentBottom%, Z%, OldTop%
	STATIC Times%, Row%, Col%, Button%
	STATIC HelpTop%, HelpBottom%, HelpLeft%, HelpLen%, PageHelpLen%, LenMenu%
	STATIC HaveScroll%, MidLocation%, OldToggle%, Toggle%

	'Save space by using constants
	CONST TQUIT$ = "[Quit]"
	CONST TUP$ = "": CONST TDOWN$ = ""
	CONST TTop$ = "ͻ": CONST TBot$ = "ͼ"
	CONST MScrollOn$ = ""
	CONST MScrollOff$ = "":  'CONST MScrollOff$ = ""

IF Inits% = False THEN
	CALL SetHelpSize(Msg%)
    
	'Get the basic dimensions of the outside box
	ULR% = WINDOWS(6).ULR
	ULC% = WINDOWS(6).ULC
	LRR% = WINDOWS(6).LRR
	LRC% = WINDOWS(6).LRC
	MenuItems% = WINDOWS(6).MaxMenNum
    
	REM $DYNAMIC
	DIM HelpMenus(1 TO MenuItems%) AS HelpType
	REM $STATIC
    
	Init% = False
	CALL InitHelp(Msg%, Init%, HelpMenus())
	Init% = False
    
	HelpTop% = ULR% + 1
	HelpBottom% = LRR% - 1
	HelpLeft% = ULC% + 1
	HelpLen% = HelpBottom% - HelpTop% + 1
	PageHelpLen% = HelpLen% - 1
	LenMenu% = LEN(HelpMenus(1))

END IF
Inits% = True

	'Save background area
	X% = (LRR% - ULR% + 1) * (LRC% - ULC% + 1)
	DIM Scrn4%(X%)

	'Save the background -- assumes no shadow
	CALL WINDSAVE(ULR%, ULC%, LRR%, LRC%, VARSEG(Scrn4%(0)), VARPTR(Scrn4%(0)))

	'Draw Main Box
	Border% = 6: Label$ = "Help"
	Attr% = &H70
	CALL NormalWindow(ULR%, ULC%, LRR%, LRC%, Attr%, Border%, Label$)
    
	'print a scroll bar if have more than 24 menu items
	IF MenuItems% > 24 THEN
		X% = LRR% - ULR% - 1
		SText$ = STRING$(X%, MScrollOff$)
		CALL VPRT(ULR% + 1, LRC%, SText$)
		CALL KILLSTRING(SText$)   ' delete temp string
		HaveScroll% = True       'Note that scroll bar being displayed
		MidLocation% = (LRR% - ULR%) \ 2 + ULR%    'find middle of bar
	 ELSE
		HaveScroll% = False        'else report not being displayed
	 END IF

	'Print Quit box
	CALL QPRINT(LRR%, ULC% + 3, TQUIT$, &HF)
    
	IF ColorMode% THEN
		Attr% = &H17     'White on Blue
	ELSE
		Attr% = 7        'White on Black
	END IF
    
	'This routine is used to prevent creation of temporary strings
	'in DGROUP because we grab string directly from far array
	'This speed up the printing
	Z% = False
	FOR X% = HelpTop% TO HelpBottom%
		SKIPNUM% = Z% * LenMenu%
		CALL FPRINT(X%, HelpLeft%, VARSEG(HelpMenus(1).MESSAGE), VARPTR(HelpMenus(1).MESSAGE), HelpMsgLen%, SKIPNUM%, Attr%)
		Z% = Z% + 1
	NEXT
    
	CurrentTop% = 1: CurrentBottom% = Z%
	HorizMenuChange% = False
   
	'Draw the arrows
	GOSUB AdjArrow

'Clears keyboard and mouse for testing
'keeps stale keystrokes and mouse presses from prematurely affecting
'the loop
 CALL CLRMOUSEKEY

DO

ReStart:
    
	DO
		'Read a key from the buffer.  Normal key is positive,
		'extended key is negative, no key = -256
		'Function reads normal and extended keyboards
		KeyCode% = WAITFORKEY%
    
	'skip the following if no mouse, for speed reasons
	IF NoMouse% GOTO SkipMouseSection
    
	Button% = 0 'Check for LEFT Button% Press
	CALL MMPRESSLOCATE(Button%, Times%, Row%, Col%)
	IF Times% THEN  'Got a LEFT Button% press
		'if click on top Row% exit
		IF Row% = WINDOWS(5).ULR THEN
			HorizMenuChange% = -3
			GOTO Ender
		END IF
	    
		'Check if Mouse is within range
		IF Row% = ULR% THEN
			SELECT CASE Col%
				CASE (LRC% - 1) TO LRC%  'Did we get up arrow key
					GOSUB MoveUp
				CASE ELSE
			END SELECT
		END IF
		IF Row% = LRR% THEN
			SELECT CASE Col%
				CASE (LRC% - 1) TO LRC%  'Did we get down arrow key
					GOSUB MoveDown
				CASE (ULC% + 3) TO (ULC% + 8)   'Did we get [QUIT]
					GOTO Ender
				CASE ELSE
			END SELECT
		END IF
		GOTO ReStart
	END IF

	IF MMRIGHTBUT% THEN   'see if right Button% is down
		CALL MMLOCATE(Row%, Col%)
		'Check if Mouse is within range
		IF Row% = ULR% THEN
			SELECT CASE Col%
				CASE (LRC% - 1) TO LRC%  'Did we get up arrow key
					GOSUB MoveUp
				CASE ELSE
			END SELECT
		END IF
		IF Row% = LRR% THEN
			SELECT CASE Col%
				CASE (LRC% - 1) TO LRC%  'Did we get down arrow key
					GOSUB MoveDown
				CASE ELSE
			END SELECT
		END IF
		GOTO ReStart
	END IF

SkipMouseSection:
	LOOP UNTIL KeyCode% <> -256

	SELECT CASE KeyCode%
		CASE -&H4B, -&HF  'Cursor Left, Shift Tab
			HorizMenuChange% = -1
			EXIT DO
		CASE -&H4D, 9     'Cursor Right, Tab
			HorizMenuChange% = -2
			EXIT DO
		CASE -&H48        'Cursor Up
			GOSUB MoveUp
		CASE -&H50        'Cursor Down
			GOSUB MoveDown
		CASE -&H51        'Page Down
			GOSUB PageDown
		CASE -&H49        'Page Up
			GOSUB PageUp
		CASE -&H47        'Home key
			GOSUB Home
		CASE -&H4F        'End key
			GOSUB EndHelp
		CASE 13, 27, 81, 113  'ENTER, ESC, "Q", "q" keys exit
			EXIT DO
		CASE ELSE
			'do nothing
	END SELECT
    
LOOP

Ender:             'Common Exit Section

	'restore the window
	CALL WINDREST(ULR%, ULC%, LRR%, LRC%, VARSEG(Scrn4%(0)), VARPTR(Scrn4%(0)))
	'erase the window so can redimension it again
	ERASE Scrn4%
	ERASE HelpMenus
	Inits% = False: OldToggle% = False  'reset for next time through
	'Clears keyboard and mouse for testing
	CALL CLRMOUSEKEY

EXIT SUB

'---------------------------------GOSUBS------------------------
MoveUp:   'Moves the Screen up
		'It is faster to print than scroll up especially on a CGA on a PC
		'more attractive too because many CGA's turn off the CGA to scroll
		'which leaves a dark line at the top.
    
	IF CurrentTop% <= 1 THEN GOSUB Home: RETURN
	CurrentTop% = CurrentTop% - 1
	CurrentBottom% = CurrentBottom% - 1
	GOSUB RePrintArray
	RETURN

MoveDown:  'Moves the Screen Down
	IF CurrentBottom% >= MenuItems% THEN GOSUB EndHelp: RETURN
	CurrentTop% = CurrentTop% + 1
	CurrentBottom% = CurrentBottom% + 1
	GOSUB RePrintArray
	RETURN
    
PageDown: 'Moves the screen down a page and reprints
	IF (CurrentBottom% + PageHelpLen%) >= MenuItems% THEN GOSUB EndHelp: RETURN
	CurrentTop% = CurrentTop% + PageHelpLen%
	CurrentBottom% = CurrentBottom% + PageHelpLen%
	GOSUB RePrintArray
	RETURN
   
PageUp:  'Moves the screen up a page and reprints
	IF (CurrentTop% - PageHelpLen%) <= 1 THEN GOSUB Home: RETURN
	CurrentTop% = CurrentTop% - PageHelpLen%
	CurrentBottom% = CurrentBottom% - PageHelpLen%
	GOSUB RePrintArray
	RETURN

Home:   'Moves the screen to top position
	IF CurrentTop% < 1 THEN
		CurrentTop% = 1
		CurrentBottom% = CurrentTop% + HelpLen%
		RETURN
	END IF
	CurrentTop% = 1
	CurrentBottom% = CurrentTop% + HelpLen%
	GOSUB RePrintArray
	RETURN

EndHelp:  'Moves the screen to end of array
	IF CurrentBottom% > MenuItems% THEN
		CurrentBottom% = MenuItems%
		CurrentTop% = CurrentBottom% - PageHelpLen%
		RETURN
	END IF
	CurrentBottom% = MenuItems%
	CurrentTop% = CurrentBottom% - PageHelpLen%
	GOSUB RePrintArray
	RETURN

RePrintArray:  'This does the actual printing
	'if no need to repaint then skip
	IF OldTop% = CurrentTop% THEN RETURN
    
	Z% = CurrentTop% - 1
	FOR X% = HelpTop% TO HelpBottom%
		SKIPNUM% = Z% * LenMenu%
		'This routine is used to prevent creation of temporary strings
		'in DGROUP because we grab string directly from far array
		CALL FPRT(X%, HelpLeft%, VARSEG(HelpMenus(1).MESSAGE), VARPTR(HelpMenus(1).MESSAGE), HelpMsgLen%, SKIPNUM%)
		Z% = Z% + 1
	NEXT
   
AdjArrow:
	'Adjusts the arrow
	IF CurrentBottom% = MenuItems% THEN
		CALL QPRINT(LRR%, LRC% - 1, TBot$, &H70)
	ELSE
		CALL QPRINT(LRR%, LRC% - 1, TDOWN$, &HF)
	END IF
	IF CurrentTop% = 1 THEN
		CALL QPRINT(ULR%, LRC% - 1, TTop$, &H70)
	ELSE
		CALL QPRINT(ULR%, LRC% - 1, TUP$, &HF)
	END IF
	OldTop% = CurrentTop%
    
	IF HaveScroll% THEN
		Toggle% = 2     'report not at bottom or top
		IF CurrentTop% = 1 THEN Toggle% = 1            'report at top
		IF CurrentBottom% = MenuItems% THEN Toggle% = 3 'report at bottom
		IF OldToggle% <> Toggle% THEN     'only if the position has changed
			SELECT CASE Toggle%         'repaint
				CASE 2  'neither
					CALL QPRT(MidLocation%, LRC%, MScrollOn$)
					CALL QPRT(LRR% - 1, LRC%, MScrollOff$)
					CALL QPRT(ULR% + 1, LRC%, MScrollOff$)
				CASE 1  'top
					CALL QPRT(ULR% + 1, LRC%, MScrollOn$)
					CALL QPRT(MidLocation%, LRC%, MScrollOff$)
					CALL QPRT(LRR% - 1, LRC%, MScrollOff$)
				CASE 3  'bottom
					CALL QPRT(LRR% - 1, LRC%, MScrollOn$)
					CALL QPRT(MidLocation%, LRC%, MScrollOff$)
					CALL QPRT(ULR% + 1, LRC%, MScrollOff$)
			END SELECT
		END IF
		OldToggle% = Toggle%             'keep track of last status
	END IF
	RETURN

END SUB

'=======================================================================
' This is simply a modular feature
' It loads up the HelpMenu array
'=======================================================================
SUB InitHelp (Msg%, Init%, HelpMenus() AS HelpType) STATIC

	STATIC X%

	CONST Sampleh$ = " This is a sample help menu."
	'This just fills display up with spaces
	CONST T11th$ = "                                            "
	
IF Init% = False THEN
	SELECT CASE Msg
		CASE 9
			HelpMenus(1).MESSAGE = "          PURPOSE OF DIALOG BOXES"
			HelpMenus(2).MESSAGE = T11th$
			HelpMenus(3).MESSAGE = "       LEFT Box is the Escape box.  It"
			HelpMenus(4).MESSAGE = "   allows user to cancel current action."
			HelpMenus(5).MESSAGE = T11th$
			HelpMenus(6).MESSAGE = "       RIGHT Box is the Confirm box."
			HelpMenus(7).MESSAGE = "   A click on this box will EXIT the"
			HelpMenus(8).MESSAGE = "   program."
			HelpMenus(9).MESSAGE = T11th$
			HelpMenus(10).MESSAGE = T11th$
			HelpMenus(11).MESSAGE = "     Copyright 1991, Sidney J. Kelly"
			HelpMenus(12).MESSAGE = "          All rights reserved."
		    
			FOR X% = 13 TO WINDOWS(6).MaxMenNum
				HelpMenus(X%).MESSAGE = T11th$
			NEXT
	    
		CASE 10
			HelpMenus(1).MESSAGE = "        VERTICAL MENUS WITH POPUPS, "
			HelpMenus(2).MESSAGE = "          HELP AND DIALOG BOXES."
			HelpMenus(3).MESSAGE = T11th$
			HelpMenus(4).MESSAGE = "    Click of Left Button selects."
			HelpMenus(5).MESSAGE = T11th$
			HelpMenus(6).MESSAGE = " Right Button Press or Drag Moves cursor."
			HelpMenus(7).MESSAGE = T11th$
			HelpMenus(8).MESSAGE = " A Right Button Release outside Menu area "
			HelpMenus(9).MESSAGE = " = ESC from current menu."
			HelpMenus(10).MESSAGE = T11th$
			HelpMenus(11).MESSAGE = T11th$
			HelpMenus(12).MESSAGE = "     Copyright 1991, Sidney J. Kelly"
			HelpMenus(13).MESSAGE = "          All rights reserved."
		    
			FOR X% = 14 TO WINDOWS(6).MaxMenNum
				HelpMenus(X%).MESSAGE = T11th$
			NEXT

		CASE ELSE
		    
			FOR X% = 1 TO WINDOWS(6).MaxMenNum
				HelpMenus(X%).MESSAGE = Sampleh$
			NEXT

			HelpMenus(9).MESSAGE = T11th$
			HelpMenus(10).MESSAGE = " This is Message number 10."
			HelpMenus(19).MESSAGE = T11th$
			HelpMenus(20).MESSAGE = " This is Message number 20."
			HelpMenus(29).MESSAGE = T11th$
			HelpMenus(30).MESSAGE = " This is Message number 30."
			HelpMenus(39).MESSAGE = T11th$
			HelpMenus(40).MESSAGE = " This is Message number 40."
			HelpMenus(49).MESSAGE = T11th$
			HelpMenus(50).MESSAGE = " This is Message number 50."
			HelpMenus(59).MESSAGE = T11th$
			HelpMenus(60).MESSAGE = " This is Message number 60."
			HelpMenus(69).MESSAGE = T11th$
			HelpMenus(70).MESSAGE = " This is Message number 70."
			HelpMenus(79).MESSAGE = T11th$
			HelpMenus(80).MESSAGE = " This is Message number 80."
			HelpMenus(89).MESSAGE = T11th$
			HelpMenus(90).MESSAGE = " This is Message number 90."
			HelpMenus(99).MESSAGE = T11th$
			HelpMenus(100).MESSAGE = " This is Message number 100."
			HelpMenus(109).MESSAGE = T11th$
			HelpMenus(110).MESSAGE = " This is Message number 110."
			HelpMenus(119).MESSAGE = T11th$
			HelpMenus(120).MESSAGE = " This is Message number 120."
		END SELECT
END IF
Init% = True

END SUB

'======================================================================
'This sets up the window size based on the type of help desired
'======================================================================
SUB SetHelpSize (Msg%) STATIC
	SELECT CASE Msg
		CASE 9, 10
			'This makes the DIALOG Exit Window help
			'and HORIZONTAL Menu Help
			WINDOWS(6).ULR = 5
			WINDOWS(6).ULC = 17
			WINDOWS(6).LRR = WINDOWS(6).ULR + 16
			WINDOWS(6).LRC = WINDOWS(6).ULC + HelpMsgLen% + 1
			WINDOWS(6).MaxMenNum = 15
		CASE ELSE
			'This makes the Main Window help
			WINDOWS(6).ULR = 5
			WINDOWS(6).ULC = 17
			WINDOWS(6).LRR = WINDOWS(6).ULR + 16
			WINDOWS(6).LRC = WINDOWS(6).ULC + HelpMsgLen% + 1
			WINDOWS(6).MaxMenNum = HelpEntries%
	END SELECT
END SUB

