/* hash.h -	include file for the HASH table ADT.
 *		This is the open hashing implementation.
 *		Objects stored in the table are assumed to be
 *		strings.
 */

#ifndef VHASH_H
#define VHASH_H

#include "stypes.h"
#include "vlist.h"


typedef CHAR *HASHKEY;		/* keys are pointers to char */

struct vhash_head
   {
   USHORT size;			/* # of buckets in table */
   USHORT count;		/* number of items in table */
   VM7HDL htable;		/* ptr to array of lists */
   VM7HDL *tablep;		/*  pointer to array when hash table
				|   is locked  */
   };
typedef struct vhash_head *VHASHPV;


void FreeVHash(VM7HDL);			/* free hash table resources */

int VHashAdd(VHASHPV, VM7HDL, HASHKEY);	/* add entry to hash table */
					/* apply a function to table items */
USHORT VHashApply(VHASHPV, int (*)());
					/* delete entry from hash table */
BOOL VHashDel(VHASHPV, int (*)(), HASHKEY);
					/* lookup an entry in table */
VM7HDL VHashLook(VHASHPV, int (*)(), HASHKEY);
USHORT VHashSize(VHASHPV);
VM7HDL MakeVHash(USHORT, double);	/* make a new hash table */
VHASHPV VHashLock(VM7HDL);
VOID VHashUnlock(VM7HDL, VHASHPV); 


#define OB_VHash_Make		MakeVHash
#define OB_VHash_Lock(a)	(VHASHPV)VHashLock(a)
#define OB_VHash_Unlock(a,b)	VHashUnlock((a), (b))
#define OB_VHash_Free(a)	FreeVHash(a)
#define OB_VHash_Add(a,b,c)	VHashAdd((a), (b), (c))
#define OB_VHash_Del(a,b,c)	VHashDel((a), (int (*)())(b), (c))
#define OB_VHash_Apply(a,b,c)   VHashApply((a), (int (*)())(b))
#define OB_VHash_Look(a,b,c)	VHashLook((a), (int (*)())(b), (c))
#define OB_VHash_Get_Size(a)	(((VHASHPV)(a))->size)
#endif
