;created
;Updated 12/13/91

;-----------------------------------------------------------------------
;   Copyright (C) Copr. 1990, 1991 by Sidney J. Kelly
;                   All Rights Reserved.
;           Sidney J. Kelly
;           150 Woodhaven Drive
;           Pittsburgh, PA 15228
;           home phone 412-561-0950 (7pm to 9:30pm EST)
;-----------------------------------------------------------------------
;RM82TS8,16,24,32,40

;=======================================================================
; DECLARE FUNCTION VIDELSTAT% ()
;
; VideoStatus Analyzer
; Program returns video information for use by QBASIC
; returns highest level of SCREEN mode supported by the Hardware.
; Returns information about the PRIMARY display only
; This routine with a SELECT CASE can tell you how many display pages
; are available in text mode, SCREEN 0: WIDTH 80,25
; VGA has 8 pages
; EGA with > 64kb has 8 pages
; EGA with 64kb has 4 pages
; CGA has 4 pages
; MONO has 1 pages
; HERC has 1 page in SCREEN 0, 2 pages in SCREEN 3
;
; Exceptions:
; MCGA color returns a mode 11, so can contrast w/ VGA
; HERC w/o MSHERC.COM reports a -3 so you can tell user to correct matters
; COMPAQ with CGA reports -2 so can force use of monchrome attributes.
; EGA's with CGA diplays report -8 if have only 64kb of memory
; EGA's with ECD display report  8 if have only 64kb of memory
; Mono MCGA/VGA reports -10 so can contrast with Mono EGA's
; Mono VGA's that may really be color VGAs report -13
;
; Returns:
; Mono         =   0
; CGA          =   2
; PGA          =   2   (Untested, assume PGA would be treated as CGA)
; COMPAQ - CGA =  -2   to tell that user may have a single color display
; HERC         =   3   if have HERC and MSHERC.COM
; HERC         =  -3   if have HERC but w/o MSHERC.COM
; AT&T         =   4,  but only if have just CGA display, not EGA or VGA
;     Microsoft QuickC/C makes Olivetti EGA and VGA return a separate code.
;     The only difference I could find was that the early AT&T/Olivetti EGA
;     adapter did not permit WORD OUT's to the graphics control registers.
;     Abrash, "Power Graphics Programming", (Que 1989) page 73
;     Are there other differences???
; EGA64        =   8   Because resolution limited for 64k EGA
; EGA64 w/CGA  =  -8   Because resolution limited to CGA modes
; EGA          =   9   Have minimum of 128kb of video ram
; EGA w/CGA    =  -9   Since resolution limited to CGA modes
;		       This has been tested, QBASIC will refuse to use
;		       hirez EGA screen modes if CRT is CGA display.
; EGA Mono     =  10   If have EGA with mono display
; MCGA w/EGA   = -12   If have an EGA ECD digital color display, since
;                      resolution of this display is limited to EGA modes
; MCGA Mono    = -11   If have MCGA with a mono display
; MCGA         =  11   If have an analog color display
; VGA Mono     = -10   To distinguish between EGA mono
; VGA          =  13   Normal VGA color display
; VGA          = -13   Monochrome monitor really appears to be color
;
;   Note, some VGA's with color displays will report that they are monochrome
;   if the CRT is not turned on before the Computer is turned on or if 8514/A
;   is active.  Source: Kliewer, "EGA/VGA: A Programmer's Reference Guide" 
;   (2d ED, Intertext/McGraw Hill 1990) page 250
;
; Version 0.96
;
; Tested:     CGA (Tecmar), HERC clone, EGA mono, EGA color, VGA color
;             COMPAQ CGA (Portable & 286/AT), VGA clone (CARDINAL, Paradise),
;             EGA clone (Paradise clone, ITT clone, ATI EGA Wonder),
;             EGA w/ CGA CRT.
; Not tested: PGA, VGA monochrome, MCGA (color or monochrome), AT&T 6300
;=======================================================================

DOSSEG
.MODEL MEDIUM

	PUBLIC VIDEOSTAT, HERCTYPE

.CODE

HERC_PORT       Equ     03BAh       ; Herc display status port
COMPAQ          DB     'COMPAQ'     ; BIOS ID string in COMPAQ's
COMPAQ_LEN      Equ     6           ; length of string

;Following is commented out because alternative method is faster
;ATT             DB     'OLIVETTI'   ; BIOS ID string for OLIVETTI/AT&T
;ATT_LEN         Equ     8           ; length of string

; Please do not remove
Copyright       DB    13,10,'Copyright Copr. (C) 1990 Sidney J. Kelly',13,10
Copyright1      DB    'All Rights Reserved',13,10,26

EVEN
VIDEOSTAT   PROC far
Begin:
     Push    BP                 ; save BP because old PC Video BIOS
                                ; didn't necessarily preserve this
     Push    DI                 ; QBASIC requires these 2 to be saved
     Push    SI
vgatest:                        ; this just tests for primary display
     Mov     AX,1A00h		; because that is all QBASIC cares about
     Mov     BX,10h             ; Test for VGA adapter
     Int     10h
     Cmp     AL,1Ah             ; If VGA adapter exists, it returns AL=1Ah
     Jne     egatest            ; VGA does not exist so test for EGA
				; Else VGA exists, primary display info
     Cmp     BL,8               ; routine returns primary display in BL
     Jne     @F
VGA_Color:
     Mov     AX,13		; VGA color exists. This does not test
     Jmp     Quit		;  if VGA adapter clone is in emulation
                                ;  mode, such as HERC emulation
                                ;  because QBASIC does not check
                                ;  for this either.
@@:
     Cmp     BL,7		; this may be a disguised color
     Jne     @f			; else, test other adapter
vga_unk:
     Xor     AX,AX		; some VGA w/ color monitor will report
     Mov     ES,AX		; mono if CRT is not on when computer started
     Mov     BX,ES:[0463h]      ; check if MONO port in use
     Mov     AX,-10             ; assume VGA mono
     Cmp     BL,0B4h
     Je	     vga_unk1 	        ; report VGA mono
     Mov     AX,-13		; else report may be a color CRT
vga_unk1:
     Jmp     Quit 	        ; Kliewer, "EGA/VGA: A Programmer's Reference
				; Guide" (2d ED, Intertext/McGraw Hill 
				; 1990) page 250
@@:
     Cmp     BL,11
     Jne     @f
MCGA_Mono:
     Mov     AX,-11             ; MCGA mono exists
     Jmp     Quit
@@:
     Cmp     BL,12
     Jne     @f			; test if MCGA analog color exists
MCGA_Color:
     Mov     AX,11              ; near max screen mode is QBASIC SCREEN 11
     Jmp     Quit               ; MCGA can do SCREEN 13 too, but not 12.
@@:
     Cmp     BL,10
     Jne     @f
MCGA_EGA:
     Mov     AX,-12		; MCGA w/EGA display (untested)
     Jmp     Quit
@@:
     Cmp     BL,4               ; EGA color screen found, we have to
     Jne     @f                 ; test if EGA is attached to ECD
     Jmp     Short egatest      ; monitor or just plain CGA monitor
@@:
     Cmp     BL,5
     Jne     @f
EGA_Mono:
     Mov     AX,10
     Jmp     Quit               ; report have EGA mono
@@:
     Cmp     BL,2
     Jne     @F
     Jmp     short att_test     ; CGA color found, test for which type
@@:
     Cmp     BL,1
     Jne     @f
     Jmp     herctest           ; MDA mono found
@@:
     Cmp     BL,6               ; test for PGA, treat as a CGA
     Jne     @f
     Jmp     Short cga          ; I guess this should work, untested
@@:                             ; With everything else
     Jmp     unknown            ; merely assume can do normal b&w text
egatest:
     Mov     AX,1200h
     Mov     BX,10h
     Int     10h
     Cmp     BL,10h             ; EGA changes BL from 10h
     Je      cgatest
     Mov     DX,BX              ; Save BX temporarily in DX
     Xor     BX,BX
     Mov     ES,BX
     Test    ES:Byte Ptr [487h],1000b  ;EGA active?, bit 3 of 0040:0087 set?
     JZ      EGA_active         ; if bit clear, then EGA is active
     Jmp     Short cgatest      ; else if EGA not active (primary) check CGA
EGA_active:
     Mov     BX,DX              ; get former value back from CX
     Or      BH,BH              ; if BH = 1 then have MONO display
     JZ      ega_color          ; else have color display
ega_mono:			; max screen mode is QBASIC SCREEN 10
     Mov     AX,10
     Jmp     Short Quit         ; report have mono
ega_color:
;********* Based on information returned by AH 12h, BL 10h of INT 10h,
;********* this routine reads the EGA Sense switches stored in CL
;********* to see if really have a primary monitor that can do EGA graphics.
;********* If do not have an ECD/multisync display, report a -9.
;********* If have a 64kb EGA reports 8 if have ECD or -8 if have CGA
;********* It is unclear if this will work on all clone EGA's.
;
;********* It works with Paradise VGA clone, Cirrus VGA clone, BOCA VGA clone,
;********* ATI EGA Wonder, ACER (Paradise) EGA clone, and an ITT EGA clone
;********* CL information derived from RAM memory at 0040:0088h
;
;********* Source of CL information:
;********* Ferro, "Programmer's Guide to the EGA and VGA Cards,"
;********* (Addison-Wesley 1988), page 337.  Yet on page 473 says you can't
;********* tell what CL will report.
;
;********* Kliewer, "EGA/VGA: A Programmer's Reference Guide" (2d Ed,
;********* Intertext/McGraw Hill 1990), page 66-67
;
;********* Waite Group's, "MS-DOS Developer's Guide", (2d Ed. Howard Sams
;********* 1989), page 496 (shown as part of a .C program)
;
;********* Similar information hidden in Wilton, "Programmer's Guide to
;********* PC & PS/2 Video Systems" (Microsoft 1987), page 520.
;
;********* Partial information set forth in Phoenix's "System Bios for
;********* PC/XT/AT Computers and Compatibles (Addison-Wesley 1989)
;********* page 243 (which states that VGA's follow this too)
;
;********* Wilton's routine also appears in other books, e.g.
;********* Tischer, "PC System Programming for Developers" (ABACUS 1989)
;********* page 542
;
;********* See also, Woram, "PC Configuration Book" (Bantam 1987),
;*********   pages 279-280 Provides listing of IBM official switch
;*********   settings which does not seem to match anything
;*********   in the above two books.
;
;*********   CL returns on good clones:
;*********     EGA inactive w/ MDA primary   =  00h, 01h, 02h, 03h
;*********     EGA inactive w/ CGA primary   =  04h, 05h
;*********     EGA active w/ CGA             =  06h, 07h
;*********     EGA active w/ ECD in CGA mode =  08h
;*********     EGA active w/ ECD             =  09h
;*********     EGA active w/ MDA mono        =  0Bh, 0Ah
     Cmp     CL,9               ; read EGA Sense switches to see if
				; have ECD display and not a CGA display
     JE      ega1
     Mov     AX,-9              ; report have a CGA type display
     Or      BL,BL              ; check if BL=0, i.e. 64kb Memory
     JNZ     @f                 ; nope so quit
     Inc     AX                 ; report have a CGA display w/ 64kb Memory
@@:
     Jmp     Short Quit         ; with an EGA adapter
ega1:
     Mov     AX,9               ; max screen mode is QBASIC SCREEN 9
     Or      BL,BL              ; check if BL = 0, i.e. 64kb memory
     JNZ     @f                 ; nope, so quit
     Dec     AX                 ; report have EGA w/ ECD display
                                ; but only 64KB memory (makes 9 an 8)
@@:
     Jmp     Short Quit
cgatest:
     Xor     AX,AX
     Mov     ES,AX
     Mov     BX,ES:[0463h]      ; check if MONO port in use
     Cmp     BL,0B4h
     Je      herctest           ; this test looks for Mono port address
att_test:
;******* The AT&T CGA is just a CGA that responds to a special mode set
;******* command.  Mode 48h is equivalent to SCREEN 4.  It is possible
;******* to load BIOS ram with this screen mode and fool QBASIC.  Try it and
;******* you will get some weird displays on non-AT&T displays.
;******* Unknown if that "gamemanship" may hurt your monitor.
;
;******* Cannot merely attempt to change to mode &H48 to test for AT&T
;******* because some old CGA's (e.g. Tecmar Color Graphics Card) will let you
;******* change mode number to anything you try.  However HERC clones,
;******* EGA clones and VGA clones will only allow a change to a supported
;******* video mode so such a test would work with them, but would be
;******* worthless unless you could be confident that all CGA's were new.

     Mov     AX,0F000h          ; Get ROM ID segment
     Mov     ES,AX              ; Store segment in ES
     CLD                        ; clear the direction flag
     Push    DS                 ; Save DS
     Mov     AX,CS
     Mov     DS,AX              ; set DS = to CS
     Assume  DS:@code

     comment		|*
            ; this is the long form routine, using full name
     Mov     DI,0C050h          ; point DI at ROM
     Mov     SI,OFFSET ATT      ; DS:SI points to  "OLIVETTI".
     Mov     CX,ATT_LEN         ; CX has length of "OLIVETTI".
     REPE    CMPSB              ; ES:DI destination, DS:SI source
     Jne     compaq_test        ; if no change test for COMPAQ
     			|*

     				; faster version
     				; Source:  XMS.SYS version 2.06 MASM source
     Cmp     ES:[0C05h],'LO'    ; look for byte reversed 'OL' at FC00:0050h
     				; in memory letters are reversed
     Jne     compaq_test	; AT&T not found

att_found:
     ; program assumes that if a CGA is active on an AT&T
     ; that built-in adapter is in use.
     Pop     DS                 ; restore DS
     Assume  DS:@data
     Mov     AX,4               ; max screen mode is QBASIC SCREEN 4
     Jmp     Short Quit

compaq_test:
     ; Purpose of this test is to allow other routines to know if
     ; "snow" testing is necessary, as well as to tell programmer
     ; to use mono attributes in program rather than color attributes
     ; which would confuse the viewer of a COMPAQ single color display.
     ; source: XMS.SYS Version 2.06 MASM Source Code
     Mov     DI,0FFEAh          ; Get ID offset & store in DI
     Mov     SI,OFFSET COMPAQ   ; DS:SI points to  "COMPAQ".
     Mov     CX,COMPAQ_LEN      ; CX has length of "COMPAQ".
     REPE    CMPSB              ; ES:DI destination, DS:SI source
     Pop     DS                 ; restore DS
     Assume  DS:@data
     ; Program assumes that if a CGA is active on a COMPAQ
     ; that built-in single color display is in use because
     ; even though it does not have color it has much better
     ; text resoluton.  Tested on both Compaq and Compaq 286 portable
     Mov     AX,-2              ; assume found COMPAQ CGA mode
				; to tell us that user likely has a
				; single color display
     JE      found_compaq       ; if match found
cga:                            ; Normal CGA
     Mov     AX,2               ; max screen mode is QBASIC SCREEN 2
found_compaq:
     Jmp     Short Quit
herctest:
     ;*****Main Herc test entry point****
     ; Herc display changes Vertical retrace bit during vertical retrace
     ; however normal MDA display does not.
     ; Source for HERC test only:
     ; Hercules Computer Technology, Inc. and Doty,
     ; "Programmer's Guide to Hercules Graphics Cards"
     ; (Addison-Wesley 1988) page 138.
     ; Similar routines have been  published in PC Magazine
     ; e.g.   V6,N21,P393 (Dec. 8, 1987)
     ; and Young, "Inside DOS: A Programmer's Guide", (Sybex 1990)
     ; page 178, as well as Young's book on OS/2.
     ;
     ; No effort is made to test for Herc INCOLOR or any other HERC
     ; displays since QBASIC would not use the information.
     ; If you must have it, use HERCTYPE, below

     ; An alternative method exists.  If BIOS ram reports that current video
     ; is 7 (Mode 8 if MSHERC in SCREEN 3), and page size is 16,384
     ; chances are very good that you have a Herc display.
                                ; test the HERC_Port
     Mov     DX,HERC_PORT       ; load DX with display status port
                                ; a Read-only port
     Xor     BX,BX              ; clear counter
     IN      AL,DX              ; read port
     And     AL,80h             ; mask off every bit but bit 7
     Mov     AH,AL              ; store in AH original status of bit
     Mov     CX,8000h           ; load loop counter (it may need to be
				; a larger loop counter on a 50 Mhz 486)
herc1:
     IN      AL,DX              ; read byte
     And     AL,80h             ; clear everything but bit 7
     Cmp     AL,AH              ; keep testing for change in bit 7
     Je      herc2
     Inc     BL                 ; for every change Inc BL
     Cmp     BL,10              ; check routine found vertical
				; retrace changed 10 times
     JA      Test_4_MSHERC      ; found a herc so test for MSHERC.COM
herc2:
     Loop    herc1              ; Herc display not found
unknown:                        ; Handles mono displays too
     Xor     AX,AX              ; return a 0 if just normal text display

EVEN                            ; speed up access on 8086/80286/80386
Quit:
     Pop     SI                 ; restore saved registers
     Pop     DI
     Pop     BP
     Ret

Test_4_MSHERC:
     Mov     DL,0FFh            ; Make sure DL = 0FFh so we can test if changed
     Mov     AH,0EFh            ; test for MSHERC.COM Version 1.10
     Int     10h                ; works w/ MSHERC.COM Version 1.11 too
     Mov     AX,3               ; Assume max screen mode is QBASIC SCREEN 3
     Cmp     DL,0FFh            ; ***this is undocumented and may change!!
     JE      No_MSHERC		; no change in DL, so no MSHERC
     Or      DL,DL		; if no change ******!!!!!
     JZ      Have_MSHerc        ; if DL=00 then MSHERC.COM is installed
     Cmp     DL,01h             ; see if have HGC Plus or Incolor
     JE      Have_MSHerc
     Cmp     DL,02h             ; see if have HGC Plus or Incolor
     JE      Have_MSHerc
No_MSHERC:
    Mov     AX,-3               ; else report MSHERC.COM is not installed
				; & return -3
Have_MSHerc:
     Jmp     Short Quit
VIDEOSTAT       ENDP

;======================================================================
; DECLARE FUNCTION HERCTYPE% ()
; t% = HERCTYPE%
; Returns:
;        0  if no Hercules display
;        1  if normal Hercules or Hercules clone
;        2  if Graphics Plus
;        3  if Incolor
;Doty,  "Programmer's Guide to Hercules Graphics Cards"
; (Addison-Wesley 1988) page 138, 166, 209.
;
; Updated 6/5/91
;======================================================================

EVEN
HERCTYPE  PROC  FAR
	Push    BP              ; save BP for safety
;eliminate mono EGA's and mono VGA's
	Mov     AX,1200h        ; see if we have an EGA/VGA
	Mov     BX,10h
	Int     10h
	Cmp     BL,10h          ; EGA changes BL from 10h (VGA too)
	Je      Test_4_Herc     ; nope, no VGA/EGA
	Xor     AX,AX           ; yes, see if EGA active
	Mov     ES,BX
	Test    ES:Byte Ptr [487h],1000b  ;EGA active?, bit 3 of 0040:0087 set?
	JZ      No_Herc_Here    ; if bit clear, then EGA is active, any Herc
				; would be inactive, so ignore
Test_4_Herc:
	Mov     DX,03BAh        ; check vertical retrace in display stat port
	IN      AL,DX
	AND     AL,80h          ; clear everything but bit 7
	Mov     AH,AL           ; store value in AH
	Mov     CX,8000h        ; loop 32k times if necessary
;this test makes sure we have some type of HERC
Wait_4_Retrace:
	IN      AL,DX           ; read current value
	AND     AL,80h          ; clear everything but bit 7
	CMP     AH,AL
	LOOPE   Wait_4_Retrace  ; loop while CX <> 0 & no change
	Mov     AX,0            ; assume HERC not found (must use MOV)
	JZ      No_Herc_Here    ; any change because of retrace?
	Mov     AL,01           ; select HALF mode (necessary for advanced
				; features & allows coexistence w/ color CRT)
	Mov     DX,03BFh        ; adjust HERC configuration port
	OUT     DX,AL
	Mov     DX,03BAh        ; read Display Status port again
	IN      AL,DX           ; store in AL, AH already 0
	AND     AX,70h          ; mask off bits 6-4 & clear 0-3, 1110000b
; must test for each type because some EGA/VGA Herc. emulators
; don't set all the same bits a real Herc/Herc clone would set,
; e.g. my ATI EGA Wonder sets bit 6 & clears bits 5,4
	Mov	BX,3		; assume Incolor
	Cmp	AX,1010000b	; x101xxxx is the Incolor ID
	Je	H_Adjust	; yep, it is Incolor
	And	AX,110000b	; clear bit 6
	Mov	BX,2		; assume it is a Graphics Plus
	Cmp     AX,0010000b     ; xx01xxxx is the Graphics Plus ID
	Je	H_Adjust	; yep it is a Graphics Plus
	Mov	BX,1		; else, it must be a normal clone or
				; very early Herc
H_Adjust:
	Mov	AX,BX		; get the assumed value in AX
No_Herc_Here:
	Pop     BP              ; restore saved register, value in AX
	Ret
HERCTYPE  ENDP
END

