/* list.h -     include file for the VLIST ADT.
 *              This is the dynamic array implementation. 
 */

#ifndef VLIST_H
#define VLIST_H

#include "vm7.h"

struct vlist_head
   {
   USHORT current;              /* number of current item */
   USHORT count;                /* # of items in the list */
   USHORT list_size;            /* size of allocated memory  */
   UBYTE  alloc_size;           /* number of items to allocate initially  */
   UBYTE  bump_size;            /* number of items to add in reallocation  */
   VM7HDL hfirst;               /* handle to array of handles  */ 
   VM7HDL *afirst;              /* pointer that is locked dynamically  */
   };
typedef struct vlist_head * VLISTPV;



VLISTPV LockVList(VM7HDL);
void UnlockVList(VM7HDL, VLISTPV);
void FreeVList(VM7HDL);         /* deletes and frees an entire list */
                                /* applies a function to list items */
int VListApply(int (*)(), VLISTPV);
int VListDelete(VLISTPV);               /* delete current item from list */
int VListFirst(VLISTPV);                /* make first item the current item */
VM7HDL VListGet(VLISTPV);               /* get current item from a list */
int VListInsert(VM7HDL, VLISTPV);       /* insert item before current in a list */
int VListLast(VLISTPV);         /* make the last item the current item */
void VListLoad(VM7HDL *, SHORT, VLISTPV);
int VListNext(VLISTPV);         /* position to next item in list */
int VListPos(USHORT, VLISTPV);          /* position to ordinal item number */
int VListPrev(VLISTPV);         /* position back to previous item in list */
int VListSize(VLISTPV);         /* get # of items in a list */
void VListTop(VLISTPV);         /* set current BEFORE first item in list */
VM7HDL MakeVList(UBYTE alloc_size, UBYTE bump_size);
                                /* make a new list */
VM7HDL VListAssign(CHAR *array, SHORT size, UBYTE bump_size);
                                /* assign array to new list  */
void VListSetup(VLISTPV lid, SHORT count, UBYTE alloc_size, UBYTE bump_size);
ULONG SearchVList(VOIDPV , int (*)(), VLISTPV);
                                /* find a particular item in list */
void SortVList(int (*compare)(), VLISTPV lid);
                                /* sort a list */
#ifndef NULL_ITEM
#define NULL_ITEM 0xffffffff
#endif

#define OB_VList_Lock(a)                (VLISTPV)(LockVList(a))
#define OB_VList_Unlock(a,b)    (VOID)(UnlockVList((a),(b)))
#define OB_VList_Top(a)         (a->current = NULL_ITEM)
#define OB_VList_Next(a)                (int)(VListNext(a))
#define OB_VList_Prev(a)                (int)(VListPrev(a))
#define OB_VList_Pos(a,b)       (int)(VListPos((b), (a)))
#define OB_VList_Get_Item(a)     (VM7HDL)(VListGet(a))
#define OB_VList_Get_Current(a)  (int)(a->current)
#define OB_VList_Get_Count(a)    (int)(a->count)
#define OB_VList_Insert(a,b)     (int)(VListInsert((b),(a)))
#define OB_VList_Delete(a)      (int)(VListDelete(a))
#define OB_VList_New(a,b)       (VM7HDL)(MakeVList((UBYTE)(a),(UBYTE)(b)))
#define OB_VList_Sort(a,b)      SortVList((int (*)())(b),(a))
#define OB_VList_Search(a,b,c)  (int)SearchVList((c),(int (*)())(b),(a))
#define OB_VList_Apply(a,b)     VListApply((int (*)())(b),(a))
#define OB_VList_Last(a)                VListLast(a)
#define OB_VList_First(a)       VListFirst(a)
#define OB_VList_Free(a)                FreeVList(a)

#endif
