;WINDSAVE.ASM - saves a portion of the text screen in QuickBASIC
;call from Quickbasic:
;call   windsave(fline,fcol,lline,lcol,seg storage(0))
;where
;     fline       =     first line of window
;     fcol        =     first column of window
;     lline       =     last line of window
;     lcol        =     last column of window
;     storage()   =     storage array for screen contents
;
;     all are type integer

dgroup      group _Data
_Data       Segment  word public 'Data'

row         dw    0           ;current row
wrdcpy      dw    0           ;words to copy per line
stcol       dw    0           ;Starting Column
endrow      dw    0           ;Ending Row

_Data       ends

Cseg_Text        Segment Word Public 'Code'
            Assume  CS:Cseg_Text,DS:dgroup,SS:dgroup
            Public  WindSave

WindSave    Proc    Far

            Push  BP            ;save registers for BASIC
            Push  DS
            Mov   BP,SP         ;locate stack to get variable addresses later

            Mov   DX,0          ;look at low memory using ES
            Mov   ES,DX
            Mov   BX,0B000h     ;assume mono screen segment for now
            Mov   AL,ES:[410h]  ;get the equipment list
            And   AL,48         ;just look at the monitor type
            Cmp   AL,48         ;is it mono?
            JZ    Get_Params    ;yes, skip over adding 800h
            Add   BX,800h       ;no, adjust for color screen memory

            Mov   AL,ES:[487h]  ;if an EGA is present, AL will not be zero
            Cmp   AL,0          ;is it an EGA?
            JNZ   Get_Params    ;yes, leave DX set to zero as a flag for later
            Mov   DX,3DAh       ;no, specify the port to check for retrace

Get_Params: Push  BX            ;save the screen segment for later
            Mov   SI,[BP+08]    ;get the starting address of the storage array
            Mov   DI,SI         ;and put it into DI
            Mov   SI,[BP+10]    ;get the segment for the array
            Mov   ES,SI         ;and assign it to ES

            mov   bx,[bp+16]        ;get first column
            mov   ax,[bx]
            mov   stcol,ax

            mov   bx,[bp+12]        ;get last column
            mov   ax,[bx]

            sub   ax,stcol          ;subtract first column from first column
            inc   ax                ;add 1 to get number of columns to print
            mov   wrdcpy,ax         ;ax=# of words to copy per line

            mov   bx,[bp+14]        ;get last line
            mov   ax,[bx]
            mov   endrow,ax

            mov   bx,[bp+18]        ;get first line
            mov   ax,[bx]          ;set current row to first line
            mov   row,ax

            call  getoff            ;calculate offset for line

            mov   cx,wrdcpy         ;load counter with words to copy

            Pop   DS            ;retrieve the screen segment saved earlier
 
ShowNext:
            Cld                 ;all data moves below will be forward
            Cmp   DL,0          ;are we doing monochrome or EGA?
            JZ    Mono          ;yes, skip over the retrace stuff

No_Retrace: In    AL,DX         ;get the video status byte
            Test  AL,1          ;test just the horizontal retrace bit
            JNZ   No_Retrace    ;if doing a retrace, wait until it's not
Retrace:    In    AL,DX         ;get the status byte again
            Test  AL,1          ;are we currently doing a retrace?
            JZ    Retrace       ;no wait until we are
            Lodsw               ;now get the word from the screen
            Stosw               ;and put it into the array
            Loop  No_Retrace    ;loop until done
            Jmp   NextLine      ;skip over the mono routine and exit

Mono:       Rep   Movsw         ;move the data in one operation

NextLine:   push  DS                ;save video segment
            mov   ax,[bp]           ;get old data segment to address vars.
            mov   DS,ax             ;set data segment
            mov   ax,endrow
            cmp   ax,row            ;Are we done restoring window?
            je    Exit              ;exit if complete
            inc   row               ;next line
            call  getoff            ;compute new offset address
            mov   cx,wrdcpy         ;set counter to number of words to copy
            pop   DS                ;reset to video segment
            jmp   ShowNext          ;jump to print next line


Exit:       Pop   DS               ;pop video segment off stack
            Pop   DS            ;restore registers for BASIC
            Pop   BP
            Ret   12            ;return skipping the passed parameters

WindSave    Endp

getoff      proc  near
            ;calculate offset of start of line
            ;line is contained in row, column is contained in variable stcol

            mov   ax,row
            dec   ax          ;change from 1-25 to 0-24
            mov   cl,160
            mul   cl          ;multiply row by 160
            mov   bx,stcol    ;get starting column
            dec   bx          ;change from 1-80 to 0-79
            shl   bx,1        ;multiply (row-1) by 2
            add   ax,bx       ;get offset
            mov   si,ax       ;move to source index

            ret
getoff      endp

Cseg_Text        Ends
            End

